@@ -6,7 +6,6 @@
  *
  * released under GNU GPL v2 licence
  *
- * $Id: AddrAddr.cpp,v 1.13 2008-08-17 22:41:41 thomson Exp $
  */
 
 #include <iostream>
@@ -17,6 +16,8 @@
 #include "DHCPConst.h"
 #include "Logger.h"
 
+using namespace std;
+
 /**
  * @brief AddrAddr constructor for addresses
  *
@@ -31,8 +32,8 @@ TAddrAddr::TAddrAddr(SPtr<TIPv6Addr> addr, long pref, long valid) {
     this->Prefered = pref;
     this->Valid = valid;
     this->Addr=addr;
-    this->Timestamp = now();
-    this->Tentative = TENTATIVE_UNKNOWN;
+    this->Timestamp = (unsigned long)time(NULL);
+    this->Tentative = ADDRSTATUS_UNKNOWN;
     this->Prefix = 128;
 
     if (pref>valid) {
@@ -57,8 +58,8 @@ TAddrAddr::TAddrAddr(SPtr<TIPv6Addr> addr, long pref, long valid, int prefix) {
     this->Prefered = pref;
     this->Valid = valid;
     this->Addr=addr;
-    this->Timestamp = now();
-    this->Tentative = TENTATIVE_UNKNOWN;
+    this->Timestamp = (unsigned long)time(NULL);
+    this->Tentative = ADDRSTATUS_UNKNOWN;
     this->Prefix = prefix;
 
     if (pref>valid) {
@@ -95,7 +96,7 @@ SPtr<TIPv6Addr> TAddrAddr::get() {
 unsigned long TAddrAddr::getPrefTimeout()
 {
     unsigned long ts = Timestamp + Prefered;
-    unsigned long x  = now();
+    unsigned long x  = (unsigned long)time(NULL);
     if (ts<Timestamp) { // (Timestamp + T1 overflowed (unsigned long) maximum value
         return DHCPV6_INFINITY;
     }
@@ -115,7 +116,7 @@ unsigned long TAddrAddr::getPrefTimeout()
 unsigned long TAddrAddr::getValidTimeout()
 {
     unsigned long ts = Timestamp + Valid;
-    unsigned long x  = now();
+    unsigned long x  = (unsigned long)time(NULL);
     if (ts<Timestamp) { // (Timestamp + T1 overflowed (unsigned long) maximum value
 	return DHCPV6_INFINITY;
     }
@@ -138,7 +139,7 @@ void TAddrAddr::setTimestamp(long ts)
     this->Timestamp = ts;
 }
 
-void TAddrAddr::setTentative(enum ETentative state)
+void TAddrAddr::setTentative(enum EAddrStatus state)
 {
     this->Tentative = state;
 }
@@ -156,10 +157,10 @@ void TAddrAddr::setValid(unsigned long valid)
 // set timestamp
 void TAddrAddr::setTimestamp()
 {
-    this->Timestamp = now();
+    this->Timestamp = (unsigned long)time(NULL);
 }
 
-enum ETentative TAddrAddr::getTentative()
+enum EAddrStatus TAddrAddr::getTentative()
 {
     return Tentative;
 }
