@@ -5,8 +5,6 @@
  *          Marek Senderski  <msend@o2.pl>
  *
  * released under GNU GPL v2 only licence
- * $Id: AddrAddr.h,v 1.9 2008-08-29 00:07:26 thomson Exp $
- *
  */
 
 #ifndef ADDRADDR_H
@@ -18,11 +16,9 @@
 #include "SmartPtr.h"
 #include "DHCPConst.h"
 
-using namespace std;
-
 class TAddrAddr
 {
-    friend ostream & operator<<(ostream & strum,TAddrAddr &x);
+    friend std::ostream & operator<<(std::ostream & strum, TAddrAddr &x);
   public:
 
     TAddrAddr(SPtr<TIPv6Addr> addr, long pref, long valid);
@@ -46,11 +42,11 @@ class TAddrAddr
     void setTimestamp();
     
     // tentative
-    enum ETentative getTentative();
-    void setTentative(enum ETentative state);
+    enum EAddrStatus getTentative();
+    void setTentative(enum EAddrStatus state);
 
   protected:
-    enum ETentative Tentative;
+    enum EAddrStatus Tentative;
 
     unsigned long Prefered;
     unsigned long Valid;
