@@ -7,22 +7,22 @@
  *
  * released under GNU GPL v2 licence
  *
- * $Id: AddrClient.h,v 1.9 2008-02-25 17:49:06 thomson Exp $
- *
  */
 
 class TAddrClient;
 #ifndef ADDRCLIENT_H
 #define ADDRCLIENT_H
 
+#include <vector>
 #include "SmartPtr.h"
 #include "Container.h"
 #include "AddrIA.h"
 #include "DUID.h"
+#include "Portable.h"
 
 class TAddrClient
 {
-    friend ostream & operator<<(ostream & strum,TAddrClient &x);
+    friend std::ostream & operator<<(std::ostream & strum, TAddrClient &x);
 
 public:
     TAddrClient(SPtr<TDUID> duid);
@@ -63,21 +63,28 @@ public:
     void setSPI(uint32_t val);
     uint64_t getReplayDetectionRcvd();
     void setReplayDetectionRcvd(uint64_t val);
-    uint64_t getNextReplayDetectionSent();
+
+    void generateReconfKey();
 
     unsigned long getLastTimestamp();
-    
+
+    /// @brief 128 bits of pure randomness used in reconfigure process
+    ///
+    /// Reconfigure Key nonce is set be the server and the stored by the client.
+    /// It is later used to check if the reconfigure message came from the same
+    /// server that initially provided the configuration.
+    std::vector<uint8_t> ReconfKey_;
+
 private:
     List(TAddrIA) IAsLst;
     List(TAddrIA) TALst;
     List(TAddrIA) PDLst;
-    SPtr<TDUID> DUID;
+    SPtr<TDUID> DUID_;
 
-    uint32_t SPI;
-    uint64_t ReplayDetectionRcvd;
-    uint64_t ReplayDetectionSent;
+    uint32_t SPI_;
+    uint64_t ReplayDetectionRcvd_;
 };
 
 
 
-#endif 
+#endif
