@@ -13,18 +13,12 @@ class TAddrMgr;
 #define ADDRMGR_H
 
 #include <string>
+#include <map>
 #include "SmartPtr.h"
 #include "Container.h"
 #include "AddrClient.h"
 #include "AddrIA.h"
 
-#ifdef MOD_LIBXML2
-#include <libxml/xmlmemory.h>
-#include <libxml/parser.h>
-
-#define ADDRDB_DTD (xmlChar*)"AddrMgr.dtd"
-#endif
-
 ///
 /// @brief Address Manager that holds address and prefix information.
 ///
@@ -44,10 +38,24 @@ class TAddrMgr;
 class TAddrMgr
 {
   public:
-    friend ostream & operator<<(ostream & strum,TAddrMgr &x);
-    TAddrMgr(string addrdb, bool loadfile = false);
+
+    /// holds network interface name to ifindex mapping
+    typedef std::map<std::string, int> NameToIndexMapping;
+
+    /// holds network interface ifindex to name mapping
+    typedef std::map<int, std::string> IndexToNameMapping;
+
+    friend std::ostream & operator<<(std::ostream & strum,TAddrMgr &x);
+    TAddrMgr(const std::string& addrdb, bool loadfile = false);
     virtual ~TAddrMgr();
 
+    bool updateInterfacesInfo(const NameToIndexMapping& nameToIndex,
+                              const IndexToNameMapping& indexToName);
+
+    bool updateInterfacesInfoIA(SPtr<TAddrIA> ia,
+                                const NameToIndexMapping& nameToIndex,
+                                const IndexToNameMapping& indexToName);
+
     //--- Client container ---
     void addClient(SPtr<TAddrClient> x);
     void firstClient();
@@ -58,17 +66,24 @@ class TAddrMgr
     int countClient();
     bool delClient(SPtr<TDUID> duid);
 
+    // checks if address is conformant to current configuration (used in loadDB())
+    virtual bool verifyAddr(SPtr<TIPv6Addr> addr) { return true; }
+    virtual bool verifyPrefix(SPtr<TIPv6Addr> addr) { return true; }
+
     // --- prefix related ---
     virtual bool addPrefix(SPtr<TDUID> clntDuid, SPtr<TIPv6Addr> clntAddr,
-                           int iface, unsigned long IAID, unsigned long T1, unsigned long T2,
+                           const std::string& ifname,
+                           int ifindex, unsigned long IAID, unsigned long T1, unsigned long T2,
                            SPtr<TIPv6Addr> prefix, unsigned long pref, unsigned long valid,
                            int length, bool quiet);
     virtual bool updatePrefix(SPtr<TDUID> duid , SPtr<TIPv6Addr> addr,
-                              int iface, unsigned long IAID, unsigned long T1, unsigned long T2,
-                              SPtr<TIPv6Addr> prefix, unsigned long pref, unsigned long valid,
-                              int length, bool quiet);
+                              const std::string& ifname,
+                              int ifindex, unsigned long IAID, unsigned long T1,
+                              unsigned long T2, SPtr<TIPv6Addr> prefix, unsigned long pref,
+                              unsigned long valid, int length, bool quiet);
 
-    virtual bool delPrefix(SPtr<TDUID> clntDuid, unsigned long IAID, SPtr<TIPv6Addr> prefix, bool quiet);
+    virtual bool delPrefix(SPtr<TDUID> clntDuid, unsigned long IAID,
+                           SPtr<TIPv6Addr> prefix, bool quiet);
     bool prefixIsFree(SPtr<TIPv6Addr> prefix);
 
     //--- Time related methods ---
@@ -81,7 +96,6 @@ class TAddrMgr
     void dbLoad(const char * xmlFile);
     virtual void dump();
     bool isDone();
-    bool restore;
 
 #ifdef MOD_LIBXML2
     // database loading methods that use libxml2
@@ -94,19 +108,25 @@ class TAddrMgr
     // database loading methods that use internal loading routines
     bool xmlLoadBuiltIn(const char * xmlFile);
     SPtr<TAddrClient> parseAddrClient(const char * xmlFile, FILE *f);
-    SPtr<TAddrIA> parseAddrIA(const char * xmlFile, FILE * f, int t1,int t2,int iaid,int iface);
-    SPtr<TAddrIA> parseAddrPD(const char * xmlFile, FILE * f, int t1,int t2,int iaid,int iface);
+    SPtr<TAddrIA> parseAddrIA(const char * xmlFile, FILE * f, int t1,int t2,
+                              int iaid, const std::string& ifname, int ifindex,
+                              SPtr<TIPv6Addr> unicast = SPtr<TIPv6Addr>());
+    SPtr<TAddrIA> parseAddrPD(const char * xmlFile, FILE * f, int t1,int t2,
+                              int iaid, const std::string& ifname, int ifindex,
+                              SPtr<TIPv6Addr> unicast = SPtr<TIPv6Addr>());
     SPtr<TAddrAddr> parseAddrAddr(const char * xmlFile, char * buf,bool pd);
     SPtr<TAddrPrefix> parseAddrPrefix(const char * xmlFile, char * buf,bool pd);
     SPtr<TAddrIA> parseAddrTA(const char * xmlFile, FILE *f);
 #endif
 
+    uint64_t getNextReplayDetectionValue();
+
 protected:
-    virtual void print(ostream & out) = 0;
+    virtual void print(std::ostream & out) = 0;
     bool addPrefix(SPtr<TAddrClient> client, SPtr<TDUID> duid , SPtr<TIPv6Addr> clntAddr,
-                   int iface, unsigned long IAID, unsigned long T1, unsigned long T2,
-                   SPtr<TIPv6Addr> prefix, unsigned long pref, unsigned long valid,
-                   int length, bool quiet);
+                   const std::string& ifname, int ifindex, unsigned long IAID,
+                   unsigned long T1, unsigned long T2, SPtr<TIPv6Addr> prefix,
+                   unsigned long pref, unsigned long valid, int length, bool quiet);
     bool updatePrefix(SPtr<TAddrClient> client, SPtr<TDUID> duid , SPtr<TIPv6Addr> clntAddr,
                       int iface, unsigned long IAID, unsigned long T1, unsigned long T2,
                       SPtr<TIPv6Addr> prefix, unsigned long pref, unsigned long valid,
@@ -114,9 +134,12 @@ protected:
 
     bool IsDone;
     List(TAddrClient) ClntsLst;
-    string XmlFile;
+    std::string XmlFile;
 
-    bool DeleteEmptyClient; // should the client without any IA, TA or PDs be deleted? (srv = yes, client = no)
+    /// should the client without any IA, TA or PDs be deleted? (srv = yes, client = no)
+    bool DeleteEmptyClient;
+
+    uint64_t ReplayDetectionValue_;
 };
 
 #endif
