@@ -6,7 +6,6 @@
  *
  * released under GNU GPL v2 only licence
  *
- * $Id: ClntAddrMgr.cpp,v 1.26 2008-11-11 22:02:43 thomson Exp $
  */
 
 #include "SmartPtr.h"
@@ -15,10 +14,12 @@
 #include "AddrClient.h"
 #include "Logger.h"
 
+using namespace std;
+
 TClntAddrMgr * TClntAddrMgr::Instance = 0;
 
-void TClntAddrMgr::instanceCreate(SPtr<TDUID> clientDUID, bool useConfirm, string xmlFile, bool loadDB)
-{
+void TClntAddrMgr::instanceCreate(SPtr<TDUID> clientDUID, bool useConfirm,
+                                  const std::string& xmlFile, bool loadDB) {
   if (Instance) {
       Log(Crit) << "Attempt to create another instance of TClntAddrMgr!" << LogEnd;
       return;
@@ -42,15 +43,16 @@ TClntAddrMgr& TClntAddrMgr::instance()
  * @param loadDB should existing dump be loaded?
  *
  */
-TClntAddrMgr::TClntAddrMgr(SPtr<TDUID> clientDUID, bool useConfirm, string xmlFile, bool loadDB)
+TClntAddrMgr::TClntAddrMgr(SPtr<TDUID> clientDUID, bool useConfirm,
+                           const std::string& xmlFile, bool loadDB)
     :TAddrMgr(xmlFile, useConfirm)
 {
     // client may have been already loaded from client-AddrMgr.xml file
     firstClient();
     if (!getClient()) {
-        // add this client (with proper duid)
-        SPtr<TAddrClient> client = new TAddrClient(clientDUID);
-        addClient(client);
+	// add this client (with proper duid)
+	SPtr<TAddrClient> client = new TAddrClient(clientDUID);
+	addClient(client);
     }
 
     // set Client field
@@ -59,23 +61,22 @@ TClntAddrMgr::TClntAddrMgr(SPtr<TDUID> clientDUID, bool useConfirm, string xmlFi
     Client = getClient();
 
     if (useConfirm)
-        processLoadedDB();
+	processLoadedDB();
 }
 
 void TClntAddrMgr::processLoadedDB() {
     SPtr<TAddrIA> ia;
     Client->firstIA();
     while (ia=Client->getIA()) {
-        ia->setState(STATE_CONFIRMME);
+	ia->setState(STATE_CONFIRMME);
     }
 
     Client->firstPD();
     while (ia=Client->getPD()) {
-        ia->setState(STATE_CONFIRMME);
+	ia->setState(STATE_CONFIRMME);
     }
 }
 
-
 unsigned long TClntAddrMgr::getT1Timeout() {
     return Client->getT1Timeout();
 }
@@ -91,14 +92,18 @@ unsigned long TClntAddrMgr::getValidTimeout() {
 
 unsigned long TClntAddrMgr::getTimeout()
 {
-    unsigned long val,val2;
+    unsigned long val, val2;
     val = this->getT1Timeout();
     if ( (val2 = this->getT2Timeout()) < val)
-        val = val2;
-    if ( (val2 = this->getPrefTimeout()) < val)
-        val = val2;
+	val = val2;
+
+    // no special action is required after preferred timeout reached
+    // (except perhaps logging that we are screwed, as no new connections could
+    // be created)
+    //if ( (val2 = this->getPrefTimeout()) < val)
+    //  val = val2;
     if ( (val2 = this->getValidTimeout()) < val)
-        val = val2;
+	val = val2;
     return val;
 }
 
@@ -106,14 +111,13 @@ unsigned long TClntAddrMgr::getTentativeTimeout()
 {
     SPtr<TAddrIA> ptrIA;
     Client->firstIA();
-    unsigned long min = DHCPV6_INFINITY;
-    unsigned long tmp;
+    uint32_t min = DHCPV6_INFINITY;
 
     while(ptrIA=Client->getIA())
     {
-	      tmp = ptrIA->getTentativeTimeout();
-	      if (min > tmp)
-	          min = tmp;
+        uint32_t tmp = ptrIA->getTentativeTimeout();
+        if (min > tmp)
+            min = tmp;
     }
 
     return min;
@@ -130,24 +134,26 @@ void TClntAddrMgr::doDuties()
 {
     SPtr<TAddrIA> ptrIA;
     SPtr<TAddrAddr> ptrAddr;
+    
+    firstIA();
+    while ( ptrIA = this->getIA() ) {
 
-    this->firstIA();
-    while ( ptrIA = this->getIA() ) 
-    {
-	      ptrIA->firstAddr();
-	      while( ptrAddr=ptrIA->getAddr())
-	      {
-	          //Removing outdated addresses
-	          if(!ptrAddr->getValidTimeout())
-	          {
-		            ptrIA->delAddr(ptrAddr->get());
-		            Log(Notice) << "Address " << ptrAddr->get()->getPlain() 
-			                 << " from IA " << ptrIA->getIAID() 
-			                 << " has been removed from addrDB." << LogEnd;
-		            ptrIA->setState(STATE_NOTCONFIGURED);
-	          }
-	      }
-    }	    
+	ptrIA->firstAddr();
+	while( ptrAddr=ptrIA->getAddr()) {
+	    //Removing outdated addresses
+	    if(!ptrAddr->getValidTimeout()) {
+		ptrIA->delAddr(ptrAddr->get());
+		Log(Info) << "Expired address " << ptrAddr->get()->getPlain()
+			  << " from IA " << ptrIA->getIAID()
+			  << " has been removed from addrDB." << LogEnd;
+	    }
+	}
+
+	if ( ((ptrIA->getState() == STATE_CONFIGURED) || (ptrIA->getState() == STATE_CONFIRMME))
+		 && !ptrIA->countAddr()) {
+	    ptrIA->setState(STATE_NOTCONFIGURED);
+	}
+    }
 }
 
 void TClntAddrMgr::firstIA() {
@@ -171,26 +177,26 @@ int TClntAddrMgr::countIA() {
 }
 
 TClntAddrMgr::~TClntAddrMgr() {
-    this->Client = 0;
+    Client.reset();
     Log(Debug) << "ClntAddrMgr cleanup." << LogEnd;
 }
 
 SPtr<TAddrIA> TClntAddrMgr::getIA(unsigned long IAID)
 {
     SPtr<TAddrIA> ptrIA;
-    this->Client->firstIA();
-    while (ptrIA = this->Client->getIA() ) {
+    Client->firstIA();
+    while (ptrIA = Client->getIA() ) {
 	if (ptrIA->getIAID() == IAID)
 	    return ptrIA;
     }
-    return 0;
+    return SPtr<TAddrIA>();
 }
 
-/** 
+/**
  * sets specified interface to CONFIRMME state
- * when network switch off signal received, the funtion will be invoked to 
+ * when network switch off signal received, the funtion will be invoked to
  * set valid IA to CONFIRMME state.
- * 
+ *
  * @param changedLinks structure containing interface indexes to be confirmed
  */
 void TClntAddrMgr::setIA2Confirm(volatile link_state_notify_t * changedLinks)
@@ -200,26 +206,26 @@ void TClntAddrMgr::setIA2Confirm(volatile link_state_notify_t * changedLinks)
     while(ptrIA = this->getIA()){
 
 	bool found = false;
-	int ifindex = ptrIA->getIface(); // interface index of this IA
+	int ifindex = ptrIA->getIfindex(); // interface index of this IA
 
 	// is this index on the list of interfaces to be confirmed?
 	for (int i=0; i < MAX_LINK_STATE_CHANGES_AT_ONCE; i++)
 	    if (changedLinks->ifindex[i]==ifindex)
 		found = true;
 
-        if (!found)
-            continue;
+	if (!found)
+	    continue;
 
-        if( (ptrIA->getState() == STATE_CONFIGURED || ptrIA->getState() == STATE_INPROCESS) )
+	if( (ptrIA->getState() == STATE_CONFIGURED || ptrIA->getState() == STATE_INPROCESS) )
 	{
 	    ptrIA->setState(STATE_CONFIRMME);
 	    Log(Notice) << "Network switch off event detected. do Confirmming." << LogEnd;
 	}
-    } 
+    }
 
 }
 
-// pd functions 
+// pd functions
 
 void TClntAddrMgr::firstPD() {
     Client->firstPD();
@@ -249,13 +255,10 @@ SPtr<TAddrIA> TClntAddrMgr::getPD(unsigned long IAID)
 	if (ptrPD->getIAID() == IAID)
 	    return ptrPD;
     }
-    return 0;
+    return SPtr<TAddrIA>(); // NULL
 }
 
-
-
-
-void TClntAddrMgr::firstTA() 
+void TClntAddrMgr::firstTA()
 {
     Client->firstTA();
 }
@@ -273,7 +276,7 @@ SPtr<TAddrIA> TClntAddrMgr::getTA(unsigned long iaid)
 	if (ta->getIAID() == iaid)
 	    return ta;
     }
-    return 0;
+    return SPtr<TAddrIA>(); // NULL
 }
 
 void TClntAddrMgr::addTA(SPtr<TAddrIA> ptr)
@@ -291,15 +294,15 @@ int TClntAddrMgr::countTA()
     return Client->countTA();
 }
 
-void TClntAddrMgr::print(ostream &x) {
-    
+void TClntAddrMgr::print(std::ostream &) {
+
 }
 
 /**
- * @brief Adds a prefix. 
+ * @brief Adds a prefix.
  *
- * @param srvDuid Server DUID 
- * @param srvAddr Server address. 
+ * @param srvDuid Server DUID
+ * @param srvAddr Server address.
  * @param iface   interface index
  * @param IAID    IAID
  * @param T1      T1 timer
@@ -313,9 +316,9 @@ void TClntAddrMgr::print(ostream &x) {
  * @return true if successful, false otherwise
  */
 bool TClntAddrMgr::addPrefix(SPtr<TDUID> srvDuid , SPtr<TIPv6Addr> srvAddr,
-			     int iface, unsigned long IAID, unsigned long T1, unsigned long T2, 
+			     const std::string& ifacename, int iface, unsigned long IAID, unsigned long T1, unsigned long T2,
 			     SPtr<TIPv6Addr> prefix, unsigned long pref, unsigned long valid,
-			     int length, bool quiet) 
+			     int length, bool quiet)
 {
     if (!prefix) {
 	Log(Error) << "Attempt to add null prefix failed." << LogEnd;
@@ -327,11 +330,13 @@ bool TClntAddrMgr::addPrefix(SPtr<TDUID> srvDuid , SPtr<TIPv6Addr> srvAddr,
     this->firstClient();
     ptrClient = getClient();
 
-    return TAddrMgr::addPrefix(ptrClient, srvDuid, srvAddr, iface, IAID, T1, T2, prefix, 
+    return TAddrMgr::addPrefix(ptrClient, srvDuid, srvAddr, ifacename,
+                               iface, IAID, T1, T2, prefix,
 			       pref, valid, length, quiet);
 }
 
 bool TClntAddrMgr::updatePrefix(SPtr<TDUID> srvDuid , SPtr<TIPv6Addr> srvAddr,
+                                const std::string& ifname,
 				int iface, unsigned long IAID, unsigned long T1, unsigned long T2,
 				SPtr<TIPv6Addr> prefix, unsigned long pref, unsigned long valid,
 				int length, bool quiet)
@@ -346,26 +351,26 @@ bool TClntAddrMgr::updatePrefix(SPtr<TDUID> srvDuid , SPtr<TIPv6Addr> srvAddr,
     this->firstClient();
     ptrClient = getClient();
 
-    return TAddrMgr::updatePrefix(ptrClient, srvDuid, srvAddr, iface, IAID, T1, T2, prefix, 
+    return TAddrMgr::updatePrefix(ptrClient, srvDuid, srvAddr, iface, IAID, T1, T2, prefix,
 				  pref, valid, length, quiet);
 }
 
 SPtr<TIPv6Addr> TClntAddrMgr::getPreferredAddr() {
     SPtr<TAddrIA> ia;
     SPtr<TAddrAddr> addr;
-    
+
     firstIA();
     while ( ia = getIA() ) {
-	if (ia->getTentative() != TENTATIVE_NO)
+	if (ia->getTentative() != ADDRSTATUS_NO)
 	    continue;
 
 	ia->firstAddr();
 	while (addr = ia->getAddr()) {
-	    if (addr->getTentative() == TENTATIVE_NO)
+	    if (addr->getTentative() == ADDRSTATUS_NO)
 		return addr->get(); // return the first address from first non-tentative
 	    //if (is_addr_tentative(NULL, ia->getIface(), addr->get()->getPlain()) == LOWLEVEL_TENTATIVE_NO)
 	}
     }
 
-    return 0;
+    return SPtr<TIPv6Addr>(); // NULL
 }
