@@ -6,8 +6,6 @@
  *
  * released under GNU GPL v2 only licence
  *
- * $Id: ClntAddrMgr.h,v 1.14 2008-10-12 14:05:25 thomson Exp $
- *
  */
 
 #ifndef CLNTADDRMGR_H
@@ -19,17 +17,19 @@
 #include "AddrMgr.h"
 #include "Portable.h"
 #include "IPv6Addr.h"
+#include "ScriptParams.h"
 
 #define ClntAddrMgr() (TClntAddrMgr::instance())
 
 class TClntAddrMgr : public TAddrMgr
 {
  private:
-    TClntAddrMgr(SPtr<TDUID> clientDuid, bool useConfirm, string xmlFile, bool loadDB);
+    TClntAddrMgr(SPtr<TDUID> clientDuid, bool useConfirm, const std::string& xmlFile, bool loadDB);
 
   public:
     static TClntAddrMgr& instance();
-    static void instanceCreate(SPtr<TDUID> clientDUID, bool useConfirm, string xmlFile, bool loadDB);
+    static void instanceCreate(SPtr<TDUID> clientDUID, bool useConfirm, 
+                               const std::string& xmlFile, bool loadDB);
 
     unsigned long getT1Timeout();
     unsigned long getT2Timeout();
@@ -57,10 +57,12 @@ class TClntAddrMgr : public TAddrMgr
     bool delPD(long IAID);
     int countPD();
     bool addPrefix(SPtr<TDUID> srvDuid , SPtr<TIPv6Addr> srvAddr,
-		   int iface, unsigned long IAID, unsigned long T1, unsigned long T2,
+		   const std::string& ifacename, int iface, unsigned long IAID, unsigned long T1, unsigned long T2,
 		   SPtr<TIPv6Addr> prefix, unsigned long pref, unsigned long valid,
 		   int length, bool quiet);
+
     bool updatePrefix(SPtr<TDUID> srvDuid , SPtr<TIPv6Addr> srvAddr,
+                      const std::string& ifname,
 		      int iface, unsigned long IAID, unsigned long T1, unsigned long T2,
 		      SPtr<TIPv6Addr> prefix, unsigned long pref, unsigned long valid,
 		      int length, bool quiet);
@@ -79,7 +81,7 @@ class TClntAddrMgr : public TAddrMgr
     void processLoadedDB();
     
  protected:
-    void print(ostream &x);
+    void print(std::ostream &x);
  private:
     SPtr<TAddrClient> Client;
     static TClntAddrMgr * Instance;
