@@ -47,18 +47,18 @@ void TClntCfgAddr::setOptions(SPtr<TClntParsGlobalOpt> opt) {
 }
 
 TClntCfgAddr::TClntCfgAddr() {
-    this->Valid=ULONG_MAX;
-    this->Pref=ULONG_MAX;
+    this->Valid=UINT_MAX;
+    this->Pref=UINT_MAX;
     Addr= new TIPv6Addr();
 }
 
 TClntCfgAddr::TClntCfgAddr(SPtr<TIPv6Addr> addr) {
     Addr=addr;
-    Valid = ULONG_MAX;
-    Pref = ULONG_MAX;
+    Valid = UINT_MAX;
+    Pref = UINT_MAX;
 }
 
-ostream& operator<<(ostream& out,TClntCfgAddr& addr) {
+std::ostream& operator<<(std::ostream& out, TClntCfgAddr& addr) {
     out << "<addr preferred=\"" << addr.Pref 
 	<< "\" valid=\"" << addr.Valid << "\">";
     out << *addr.Addr << "</addr>" << std::endl;	
