@@ -6,56 +6,44 @@
  *
  * released under GNU GPL v2 only licence
  *
- * $Id: ClntParsGlobalOpt.cpp,v 1.20 2008-08-30 21:41:11 thomson Exp $
- *
  */
 
 #include "ClntParsGlobalOpt.h"
 #include "Portable.h"
-#include "DHCPConst.h"
+#include "DHCPDefaults.h"
 #include "Logger.h"
 
+using namespace std;
+
 TClntParsGlobalOpt::TClntParsGlobalOpt()
     :TClntParsIfaceOpt() {
-    this->WorkDir        = WORKDIR;
+    this->WorkDir        = DEFAULT_WORKDIR;
     this->PrefixLength   = CLIENT_DEFAULT_PREFIX_LENGTH;
-    this->Digest         = CLIENT_DEFAULT_DIGEST;
     this->AnonInfRequest = false;
     this->InactiveMode   = false;
     this->InsistMode     = false;
     this->FQDNFlagS      = CLIENT_DEFAULT_FQDN_FLAG_S;
     this->Experimental   = false;
     this->UseConfirm     = true;
-
-    this->AuthEnabled    = false;
-    this->AuthAcceptMethods.clear();
 }
 
 TClntParsGlobalOpt::~TClntParsGlobalOpt() {
 }
 
-void TClntParsGlobalOpt::setWorkDir(string dir) {
-    this->WorkDir=dir;
+void TClntParsGlobalOpt::setWorkDir(const std::string& dir) {
+    WorkDir = dir;
 }
 
 string TClntParsGlobalOpt::getWorkDir() {
-    return this->WorkDir;
+    return WorkDir;
 }
 
-void TClntParsGlobalOpt::setPrefixLength(int len) {
-    this->PrefixLength = len;
+void TClntParsGlobalOpt::setOnLinkPrefixLength(int len) {
+    PrefixLength = len;
 }
 
-int TClntParsGlobalOpt::getPrefixLength() {
-    return this->PrefixLength;
-}
-
-void TClntParsGlobalOpt::setDigest(DigestTypes digest) {
-    this->Digest = digest;
-}
-
-DigestTypes TClntParsGlobalOpt::getDigest() {
-    return this->Digest;
+int TClntParsGlobalOpt::getOnLinkPrefixLength() {
+    return PrefixLength;
 }
 
 void TClntParsGlobalOpt::setAnonInfRequest(bool anonymous) {
@@ -96,17 +84,6 @@ bool TClntParsGlobalOpt::getExperimental()
     return Experimental;
 }
 
-void TClntParsGlobalOpt::setAuthAcceptMethods(List(DigestTypes) lst)
-{
-    AuthAcceptMethods = lst;
-    Log(Debug) << "AUTH: " << lst.count() << " method(s) accepted." << LogEnd;
-}
-
-List(DigestTypes) TClntParsGlobalOpt::getAuthAcceptMethods()
-{
-    return AuthAcceptMethods;
-}
-
 void TClntParsGlobalOpt::setFQDNFlagS(bool s)
 {
     FQDNFlagS = s;
@@ -117,18 +94,6 @@ bool TClntParsGlobalOpt::getFQDNFlagS()
     return FQDNFlagS;
 }
 
-void TClntParsGlobalOpt::setAuthEnabled(bool enabled)
-{
-    AuthEnabled = enabled;
-    Log(Debug) << "AUTH: Authentication " << (enabled?"enabled":"disabled")
-               << "." << LogEnd;
-}
-
-bool TClntParsGlobalOpt::getAuthEnabled()
-{
-    return AuthEnabled;
-}
-
 void TClntParsGlobalOpt::setConfirm(bool conf)
 {
     UseConfirm = conf;
