@@ -1,18 +1,19 @@
-/*                                                                           *
- * Dibbler - a portable DHCPv6                                               *
- *                                                                           *
- * authors: Tomasz Mrugalski <thomson@klub.com.pl>                           *
- *          Marek Senderski  <msend@o2.pl>                                   *
- *                                                                           *
- * released under GNU GPL v2 only licence                                */
+/*
+ * Dibbler - a portable DHCPv6
+ *
+ * authors: Tomasz Mrugalski <thomson@klub.com.pl>
+ *          Marek Senderski  <msend@o2.pl>
+ *
+ * released under GNU GPL v2 only licence
+ */
 
 #include "ClntParsIAOpt.h"
 #include <limits.h>
 
 TClntParsIAOpt::TClntParsIAOpt() : TClntParsAddrOpt()
 {
-    T1=ULONG_MAX;
-    T2=ULONG_MAX;
+    T1=UINT_MAX;
+    T2=UINT_MAX;
     IAIDCnt=0;
     AddrHint=true;
     AddrParams = false;
@@ -57,7 +58,7 @@ void TClntParsIAOpt::setAddrHint(bool addrHint)
     this->AddrHint=addrHint;
 }
 
-void TClntParsIAOpt::addPrefSrv(SPtr<TStationID> prefSrv)
+void TClntParsIAOpt::addPrefSrv(SPtr<THostID> prefSrv)
 {
     this->PrefSrv.append(prefSrv);
 }
@@ -67,7 +68,7 @@ void TClntParsIAOpt::firstPrefSrv()
     this->PrefSrv.first();
 }
 
-SPtr<TStationID> TClntParsIAOpt::getPrefSrv()
+SPtr<THostID> TClntParsIAOpt::getPrefSrv()
 {
     return this->PrefSrv.get();
 }
@@ -76,16 +77,16 @@ void TClntParsIAOpt::clearPrefSrv()
     this->PrefSrv.clear();
 }
 
-void TClntParsIAOpt::setPrefSrvLst(TContainer<SPtr<TStationID> > *lst)
+void TClntParsIAOpt::setPrefSrvLst(TContainer<SPtr<THostID> > *lst)
 {
-    SPtr<TStationID> id;
+    SPtr<THostID> id;
     this->PrefSrv.clear();
     lst->first();
     while(id=lst->get())
         this->PrefSrv.append(id);
 }
 
-void TClntParsIAOpt::addRejedSrv(SPtr<TStationID> prefSrv)
+void TClntParsIAOpt::addRejedSrv(SPtr<THostID> prefSrv)
 {
     this->RejedSrv.append(prefSrv);
 }
@@ -99,16 +100,16 @@ void TClntParsIAOpt::clearRejedSrv()
 {
     this->RejedSrv.clear();
 }
-void TClntParsIAOpt::setRejedSrvLst(TContainer<SPtr<TStationID> > *lst)
+void TClntParsIAOpt::setRejedSrvLst(TContainer<SPtr<THostID> > *lst)
 {
-    SPtr<TStationID> id;
+    SPtr<THostID> id;
     this->RejedSrv.clear();
     lst->first();
     while(id=lst->get())
         this->RejedSrv.append(id);
 }
 
-SPtr<TStationID> TClntParsIAOpt::getRejedSrv()
+SPtr<THostID> TClntParsIAOpt::getRejedSrv()
 {
     return this->RejedSrv.get();
 }
