@@ -9,59 +9,33 @@
  *
  */
 
-#include "ClntParsIfaceOpt.h"
 #include <iostream>
 #include <iomanip>
+#include "ClntParsIfaceOpt.h"
+#include "DHCPDefaults.h"
 #include "Logger.h"
-#include "ClntOptVendorSpec.h"
 
 using namespace std;
 
-TClntParsIfaceOpt::TClntParsIfaceOpt() : TClntParsIAOpt()
+TClntParsIfaceOpt::TClntParsIfaceOpt()
+    : TClntParsIAOpt(), Stateless_(false), Unicast(CLIENT_DEFAULT_UNICAST),
+      RapidCommit(CLIENT_DEFAULT_RAPID_COMMIT), Timezone(""), FQDN(""),
+      NISDomain(""), Lifetime(false), ReqDNSServer(false), ReqDomain(false),
+      ReqNTPServer(false), ReqTimezone(false), ReqSIPServer(false),
+      ReqSIPDomain(false), ReqFQDN(false), ReqNISServer(false),
+      ReqNISPServer(false), ReqNISDomain(false), ReqNISPDomain(false),
+      ReqLifetime(false), ReqPrefixDelegation(false), ReqVendorSpec(false)
 {
-    DNSServerLst.clear();
-    DomainLst.clear();
-    NTPServerLst.clear();
-    Timezone="";
-    SIPServerLst.clear();
-    SIPDomainLst.clear();
-    FQDN    ="";
-    NISServerLst.clear();
-    NISDomain = "";
-    NISPServerLst.clear();
-    NISPDomain = "";
-    Lifetime = false;
-    VendorSpec.clear();
-
-    NoIAs   = false;
-
-    this->Unicast     = CLIENT_DEFAULT_UNICAST;
-    this->RapidCommit = CLIENT_DEFAULT_RAPID_COMMIT;
-
-    ReqDNSServer  = false;
-    ReqDomain     = false;
-    ReqNTPServer  = false;
-    ReqTimezone   = false;
-    ReqSIPServer  = false;
-    ReqSIPDomain  = false;
-    ReqFQDN       = false;
-    ReqNISServer  = false;
-    ReqNISPServer = false;
-    ReqNISDomain  = false;
-    ReqNISPDomain = false;
-    ReqLifetime   = false;
-    ReqPrefixDelegation = false;
-    ReqVendorSpec = false;
 }
 
-bool TClntParsIfaceOpt::getIsIAs()
+bool TClntParsIfaceOpt::getStateful()
 {
-    return !NoIAs;
+    return !Stateless_;
 }
 
-void TClntParsIfaceOpt::setIsIAs(bool state)
+void TClntParsIfaceOpt::setStateful(bool state)
 {
-    NoIAs=!state;
+    Stateless_ = !state;
 }
 
 void TClntParsIfaceOpt::setUnicast(bool unicast)
@@ -100,10 +74,10 @@ bool TClntParsIfaceOpt::getReqDNSServer() {
 }
 
 // --- option: domain ---
-List(string) * TClntParsIfaceOpt::getDomainLst() {
+List(std::string) * TClntParsIfaceOpt::getDomainLst() {
     return &this->DomainLst;
 }
-void TClntParsIfaceOpt::setDomainLst(List(string) * domain) {
+void TClntParsIfaceOpt::setDomainLst(List(std::string) * domain) {
     this->DomainLst=*domain;
     this->ReqDomain=true;
 }
@@ -124,7 +98,7 @@ bool TClntParsIfaceOpt::getReqNTPServer(){
 }
 
 // --- option: Timezone ---
-void TClntParsIfaceOpt::setTimezone(string Timezone) {
+void TClntParsIfaceOpt::setTimezone(const std::string& Timezone) {
     this->Timezone=Timezone;
     this->ReqTimezone=true;
 }
@@ -149,10 +123,10 @@ bool TClntParsIfaceOpt::getReqSIPServer(){
 }
 
 // --- option: SIP domain ---
-List(string) * TClntParsIfaceOpt::getSIPDomainLst() {
+List(std::string) * TClntParsIfaceOpt::getSIPDomainLst() {
     return &this->SIPDomainLst;
 }
-void TClntParsIfaceOpt::setSIPDomainLst(List(string) * domain) {
+void TClntParsIfaceOpt::setSIPDomainLst(List(std::string) * domain) {
     this->SIPDomainLst=*domain;
     this->ReqSIPDomain=true;
 }
@@ -161,7 +135,7 @@ bool TClntParsIfaceOpt::getReqSIPDomain() {
 }
 
 // --- option: FQDN ---
-void TClntParsIfaceOpt::setFQDN(string fqdn) {
+void TClntParsIfaceOpt::setFQDN(const std::string& fqdn) {
     this->FQDN=fqdn;
     this->ReqFQDN=true;
 }
@@ -203,7 +177,7 @@ string TClntParsIfaceOpt::getNISDomain() {
     return this->NISDomain;
 }
 
-void TClntParsIfaceOpt::setNISDomain(string domain) {
+void TClntParsIfaceOpt::setNISDomain(const std::string& domain) {
     this->NISDomain=domain;
     this->ReqNISDomain=true;
 }
@@ -230,7 +204,7 @@ bool TClntParsIfaceOpt::getReqNISPServer(){
 string TClntParsIfaceOpt::getNISPDomain() {
     return this->NISPDomain;
 }
-void TClntParsIfaceOpt::setNISPDomain(string domain) {
+void TClntParsIfaceOpt::setNISPDomain(const std::string& domain) {
     this->NISPDomain=domain;
     this->ReqNISPDomain=true;
 }
