@@ -8,14 +8,18 @@
  *
  */
 
+#include <unistd.h>
 #include <iostream>
 #include "ClntIfaceIface.h"
 #include "Portable.h"
 #include "Logger.h"
+#include "DHCPDefaults.h"
 #ifdef MINGWBUILD
 #include <io.h>
 #endif
 
+using namespace std;
+
 /*
  * stores informations about interface
  */
@@ -23,22 +27,27 @@ TClntIfaceIface::TClntIfaceIface(char * name, int id, unsigned int flags, char*
                                  int maclen, char* llAddr, int llAddrCnt, char * globalAddr,
                                  int globalAddrCnt, int hwType)
   :TIfaceIface(name, id, flags, mac, maclen, llAddr, llAddrCnt, globalAddr, globalAddrCnt, hwType),
-   TunnelEndpoint(0), LifetimeTimeout(DHCPV6_INFINITY), LifetimeTimestamp(now()) {
+   TunnelEndpoint(), LifetimeTimeout(DHCPV6_INFINITY) {
+
+    LifetimeTimestamp = (uint32_t)time(NULL);
+
     this->DNSServerLst.clear();
     this->DomainLst.clear();
     this->NTPServerLst.clear();
     this->Timezone = "";
 
-    unlink(WORKDIR"/"OPTION_DNS_SERVERS_FILENAME);
-    unlink(WORKDIR"/"OPTION_DOMAINS_FILENAME);
-    unlink(WORKDIR"/"OPTION_NTP_SERVERS_FILENAME);
-    unlink(WORKDIR"/"OPTION_TIMEZONE_FILENAME);
-    unlink(WORKDIR"/"OPTION_SIP_SERVERS_FILENAME);
-    unlink(WORKDIR"/"OPTION_SIP_DOMAINS_FILENAME);
-    unlink(WORKDIR"/"OPTION_NIS_SERVERS_FILENAME);
-    unlink(WORKDIR"/"OPTION_NIS_DOMAIN_FILENAME);
-    unlink(WORKDIR"/"OPTION_NISP_SERVERS_FILENAME);
-    unlink(WORKDIR"/"OPTION_NISP_DOMAIN_FILENAME);
+    DnsConfigured = ! FLUSH_OTHER_CONFIGURED_DNS_SERVERS;
+
+    unlink(OPTION_DNS_SERVERS_FILENAME);
+    unlink(OPTION_DOMAINS_FILENAME);
+    unlink(OPTION_NTP_SERVERS_FILENAME);
+    unlink(OPTION_TIMEZONE_FILENAME);
+    unlink(OPTION_SIP_SERVERS_FILENAME);
+    unlink(OPTION_SIP_DOMAINS_FILENAME);
+    unlink(OPTION_NIS_SERVERS_FILENAME);
+    unlink(OPTION_NIS_DOMAIN_FILENAME);
+    unlink(OPTION_NISP_SERVERS_FILENAME);
+    unlink(OPTION_NISP_DOMAIN_FILENAME);
 
     setPrefixLength(CLIENT_DEFAULT_PREFIX_LENGTH);
 }
@@ -49,7 +58,7 @@ TClntIfaceIface::TClntIfaceIface(char * name, int id, unsigned int flags, char*
 unsigned int TClntIfaceIface::getTimeout() {
     if (this->LifetimeTimeout == DHCPV6_INFINITY)
         return DHCPV6_INFINITY;
-    unsigned int current = now();
+    unsigned int current = (uint32_t)time(NULL);
     if (current > this->LifetimeTimestamp+this->LifetimeTimeout)
         return 0;
     return this->LifetimeTimestamp+this->LifetimeTimeout-current;
@@ -57,6 +66,17 @@ unsigned int TClntIfaceIface::getTimeout() {
 
 bool TClntIfaceIface::setDNSServerLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv,
                                       List(TIPv6Addr) addrs) {
+
+#ifdef WIN32
+    //remove all dns records before the first configuration
+    if (!DnsConfigured) {
+        Log(Notice) << "Removing all DNS servers on interface "
+                    << getFullName() << " (not configured yet)." << LogEnd;
+        dns_del(this->getName(), this->getID(), NULL);
+        DnsConfigured = true;
+    }
+#endif
+
     // remove old addresses
     SPtr<TIPv6Addr> old, addr;
     this->DNSServerLst.first();
@@ -76,7 +96,7 @@ bool TClntIfaceIface::setDNSServerLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv,
             Log(Notice) << "Removing DNS server " << *old << " on interface "
                         << this->getName() << "/" << this->getID() << " (no longer valid)." << LogEnd;
             dns_del(this->getName(), this->getID(), old->getPlain());
-            this->delString(WORKDIR"/"OPTION_DNS_SERVERS_FILENAME,old->getPlain());
+            this->delString(OPTION_DNS_SERVERS_FILENAME,old->getPlain());
             this->DNSServerLst.del();
         }
     }
@@ -101,16 +121,14 @@ bool TClntIfaceIface::setDNSServerLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv,
             Log(Notice) << "Setting up DNS server " << * addr << " on interface "
                         << this->getName() << "/" << this->getID() << "." << LogEnd;
             dns_add(this->getName(), this->getID(), addr->getPlain());
-            this->addString(WORKDIR"/"OPTION_DNS_SERVERS_FILENAME,addr->getPlain());
+            this->addString(OPTION_DNS_SERVERS_FILENAME,addr->getPlain());
             this->DNSServerLst.append(addr);
         }
     }
-    this->DNSServerLstAddr = srv;
-    this->DNSServerLstDUID = duid;
     return true;
 }
 
-bool TClntIfaceIface::setDomainLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, List(string) domains) {
+bool TClntIfaceIface::setDomainLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, List(std::string) domains) {
     // remove old domains
     SPtr<string> old, domain;
     this->DomainLst.first();
@@ -130,7 +148,7 @@ bool TClntIfaceIface::setDomainLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, List(s
             Log(Notice) << "Removing domain " << *old << " from interface "
                         << this->getName() << "/" << this->getID() << " (no longer valid)." << LogEnd;
             domain_del(this->getName(), this->getID(), old->c_str());
-            this->delString(WORKDIR"/"OPTION_DOMAINS_FILENAME,old->c_str());
+            this->delString(OPTION_DOMAINS_FILENAME,old->c_str());
             this->DomainLst.del();
         }
     }
@@ -155,12 +173,10 @@ bool TClntIfaceIface::setDomainLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, List(s
             Log(Notice) << "Setting up Domain " << * domain << " on interface "
                         << this->getName() << "/" << this->getID() << "." << LogEnd;
             domain_add(this->getName(), this->getID(), domain->c_str());
-            this->addString(WORKDIR"/"OPTION_DOMAINS_FILENAME,domain->c_str());
+            this->addString(OPTION_DOMAINS_FILENAME,domain->c_str());
             this->DomainLst.append(domain);
         }
     }
-    this->DomainLstAddr = srv;
-    this->DomainLstDUID = duid;
     return true;
 }
 
@@ -185,7 +201,7 @@ bool TClntIfaceIface::setNTPServerLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv,
             Log(Notice) << "Removing NTP server " << *old << " on interface "
                         << this->getName() << "/" << this->getID() << " (no longer valid)." << LogEnd;
             ntp_del(this->getName(), this->getID(), old->getPlain());
-            this->delString(WORKDIR"/"OPTION_NTP_SERVERS_FILENAME,old->getPlain());
+            this->delString(OPTION_NTP_SERVERS_FILENAME,old->getPlain());
             this->NTPServerLst.del();
         }
     }
@@ -210,18 +226,14 @@ bool TClntIfaceIface::setNTPServerLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv,
             Log(Notice) << "Setting up NTP server " << * addr << " on the interface "
                         << this->getName() << "/" << this->getID() << "." << LogEnd;
             ntp_add(this->getName(), this->getID(), addr->getPlain());
-            this->addString(WORKDIR"/"OPTION_NTP_SERVERS_FILENAME,addr->getPlain());
+            this->addString(OPTION_NTP_SERVERS_FILENAME,addr->getPlain());
             this->NTPServerLst.append(addr);
         }
     }
-    this->NTPServerLstAddr = srv;
-    this->NTPServerLstDUID = duid;
     return true;
 }
 
-bool TClntIfaceIface::setTimezone(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, string timezone) {
-    this->TimezoneAddr = srv;
-    this->TimezoneDUID = duid;
+bool TClntIfaceIface::setTimezone(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, const std::string& timezone) {
     if (timezone==this->Timezone) {
         // timezone has not changed
         Log(Info) << "Timezone " << timezone << " is already set on the interface "
@@ -233,7 +245,7 @@ bool TClntIfaceIface::setTimezone(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, string
     Log(Notice) << "Setting up timezone " << timezone << " (obtained on the interface "
                 << this->getName() << "/" << this->getID() << ")." << LogEnd;
     timezone_set(this->getName(), this->getID(), timezone.c_str());
-    this->setString(WORKDIR"/"OPTION_TIMEZONE_FILENAME,timezone.c_str());
+    this->setString(OPTION_TIMEZONE_FILENAME,timezone.c_str());
     return true;
 }
 
@@ -258,7 +270,7 @@ bool TClntIfaceIface::setSIPServerLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv,
             Log(Notice) << "Removing SIP server " << *old << " on interface "
                         << this->getName() << "/" << this->getID() << " (no longer valid)." << LogEnd;
             sipserver_del(this->getName(), this->getID(), old->getPlain());
-            this->delString(WORKDIR"/"OPTION_SIP_SERVERS_FILENAME,old->getPlain());
+            this->delString(OPTION_SIP_SERVERS_FILENAME,old->getPlain());
             this->SIPServerLst.del();
         }
     }
@@ -283,15 +295,13 @@ bool TClntIfaceIface::setSIPServerLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv,
             Log(Notice) << "Setting up SIP server " << * addr << " on interface "
                         << this->getName() << "/" << this->getID() << "." << LogEnd;
             sipserver_add(this->getName(), this->getID(), addr->getPlain());
-            this->addString(WORKDIR"/"OPTION_SIP_SERVERS_FILENAME,addr->getPlain());
+            this->addString(OPTION_SIP_SERVERS_FILENAME,addr->getPlain());
             this->SIPServerLst.append(addr);
         }
     }
-    this->SIPServerLstAddr = srv;
-    this->SIPServerLstDUID = duid;
     return true;
 }
-bool TClntIfaceIface::setSIPDomainLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, List(string) domains) {
+bool TClntIfaceIface::setSIPDomainLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, List(std::string) domains) {
     // remove old domains
     SPtr<string> old, domain;
     this->SIPDomainLst.first();
@@ -311,7 +321,7 @@ bool TClntIfaceIface::setSIPDomainLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, Lis
             Log(Notice) << "Removing SIP domain " << *old << " from interface "
                         << this->getName() << "/" << this->getID() << " (no longer valid)." << LogEnd;
             sipdomain_del(this->getName(), this->getID(), old->c_str());
-            this->delString(WORKDIR"/"OPTION_SIP_DOMAINS_FILENAME,old->c_str());
+            this->delString(OPTION_SIP_DOMAINS_FILENAME,old->c_str());
             this->SIPDomainLst.del();
         }
     }
@@ -336,19 +346,15 @@ bool TClntIfaceIface::setSIPDomainLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, Lis
             Log(Notice) << "Setting up SIP domain " << * domain << " on interface "
                         << this->getName() << "/" << this->getID() << "." << LogEnd;
             sipdomain_add(this->getName(), this->getID(), domain->c_str());
-            this->addString( WORKDIR"/"OPTION_SIP_DOMAINS_FILENAME, domain->c_str() );
+            this->addString( OPTION_SIP_DOMAINS_FILENAME, domain->c_str() );
             this->SIPDomainLst.append(domain);
         }
     }
-    this->SIPDomainLstAddr = srv;
-    this->SIPDomainLstDUID = duid;
     return true;
 }
 
-bool TClntIfaceIface::setFQDN(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, string fqdn) {
-    this->FQDN = fqdn;
-    this->FQDNDUID = duid;
-    this->FQDNAddr = srv;
+bool TClntIfaceIface::setFQDN(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, const std::string& fqdn) {
+    FQDN = fqdn;
     return true;
 }
 
@@ -372,7 +378,7 @@ bool TClntIfaceIface::setNISServerLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, Lis
             Log(Notice) << "Removing NIS server " << *old << " on interface "
                         << this->getName() << "/" << this->getID() << " (no longer valid)." << LogEnd;
             nisserver_del(this->getName(), this->getID(), old->getPlain());
-            this->delString(WORKDIR"/"OPTION_NIS_SERVERS_FILENAME,old->getPlain());
+            this->delString(OPTION_NIS_SERVERS_FILENAME,old->getPlain());
             this->NISServerLst.del();
         }
     }
@@ -397,17 +403,13 @@ bool TClntIfaceIface::setNISServerLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, Lis
             Log(Notice) << "Setting up NIS server " << * addr << " on interface "
                         << this->getName() << "/" << this->getID() << "." << LogEnd;
             nisserver_add(this->getName(), this->getID(), addr->getPlain());
-            this->addString(WORKDIR"/"OPTION_NIS_SERVERS_FILENAME, addr->getPlain());
+            this->addString(OPTION_NIS_SERVERS_FILENAME, addr->getPlain());
             this->NISServerLst.append(addr);
         }
     }
-    this->NISServerLstAddr = srv;
-    this->NISServerLstDUID = duid;
     return true;
 }
-bool TClntIfaceIface::setNISDomain(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, string domain) {
-    this->NISDomainAddr = srv;
-    this->NISDomainDUID = duid;
+bool TClntIfaceIface::setNISDomain(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, const std::string& domain) {
     if (domain==this->NISDomain) {
         // NIS Domain has not changed
         Log(Info) << "NIS Domain " << timezone << " is already set on the interface "
@@ -419,7 +421,7 @@ bool TClntIfaceIface::setNISDomain(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, string
     Log(Notice) << "Setting up NIS domain " << domain << " on the interface "
                 << this->getName() << "/" << this->getID() << "." << LogEnd;
     nisdomain_set(this->getName(), this->getID(), domain.c_str());
-    this->setString( WORKDIR"/"OPTION_NIS_DOMAIN_FILENAME, domain.c_str() );
+    this->setString( OPTION_NIS_DOMAIN_FILENAME, domain.c_str() );
     return true;
 }
 
@@ -443,7 +445,7 @@ bool TClntIfaceIface::setNISPServerLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, Li
             Log(Notice) << "Removing NIS+ server " << *old << " on interface "
                         << this->getName() << "/" << this->getID() << " (no longer valid)." << LogEnd;
             nisplusserver_del(this->getName(), this->getID(), old->getPlain());
-            this->delString(WORKDIR"/"OPTION_NISP_SERVERS_FILENAME,old->getPlain());
+            this->delString(OPTION_NISP_SERVERS_FILENAME,old->getPlain());
             this->NISPServerLst.del();
         }
     }
@@ -468,17 +470,13 @@ bool TClntIfaceIface::setNISPServerLst(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, Li
             Log(Notice) << "Setting up NIS+ server " << * addr << " on interface "
                         << this->getName() << "/" << this->getID() << "." << LogEnd;
             nisplusserver_add(this->getName(), this->getID(), addr->getPlain());
-            this->addString(WORKDIR"/"OPTION_NISP_SERVERS_FILENAME,addr->getPlain());
+            this->addString(OPTION_NISP_SERVERS_FILENAME,addr->getPlain());
             this->NISPServerLst.append(addr);
         }
     }
-    this->NISPServerLstAddr = srv;
-    this->NISPServerLstDUID = duid;
     return true;
 }
-bool TClntIfaceIface::setNISPDomain(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, string domain) {
-    this->NISPDomainAddr = srv;
-    this->NISPDomainDUID = duid;
+bool TClntIfaceIface::setNISPDomain(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, const std::string& domain) {
     if (domain==this->NISPDomain) {
         // NIS+ Domain has not changed
         Log(Info) << "NIS+ Domain " << timezone << " is already set on the interface "
@@ -490,13 +488,13 @@ bool TClntIfaceIface::setNISPDomain(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, strin
     Log(Notice) << "Setting up NIS+ domain " << domain << " on the interface "
                 << this->getName() << "/" << this->getID() << "." << LogEnd;
     nisplusdomain_set(this->getName(), this->getID(), domain.c_str());
-    this->setString(WORKDIR"/"OPTION_NISP_DOMAIN_FILENAME, domain.c_str());
+    this->setString(OPTION_NISP_DOMAIN_FILENAME, domain.c_str());
     return true;
 }
 
 bool TClntIfaceIface::setLifetime(SPtr<TDUID> duid, SPtr<TIPv6Addr> srv, unsigned int life) {
     this->LifetimeTimeout = life;
-    this->LifetimeTimestamp = now();
+    this->LifetimeTimestamp = (uint32_t)time(NULL);
     if (life == DHCPV6_INFINITY) {
         Log(Info) << "Granted options are parmanent (lifetime = INFINITY.)" << LogEnd;
         return true;
@@ -522,12 +520,11 @@ void TClntIfaceIface::addString(const char * filename, const char * str) {
 void TClntIfaceIface::delString(const char * filename, const char * str) {
     FILE *f, *fout;
     char buf[512];
-    char fileout[512];
-    snprintf(buf,511,"%s-old", filename);
     unsigned int len = (unsigned int)strlen(str);
     bool found = false;
+    string fileout(filename);
 
-    strncpy(fileout,filename,511);
+    fileout += "-old";
 
     if ( !(f=fopen(filename,"r"))) {
         Log(Debug) << "Unable to open file " << filename <<" while trying to delete " << str
@@ -535,15 +532,15 @@ void TClntIfaceIface::delString(const char * filename, const char * str) {
         return;
     }
 
-    if ( !(fout=fopen(fileout,"w"))) {
+    if ( !(fout=fopen(fileout.c_str(),"w"))) {
         Log(Debug) << "Unable to create/overwrite file " << fileout << " while trying to delete "
                    << str << " line." << LogEnd;
+        fclose(f);
         return;
     }
 
     while (!feof(f)) {
         if (!fgets(buf,511,f)) {
-            fclose(f);
             break;
         }
         if ( !found && (strlen(buf)==len) && !strncmp(str,buf,len) ) {
@@ -557,7 +554,7 @@ void TClntIfaceIface::delString(const char * filename, const char * str) {
     fclose(fout); // file with specified string cut out
 
     unlink(filename); // delete old version
-    rename(fileout,filename);
+    rename(fileout.c_str(), filename);
     return;
 }
 
@@ -680,7 +677,7 @@ SPtr<TIPv6Addr> TClntIfaceIface::getDsLiteTunnel() {
 /*
  * just prints important informations (debugging & logging)
  */
-ostream & operator <<(ostream & strum, TClntIfaceIface &x) {
+std::ostream & operator <<(std::ostream & strum, TClntIfaceIface &x) {
     char buf[48];
     SPtr<TIPv6Addr> addr;
     SPtr<string> str;
@@ -690,7 +687,14 @@ ostream & operator <<(ostream & strum, TClntIfaceIface &x) {
     strum << " name=\"" << x.Name << "\"";
     strum << " ifindex=\"" << x.ID << "\"";
     strum << " hwType=\"" << x.getHardwareType() << "\"";
-    strum << " flags=\"" << x.Flags << "\">" << endl;
+    strum << " flags=\"0x" << hex << x.Flags << dec << "\""
+          << " mBit=\"" << (x.M_bit_?"1":"0") << "\" oBit=\"" << (x.O_bit_?"1":"0")
+          << "\">" << endl;
+
+    strum << "    <!-- " << (x.flagLoopback()?"looback":"no-loopback")
+          << (x.flagRunning()?" running":" no-running")
+          << (x.flagMulticast()?" multicast -->":" no-multicast -->") << endl;
+
     strum << "    <!-- " << x.LLAddrCnt << " link scoped addrs -->" << endl;
 
     for (int i=0; i<x.LLAddrCnt; i++) {
@@ -717,11 +721,8 @@ ostream & operator <<(ostream & strum, TClntIfaceIface &x) {
     strum << "    <!-- options -->" << endl;
 
     // --- option: DNS-SERVERS ---
-    if (!x.DNSServerLstAddr || !x.DNSServerLstDUID) {
+    if (!x.DNSServerLst.count()) {
         strum << "    <!-- <dns-servers /> -->" << endl;
-    } else {
-        strum << "    <dns-servers addr=\"" << *x.DNSServerLstAddr << "\" duid=\""
-              << x.DNSServerLstDUID->getPlain() << "\" />" << endl;
     }
     x.DNSServerLst.first();
     while (addr = x.DNSServerLst.get()) {
@@ -729,11 +730,8 @@ ostream & operator <<(ostream & strum, TClntIfaceIface &x) {
     }
 
     // --- option: DOMAINS ---
-    if (!x.DomainLstAddr || !x.DomainLstDUID) {
+    if (!x.DomainLst.count()) {
         strum << "    <!-- <domains /> -->" << endl;
-    } else {
-        strum << "    <domains addr=\"" << *x.DomainLstAddr << "\" duid=\""
-              << x.DomainLstDUID->getPlain() << "\" />" << endl;
     }
     x.DomainLst.first();
     while (str = x.DomainLst.get()) {
@@ -741,11 +739,8 @@ ostream & operator <<(ostream & strum, TClntIfaceIface &x) {
     }
 
     // --- option: NTP-SERVERS ---
-    if (!x.NTPServerLstAddr || !x.NTPServerLstDUID) {
+    if (!x.NTPServerLst.count()) {
         strum << "    <!-- <ntp-servers /> -->" << endl;
-    } else {
-        strum << "    <ntp-servers addr=\"" << *x.NTPServerLstAddr << "\" duid=\""
-              << x.NTPServerLstDUID->getPlain() << "\" />" << endl;
     }
     x.NTPServerLst.first();
     while (addr = x.NTPServerLst.get()) {
@@ -756,28 +751,21 @@ ostream & operator <<(ostream & strum, TClntIfaceIface &x) {
     if (!x.Timezone.length()) {
         strum << "    <!-- <timezone /> -->" << endl;
     } else {
-        strum << "    <timezone addr=\"" << x.TimezoneAddr->getPlain() << "\" duid=\""
-              << x.TimezoneDUID->getPlain() << "\">" << x.Timezone << "</timezone>" << endl;
+        strum << "    <timezone>" << x.Timezone << "</timezone>" << endl;
     }
 
     // --- option: SIP-SERVERS ---
-    if (!x.SIPServerLstAddr || !x.SIPServerLstDUID) {
+    if (!x.SIPServerLst.count()) {
         strum << "    <!-- <sip-servers /> -->" << endl;
-    } else {
-        strum << "    <sip-servers addr=\"" << *x.SIPServerLstAddr << "\" duid=\""
-              << x.SIPServerLstDUID->getPlain() << "\" />" << endl;
-    }
+    } 
     x.SIPServerLst.first();
     while (addr = x.SIPServerLst.get()) {
         strum << "      <sip-server>" << *addr << "</sip-server>" << endl;
     }
 
     // --- option: SIP-DOMAINS ---
-    if (!x.SIPDomainLstAddr || !x.SIPDomainLstDUID) {
+    if (!x.SIPDomainLst.count()) {
         strum << "    <!-- <sip-domains /> -->" << endl;
-    } else {
-        strum << "    <sip-domains addr=\"" << *x.SIPDomainLstAddr << "\" duid=\""
-              << x.SIPDomainLstDUID->getPlain() << "\" />" << endl;
     }
     x.SIPDomainLst.first();
     while (str = x.SIPDomainLst.get()) {
@@ -785,11 +773,8 @@ ostream & operator <<(ostream & strum, TClntIfaceIface &x) {
     }
 
     // --- option: NIS-SERVERS ---
-    if (!x.NISServerLstAddr || !x.NISServerLstDUID) {
+    if (!x.NISServerLst.count()) {
         strum << "    <!-- <nis-servers /> -->" << endl;
-    } else {
-        strum << "    <nis-servers addr=\"" << *x.NISServerLstAddr << "\" duid=\""
-              << x.NISServerLstDUID->getPlain() << "\" />" << endl;
     }
     x.NISServerLst.first();
     while (addr = x.NISServerLst.get()) {
@@ -800,16 +785,12 @@ ostream & operator <<(ostream & strum, TClntIfaceIface &x) {
     if (!x.NISDomain.length()) {
         strum << "    <!-- <nis-domain /> -->" << endl;
     } else {
-        strum << "    <nis-domain add=\"" << x.NISDomainAddr->getPlain() << "\" duid=\""
-              << x.NISDomainDUID->getPlain() << "\" >" << x.NISDomain << "</nis-domain>" << endl;
+        strum << "    <nis-domain>" << x.NISDomain << "</nis-domain>" << endl;
     }
 
     // --- option: NIS+-SERVERS ---
-    if (!x.NISPServerLstAddr || !x.NISPServerLstDUID) {
+    if (!x.NISPServerLst.count()) {
         strum << "    <!-- <nisplus-servers /> -->" << endl;
-    } else {
-        strum << "    <nisplus-servers addr=\"" << *x.NISPServerLstAddr << "\" duid=\""
-              << x.NISPServerLstDUID->getPlain() << "\" />" << endl;
     }
     x.NISPServerLst.first();
     while (addr = x.NISPServerLst.get()) {
@@ -820,8 +801,7 @@ ostream & operator <<(ostream & strum, TClntIfaceIface &x) {
     if (!x.NISPDomain.length()) {
         strum << "    <!-- <nisplus-domain /> -->" << endl;
     } else {
-        strum << "    <nisplus-domain add=\"" << x.NISPDomainAddr->getPlain() << "\" duid=\""
-              << x.NISPDomainDUID->getPlain() << "\">" << x.NISPDomain << "</nisplus-domain>" << endl;
+        strum << "    <nisplus-domain>" << x.NISPDomain << "</nisplus-domain>" << endl;
     }
 
     // --- option: LIFETIME ---
