@@ -7,8 +7,6 @@
  *
  * released under GNU GPL v2 only licence
  *
- * $Id: ClntMsgConfirm.cpp,v 1.14 2009-03-24 23:17:17 thomson Exp $
- *
  */
 
 #include "SmartPtr.h"
@@ -22,7 +20,7 @@
 //  iaLst - contain all IA's to  be checked (they have to be in the same link)
 TClntMsgConfirm::TClntMsgConfirm(unsigned int iface,
                                  List(TAddrIA) iaLst)
-  :TClntMsg(iface, 0, CONFIRM_MSG) {
+    :TClntMsg(iface, SPtr<TIPv6Addr>(), CONFIRM_MSG) {
 
     IRT = CNF_TIMEOUT;
     MRT = CNF_MAX_RT;
@@ -63,7 +61,7 @@ void TClntMsgConfirm::answer(SPtr<TClntMsg> reply)
     if (!status) {
         Log(Warning) << "Received malformed REPLY for CONFIRM: no status option." << LogEnd;
         addrsRejected();
-
+	return;
     }
     switch (status->getCode() ) {
     case STATUSCODE_SUCCESS:
@@ -101,19 +99,23 @@ void TClntMsgConfirm::addrsAccepted() {
         if (!ptrIA)
             continue;
 
-        //CHANGED: ??why ts=now()-ptrIA->getT1(); So comment this line.
-        // set them to RENEW timeout
         // Uncomment this line if you don't want RENEW to be sent after
         // CONFIRM exchange is complete
-        // ptrIA->setTimestamp( now()-ptrIA->getT1() );
         ptrIA->setState(STATE_CONFIGURED);
+
+	// Once confirmed, this triggers the
+        ptrIA->setTimestamp( (uint32_t)time(NULL) - ptrIA->getT1() );
+	
+	SPtr<TIfaceIface> ptrIface = ClntIfaceMgr().getIfaceByID(ptrIA->getIfindex());
+	if (!ptrIface)
+	  continue;
+	
+	ptrIA->firstAddr();
+	while (ptrAddrAddr = ptrIA->getAddr()) {
+            ptrIface->addAddr(ptrAddrAddr->get(),ptrAddrAddr->getPref(),
+			      ptrAddrAddr->getValid(), ptrIface->getPrefixLength());
+	}
     }
-    ClntAddrMgr().firstIA();
-        SPtr<TIfaceIface> ptrIface;
-        ptrIface = ClntIfaceMgr().getIfaceByID(ptrIA->getIface());
-        ptrIA->firstAddr();
-        ptrAddrAddr = ptrIA->getAddr();
-        ptrIface->addAddr(ptrAddrAddr->get(),ptrAddrAddr->getPref(),ptrAddrAddr->getValid(),ptrIface->getPrefixLength());
 }
 
 void TClntMsgConfirm::addrsRejected() {
@@ -131,7 +133,7 @@ void TClntMsgConfirm::addrsRejected() {
 
         // release all addrs
         SPtr<TIfaceIface> ptrIface;
-        ptrIface = ClntIfaceMgr().getIfaceByID(ptrIA->getIface());
+        ptrIface = ClntIfaceMgr().getIfaceByID(ptrIA->getIfindex());
         if (!ptrIface) {
             Log(Crit) << "We have addresses assigned to non-existing interface."
                 "Help! Somebody stole an interface!" << LogEnd;
@@ -157,17 +159,16 @@ void TClntMsgConfirm::addrsRejected() {
 
 void TClntMsgConfirm::doDuties()
 {
-
     //The first Confirm message from the client on the interface MUST be
     //delayed by a random amount of time between 0 and CNF_MAX_DELAY.
-    /// @todo:
-        //if (RC == 0) ;
-                //microsleep(rand()%1000000);
+    /// @todo: MRD counters are a mess.
     if (!MRD) {
         // MRD reached. Nobody said that out addrs are faulty, so we suppose
         // they are ok. Use them
-        this->addrsAccepted();
-        this->IsDone = true;
+        Log(Info) << "MRD reached and there is no valid response for CONFIRM. "
+		  << "Assuming addresses are valid." << LogEnd;
+        addrsAccepted();
+        IsDone = true;
         return;
     }
 
@@ -184,7 +185,7 @@ bool TClntMsgConfirm::check()
         return 0;
 }
 
-string TClntMsgConfirm::getName() {
+std::string TClntMsgConfirm::getName() const {
     return "CONFIRM";
 }
 
