@@ -25,7 +25,7 @@ class TClntMsgInfRequest : public TClntMsg
     void answer(SPtr<TClntMsg> msg);
     void doDuties();    
     bool check();
-    string getName();
+    std::string getName() const;
     ~TClntMsgInfRequest();
 };
 
