@@ -26,7 +26,7 @@ class TClntMsgRelease : public TClntMsg
     void answer(SPtr<TClntMsg> Rep);
     void doDuties();
     bool check();
-    string getName();
+    std::string getName() const;
     ~TClntMsgRelease();
 };
 #endif /* CLNTMSGRELEASE_H */
