@@ -33,7 +33,7 @@ bool TClntMsgReply::check() {
     return TClntMsg::check(!anonInfReq /* clientID mandatory */, true /* serverID mandatory */ );
 }
 
-string TClntMsgReply::getName() {
+std::string TClntMsgReply::getName() const {
     return "REPLY";
 }
 
@@ -52,7 +52,7 @@ SPtr<TIPv6Addr> TClntMsgReply::getFirstAddr() {
 	}
     }
 
-    return 0;
+    return SPtr<TIPv6Addr>(); // NULL
 }
 
 TClntMsgReply::~TClntMsgReply() {
