@@ -33,7 +33,7 @@
  * opts - options list WITHOUT serverDUID
  */
 TClntMsgRequest::TClntMsgRequest(TOptList opts, int iface)
-    :TClntMsg(iface, 0, REQUEST_MSG) {
+    :TClntMsg(iface, SPtr<TIPv6Addr>(), REQUEST_MSG) {
     IRT = REQ_TIMEOUT;
     MRT = REQ_MAX_RT;
     MRC = REQ_MAX_RC;
@@ -73,7 +73,7 @@ TClntMsgRequest::TClntMsgRequest(TOptList opts, int iface)
     TOptList::iterator opt=Options.begin();
     while (opt!=Options.end())
     {
-        if ( (*opt)->getOptType() == OPTION_AAAAUTH || (*opt)->getOptType() == OPTION_ELAPSED_TIME)
+        if ( (*opt)->getOptType() == OPTION_ELAPSED_TIME)
 	          opt = Options.erase(opt);
         else {
 	          (*opt)->setParent(this);
@@ -81,18 +81,6 @@ TClntMsgRequest::TClntMsgRequest(TOptList opts, int iface)
         }
     }
 
-    // delete OPTION_KEYGEN from OPTION_ORO (needed only once)
-    /// @todo: shouldn't it be done when receiving OPTION_KEYGEN in ADVERTISE?
-    SPtr<TClntOptOptionRequest> optORO = (Ptr*) this->getOption(OPTION_ORO);
-    if (optORO) {
-        if (advertise->getOption(OPTION_KEYGEN))
-            optORO->delOption(OPTION_KEYGEN);
-        // delete also OPTION_AUTH from OPTION_ORO
-        /// @todo: shouldn't it be done when receiving OPTION_AUTH in ADVERTISE?
-        if (advertise->getOption(OPTION_AUTH))
-            optORO->delOption(OPTION_AUTH);
-    }
-
     // copy addresses offered in ADVERTISE
     copyAddrsFromAdvertise((Ptr*) advertise);
 
@@ -118,6 +106,7 @@ TClntMsgRequest::TClntMsgRequest(TOptList opts, int iface)
 	SPtr<TOpt> opt;
 	// set this unicast address in each IA in AddrMgr
 	while ( opt = getOption() ) {
+	  /// @todo: add support for unicast in IA_TA and IA_PD
 	    if (opt->getOptType()!=OPTION_IA_NA)
 		continue;
 	    SPtr<TClntOptIA_NA> ptrOptIA = (Ptr*) opt;
@@ -143,7 +132,7 @@ TClntMsgRequest::TClntMsgRequest(TOptList opts, int iface)
 
 TClntMsgRequest::TClntMsgRequest(List(TAddrIA) IAs,
 				 SPtr<TDUID> srvDUID, int iface)
-    :TClntMsg(iface, 0, REQUEST_MSG) {
+    :TClntMsg(iface, SPtr<TIPv6Addr>(), REQUEST_MSG) {
     IRT = REQ_TIMEOUT;
     MRT = REQ_MAX_RT;
     MRC = REQ_MAX_RC;
@@ -178,7 +167,6 @@ TClntMsgRequest::TClntMsgRequest(List(TAddrIA) IAs,
     appendElapsedOption();
     appendAuthenticationOption();
 
-    pkt = new char[getSize()];
     this->IsDone = false;
     this->send();
 }
@@ -205,12 +193,7 @@ void TClntMsgRequest::answer(SPtr<TClntMsg> msg)
 		continue;
 	    }
 	    SPtr<TClntOptIA_NA> IA_NA = (Ptr*)opt;
-	    IA_NA->firstOption();
-	    SPtr<TOpt> subOpt;
-	    while( subOpt = IA_NA->getOption()){
-		if(subOpt->getOptType() == OPTION_IAADDR)
-		    IA_NA->delOption();
-	    }
+            IA_NA->delOption(OPTION_IAADDR);
 	} 
 	return;   
     }
@@ -238,7 +221,7 @@ bool TClntMsgRequest::check()
     return false;
 }
 
-string TClntMsgRequest::getName() {
+std::string TClntMsgRequest::getName() const {
     return "REQUEST";
 }
 
