@@ -4,9 +4,9 @@
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>
  *          Marek Senderski <msend@o2.pl>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: ClntOptElapsed.cpp,v 1.7 2007-02-03 15:37:15 thomson Exp $
+ * $Id: ClntOptElapsed.cpp,v 1.8 2008-08-29 00:07:28 thomson Exp $
  *
  */
 
@@ -18,13 +18,13 @@
 TClntOptElapsed::TClntOptElapsed( char * buf,  int n, TMsg* parent)
     :TOptInteger(OPTION_ELAPSED_TIME, OPTION_ELAPSED_TIME_LEN, buf,n, parent)
 {
-    Timestamp = now();
+    Timestamp = (uint32_t)time(NULL);
 }
 
 TClntOptElapsed::TClntOptElapsed(TMsg* parent)
     :TOptInteger(OPTION_ELAPSED_TIME, OPTION_ELAPSED_TIME_LEN, 0, parent)
 {
-    Timestamp = now();
+    Timestamp = (uint32_t)time(NULL);
 }
 
 bool TClntOptElapsed::doDuties()
@@ -34,6 +34,6 @@ bool TClntOptElapsed::doDuties()
 
 char * TClntOptElapsed::storeSelf(char* buf)
 {
-    Value = (unsigned int)(now() - Timestamp)*100;
+    Value = (unsigned int)((uint32_t)time(NULL) - Timestamp)*100;
     return TOptInteger::storeSelf(buf);
 }
