@@ -7,13 +7,7 @@
  *
  */
 
-#ifdef WIN32
-#include <winsock2.h>
-#endif
-#ifdef LINUX
-#include <netinet/in.h>
-#endif 
-
+#include "Portable.h"
 #include "DHCPConst.h"
 #include "Opt.h"
 #include "OptIAPrefix.h"
@@ -26,7 +20,7 @@
 TClntOptIAPrefix::TClntOptIAPrefix( char * buf, int bufSize, TMsg* parent)
 	:TOptIAPrefix(buf, bufSize, parent)
 {
-    SmartPtr<TOpt> opt = 0;
+    SPtr<TOpt> opt;
     int pos=0;
     int MsgType = 0;
     if (parent)
@@ -39,10 +33,10 @@ TClntOptIAPrefix::TClntOptIAPrefix( char * buf, int bufSize, TMsg* parent)
 		       << " bytes left to parse. Bytes ignored." << LogEnd;
 	    break;
 	}
-        unsigned short code   = ntohs( *((unsigned short*) (buf+pos)));
-        pos+=2;
-        unsigned short length = ntohs( *((unsigned short*) (buf+pos)));
-        pos+=2;
+        unsigned short code   = readUint16(buf+pos);
+        pos += sizeof(uint16_t);
+        unsigned short length = readUint16(buf+pos);
+        pos+= sizeof(uint16_t);
 	if (pos+length>bufSize) {
 	    Log(Error) << "Invalid option (type=" << code << ", len=" << length 
 		       << " received (msgtype=" << MsgType << "). Message dropped." << LogEnd;
@@ -69,8 +63,9 @@ TClntOptIAPrefix::TClntOptIAPrefix( char * buf, int bufSize, TMsg* parent)
     }
 }
 
-TClntOptIAPrefix::TClntOptIAPrefix( SmartPtr<TIPv6Addr> addr, long pref, long valid, char prefixLength, TMsg* parent)
-	:TOptIAPrefix(addr,prefixLength,pref,valid, parent)
+TClntOptIAPrefix::TClntOptIAPrefix(SPtr<TIPv6Addr> addr, long pref, long valid,
+                                   char prefixLength, TMsg* parent)
+    :TOptIAPrefix(addr,prefixLength,pref,valid, parent)
 {
 
 }
@@ -80,11 +75,13 @@ bool TClntOptIAPrefix::doDuties()
     return false;
 }
 
-bool TClntOptIAPrefix::isValid()
+bool TClntOptIAPrefix::isValid() const
 {
-    if (TOptIAPrefix::isValid())
-        return this->getValid()>=this->getPref();
-    else
+    if (!TOptIAPrefix::isValid())
         return false;
 
+    if (getValid() == 0)
+        return false;
+
+    return this->getValid() >= this->getPref();
 }
