@@ -19,33 +19,32 @@
 #include "ClntAddrMgr.h"
 #include "ClntIfaceMgr.h"
 
-/** 
+/**
  * Used in CONFIRM constructor
- * 
- * @param clntAddrIA 
- * @param zeroTimes 
- * @param parent 
+ *
+ * @param clntAddrIA
+ * @param zeroTimes
+ * @param parent
  */
 TClntOptIA_NA::TClntOptIA_NA(SPtr<TAddrIA> clntAddrIA, bool zeroTimes, TMsg* parent)
     :TOptIA_NA(clntAddrIA->getIAID(), zeroTimes?0:clntAddrIA->getT1(),
-	       zeroTimes?0:clntAddrIA->getT2(), parent) 
+               zeroTimes?0:clntAddrIA->getT2(), parent), Iface_(0)
 {
     SPtr <TAddrAddr> addr;
     clntAddrIA->firstAddr();
     while ( addr = clntAddrIA->getAddr() ) {
-	SubOptions.append( new TClntOptIAAddress(addr->get(),
-						 zeroTimes?0:addr->getPref(),
-						 zeroTimes?0:addr->getValid(),
-						 parent) );
+        SubOptions.append( new TClntOptIAAddress(addr->get(),
+                                                 zeroTimes?0:addr->getPref(),
+                                                 zeroTimes?0:addr->getValid(),
+                                                 parent) );
     }
-    
 }
 
-/** 
+/**
  * Used in DECLINE, RENEW and RELEASE
- * 
- * @param addrIA 
- * @param parent 
+ *
+ * @param addrIA
+ * @param parent
  */
 TClntOptIA_NA::TClntOptIA_NA(SPtr<TAddrIA> addrIA, TMsg* parent)
     :TOptIA_NA(addrIA->getIAID(),addrIA->getT1(),addrIA->getT2(), parent)
@@ -53,34 +52,34 @@ TClntOptIA_NA::TClntOptIA_NA(SPtr<TAddrIA> addrIA, TMsg* parent)
     // should we include all addrs or tentative ones only?
     bool decline;
     if (parent->getType()==DECLINE_MSG)
-	decline = true;
+        decline = true;
     else
-	decline = false;
+        decline = false;
 
     bool zeroTimes = false;
     if ( (parent->getType()==RELEASE_MSG) || (parent->getType()==DECLINE_MSG)) {
-	this->T1 = 0;
-	this->T2 = 0;
-	zeroTimes = true;
+        T1_ = 0;
+        T2_ = 0;
+        zeroTimes = true;
     }
 
     SPtr<TAddrAddr> ptrAddr;
     addrIA->firstAddr();
     while ( ptrAddr = addrIA->getAddr() )
     {
-	if ( !decline || (ptrAddr->getTentative()==TENTATIVE_YES) )
-	    SubOptions.append(new TClntOptIAAddress(ptrAddr->get(), zeroTimes?0:ptrAddr->getPref(), 
-						    zeroTimes?0:ptrAddr->getValid(),this->Parent) );
+        if ( !decline || (ptrAddr->getTentative()==ADDRSTATUS_YES) )
+            SubOptions.append(new TClntOptIAAddress(ptrAddr->get(), zeroTimes?0:ptrAddr->getPref(),
+                                                    zeroTimes?0:ptrAddr->getValid(),this->Parent) );
     }
     DUID = SPtr<TDUID>(); // NULL
 }
 
-/** 
+/**
  * Used in REQUEST constructor
- * 
- * @param ClntCfgIA 
- * @param ClntaddrIA 
- * @param parent 
+ *
+ * @param ClntCfgIA
+ * @param ClntaddrIA
+ * @param parent
  */
 TClntOptIA_NA::TClntOptIA_NA(SPtr<TClntCfgIA> ClntCfgIA, SPtr<TAddrIA> ClntaddrIA, TMsg* parent)
     :TOptIA_NA(ClntaddrIA->getIAID(),ClntaddrIA->getT1(),ClntaddrIA->getT2(), parent)
@@ -88,8 +87,10 @@ TClntOptIA_NA::TClntOptIA_NA(SPtr<TClntCfgIA> ClntCfgIA, SPtr<TAddrIA> ClntaddrI
     //checkRequestConstructor
     ClntCfgIA->firstAddr();
     SPtr<TClntCfgAddr> ClntCfgAddr;
+
+    /// @todo: keep allocated address in TAddrClient
     while ((ClntCfgAddr = ClntCfgIA->getAddr())&&
-        ((ClntCfgIA->countAddr()-ClntaddrIA->getAddrCount())>this->countAddr() ))
+        ((ClntCfgIA->countAddr()-ClntaddrIA->countAddr())>this->countAddr() ))
     {
         SPtr<TAddrAddr> ptrAddr=ClntaddrIA->getAddr(ClntCfgAddr->get());
         if(!ptrAddr)
@@ -102,11 +103,11 @@ TClntOptIA_NA::TClntOptIA_NA(SPtr<TClntCfgIA> ClntCfgIA, SPtr<TAddrIA> ClntaddrI
     DUID = SPtr<TDUID>(); // NULL
 }
 
-/** 
+/**
  * Used in SOLICIT constructor
- * 
- * @param ClntCfgIA 
- * @param parent 
+ *
+ * @param ClntCfgIA
+ * @param parent
  */
 TClntOptIA_NA::TClntOptIA_NA(SPtr<TClntCfgIA> ClntCfgIA, TMsg* parent)
     :TOptIA_NA(ClntCfgIA->getIAID(),ClntCfgIA->getT1(),ClntCfgIA->getT2(), parent)
@@ -117,33 +118,33 @@ TClntOptIA_NA::TClntOptIA_NA(SPtr<TClntCfgIA> ClntCfgIA, TMsg* parent)
     while (ClntCfgAddr = ClntCfgIA->getAddr())
         SubOptions.append(new TClntOptIAAddress(ClntCfgAddr->get(),
         ClntCfgAddr->getPref(),
-        ClntCfgAddr->getValid(),this->Parent) ); 
+        ClntCfgAddr->getValid(),this->Parent) );
     DUID = SPtr<TDUID>(); // NULL
 }
 
-/** 
+/**
  * Used to create object from received message
- * 
- * @param buf 
- * @param bufsize 
- * @param parent 
+ *
+ * @param buf
+ * @param bufsize
+ * @param parent
  */
 TClntOptIA_NA::TClntOptIA_NA(char * buf,int bufsize, TMsg* parent)
 :TOptIA_NA(buf,bufsize, parent)
 {
     int pos=0;
-    while(pos<bufsize) 
+    while(pos<bufsize)
     {
-	if (pos+4>bufsize) {
-	    Log(Warning) << "Truncated IA_NA option received." << LogEnd;
-	    return;
-	}
+        if (pos+4>bufsize) {
+            Log(Warning) << "Truncated IA_NA option received." << LogEnd;
+            return;
+        }
         int code=buf[pos]*256+buf[pos+1];
         pos+=2;
         int length=buf[pos]*256+buf[pos+1];
         pos+=2;
         if ((code>0)&&(code<=24))
-        {                
+        {
             if(allowOptInOpt(parent->getType(),OPTION_IA_NA,code))
             {
                 switch (code)
@@ -155,20 +156,20 @@ TClntOptIA_NA::TClntOptIA_NA(char * buf,int bufsize, TMsg* parent)
                     SubOptions.append( new TClntOptStatusCode(buf+pos,length,this->Parent));
                     break;
                 default:
-		    Log(Warning) <<"Option opttype=" << code<< "not supported "
-                        <<" in field of message (type="<< parent->getType() 
+                    Log(Warning) <<"Option opttype=" << code<< "not supported "
+                        <<" in field of message (type="<< parent->getType()
                         <<") in this version of server."<<LogEnd;
                     break;
                 }
             }
             else
-		Log(Warning) << "Illegal option received, opttype=" << code 
+                Log(Warning) << "Illegal option received, opttype=" << code
                 << " in field options of IA_NA option"<<LogEnd;
         }
         else
         {
-	    Log(Warning) <<"Unknown option in option IA_NA( optType=" 
-			 << code << "). Option ignored." << LogEnd;
+            Log(Warning) <<"Unknown option in option IA_NA( optType="
+                         << code << "). Option ignored." << LogEnd;
         };
         pos+=length;
     }
@@ -197,7 +198,7 @@ int TClntOptIA_NA::countAddr()
     SubOptions.first();
     int count = 0;
     while ( ptr = SubOptions.get() ) {
-        if (ptr->getOptType() == OPTION_IAADDR) 
+        if (ptr->getOptType() == OPTION_IAADDR)
             count++;
     }
     return count;
@@ -216,21 +217,15 @@ int TClntOptIA_NA::getStatusCode()
 
 void TClntOptIA_NA::setContext(SPtr<TDUID> srvDuid, SPtr<TIPv6Addr> srvAddr, int iface)
 {
-    DUID=srvDuid;
-    if (srvAddr) {
-        Unicast = true;
-    } else {
-        Unicast = false;
-    }
+    DUID = srvDuid;
     Addr = srvAddr;
-    Iface = iface;
+    Iface_ = iface;
 }
 
 TClntOptIA_NA::~TClntOptIA_NA() {
 
 }
 
-
 bool TClntOptIA_NA::doDuties() {
 
     // find this IA in addrMgr...
@@ -246,95 +241,37 @@ bool TClntOptIA_NA::doDuties() {
     SPtr<TAddrAddr> ptrAddrAddr;
     SPtr<TClntOptIAAddress> ptrOptAddr;
 
-#if 0
-    //if not we don't like this server, cause
-    //WE WANT IA WITH AT LEAST NUMBER OF ADDRESSES
-    //and we release the whole IA with addresses just received and those in AddrMgr
-
-    //Was enough number of addresses received by client?
-    if (countValidAddrs(ptrIA) < CfgMgr->countAddrForIA(ptrIA->getIAID()))
-    {
-        //Server provided not enough addresses, so realese this IA in 
-        //this particular server (it cheated us in advertise message or
-        //or another client stole our addresses in the meantime 
-        //or the server doesn't want to prolong lease.
-
-        ptrIA->firstAddr();
-        while(SPtr<TAddrAddr> addrToRel=ptrIA->getAddr()) {
-	    SPtr<TIPv6Addr> addr2(addrToRel->get());
-            ClntIfaceMgr().getIfaceByID(ptrIA->getIface())->delAddr(addr2);
-	}
-
-        this->firstAddr();
-        while ( ptrOptAddr = this->getAddr() )
-        {
-            //add valid addresses from received IA not contained in IA 
-            //in address manager to IA in address manager
-            if(!ptrIA->getAddr(ptrOptAddr->getAddr())
-                && (ptrOptAddr->getValid()) )
-            {
-                ptrIA->addAddr(ptrOptAddr->getAddr(), 
-                    ptrOptAddr->getPref(), 
-                    ptrOptAddr->getValid()
-                    );
-            }
-            //and remove from IA in address manager those addresses, which 
-            //has expired
-            if (ptrIA->getAddr(ptrOptAddr->getAddr()) 
-                && !(ptrOptAddr->getValid()) )
-            {
-                ptrIA->delAddr(ptrOptAddr->getAddr());
-            }
-        }
-
-        //Now all addrs (new, old, expired) are in ptrIA now - those which 
-        //should be freed
-        List(TAddrIA) list;
-        list.append(ptrIA);
-
-        ClntAddrMgr().delIA(ptrIA->getIAID() );
-        ClntAddrMgr().addIA(new TAddrIA(ptrIA->getIface(), SPtr<TIPv6Addr>(), SPtr<TDUID>(),
-				   0x7fffffff,0x7fffffff,ptrIA->getIAID()));
-
-	List(TAddrIA) pdLst;
-	pdLst.clear();
-        if (ptrIA->getAddrCount())
-            TransMgr->sendRelease(list,0,pdLst);
-        return false;
-    }
-#endif /* if 0 */
-
     SPtr<TIfaceIface> ptrIface;
-    ptrIface = ClntIfaceMgr().getIfaceByID(this->Iface);
-    if (!ptrIface) 
+    ptrIface = ClntIfaceMgr().getIfaceByID(Iface_);
+    if (!ptrIface)
     {
-	Log(Error) << "Interface with ifindex=" << this->Iface << " not found." << LogEnd;
-	return true;
+        Log(Error) << "Interface with ifindex=" << Iface_ << " not found." << LogEnd;
+        return true;
     }
 
     // for each address in IA option...
     this->firstAddr();
     while (ptrOptAddr = this->getAddr() ) {
         ptrAddrAddr = ptrIA->getAddr( ptrOptAddr->getAddr() );
-        
+
         // no such address in DB ??
         if (!ptrAddrAddr) {
             if (ptrOptAddr->getValid()) {
 
-		int prefixLen = ptrIface->getPrefixLength();
-		if (ptrOptAddr->getOption(OPTION_ADDRPARAMS)) {
-		    Log(Debug) << "Experimental addr-params found." << LogEnd;
-		    SPtr<TClntOptAddrParams> optAddrParams = (Ptr*) ptrOptAddr->getOption(OPTION_ADDRPARAMS);
-		    prefixLen = optAddrParams->getPrefix();
-		}
+                int prefixLen = ptrIface->getPrefixLength();
+                if (ptrOptAddr->getOption(OPTION_ADDRPARAMS)) {
+                    Log(Debug) << "Experimental addr-params found." << LogEnd;
+                    SPtr<TClntOptAddrParams> optAddrParams = (Ptr*) ptrOptAddr->getOption(OPTION_ADDRPARAMS);
+                    prefixLen = optAddrParams->getPrefix();
+                }
 
                 // add this address in addrDB...
                 ptrIA->addAddr(ptrOptAddr->getAddr(), ptrOptAddr->getPref(), ptrOptAddr->getValid(), prefixLen);
                 ptrIA->setDUID(this->DUID);
 
-                // ... and in IfaceMgr - 
+                // ... and in IfaceMgr -
                 ptrIface->addAddr(ptrOptAddr->getAddr(), ptrOptAddr->getPref(), ptrOptAddr->getValid(), prefixLen);
-            } 
+            }
             else {
                 Log(Warning) << "Server send new addr with valid lifetime 0." << LogEnd;
             }
@@ -350,10 +287,10 @@ bool TClntOptIA_NA::doDuties() {
             }
 
             // set up new options in IfaceMgr
-            SPtr<TIfaceIface> ptrIface = ClntIfaceMgr().getIfaceByID(this->Iface);
+            SPtr<TIfaceIface> ptrIface = ClntIfaceMgr().getIfaceByID(Iface_);
             if (ptrIface)
-                ptrIface->updateAddr(ptrOptAddr->getAddr(), 
-                        ptrOptAddr->getPref(), 
+                ptrIface->updateAddr(ptrOptAddr->getAddr(),
+                        ptrOptAddr->getPref(),
                         ptrOptAddr->getValid());
             // set up new options in addrDB
             ptrAddrAddr->setPref(ptrOptAddr->getPref());
@@ -365,25 +302,27 @@ bool TClntOptIA_NA::doDuties() {
     ptrCfgIA=ClntCfgMgr().getIA(ptrIA->getIAID());
 
     if (getT1() && getT2()) {
-      ptrIA->setT1( this->getT1() );
-      ptrIA->setT2( this->getT2() );
-      Log(Debug) << "RENEW will be sent (T1) after " << ptrIA->getT1() << ", REBIND (T2) after " << ptrIA->getT2() << " seconds." << LogEnd;
+        ptrIA->setT1( this->getT1() );
+        ptrIA->setT2( this->getT2() );
+        Log(Debug) << "RENEW(IA_NA) will be sent (T1) after " << ptrIA->getT1()
+                   << ", REBIND (T2) after " << ptrIA->getT2() << " seconds." << LogEnd;
     } else {
-      this->firstAddr();
-      ptrOptAddr = this->getAddr();
-      if (ptrOptAddr) {
-	    ptrIA->setT1( ptrOptAddr->getPref()/2);
-	    ptrIA->setT2( (int)((ptrOptAddr->getPref())*0.7) );
-	    Log(Notice) << "Server set T1 and T2 to 0. Choosing default (50%, 70% * prefered-lifetime): T1=" << ptrIA->getT1() 
-                    << ", T2=" << ptrIA->getT2() << LogEnd;
-      }
+        this->firstAddr();
+        ptrOptAddr = this->getAddr();
+        if (ptrOptAddr) {
+            ptrIA->setT1( ptrOptAddr->getPref()/2);
+            ptrIA->setT2( (int)((ptrOptAddr->getPref())*0.7) );
+            Log(Notice) << "Server set T1 and T2 to 0. Choosing default (50%, "
+                        << "70% * prefered-lifetime): T1=" << ptrIA->getT1()
+                        << ", T2=" << ptrIA->getT2() << LogEnd;
+        }
     }
 
     ptrIA->setTimestamp();
     ptrIA->setState(STATE_CONFIGURED);
     ptrCfgIA->setState(STATE_CONFIGURED);
     return true;
-} 
+}
 
 //Counts all valid and diffrent addresses in sum of
 //addresses received in IA from server and addresses contained
@@ -407,12 +346,12 @@ int TClntOptIA_NA::countValidAddrs(SPtr<TAddrIA> ptrIA)
             //we can increase counter
             count++;
     }
-    
+
     //count all valid (valid>0) addresses received from server
     /// @todo: A) check if they repeats (possible with maliciious server)
     //       B) and not already assigned in addrDB (in others IAs)
     //       It's easy and worth checking
-    //       A) Create list of valid addrs and add new valid addr 
+    //       A) Create list of valid addrs and add new valid addr
     //          only if it doesn't exist already on this list
     //       B) Address can be assigned only in this IA, not in other
     //          This could be ommited if VerifyIA worked prooperly
@@ -433,7 +372,7 @@ SPtr<TClntOptIAAddress> TClntOptIA_NA::getAddr(SPtr<TIPv6Addr> addr)
     SPtr<TClntOptIAAddress> optAddr;
     this->firstAddr();
     while(optAddr=this->getAddr())
-    {   
+    {
         //!memcmp(optAddr->getAddr(),addr,16)
         if ((*addr)==(*optAddr->getAddr()))
             return optAddr;
@@ -447,30 +386,41 @@ void TClntOptIA_NA::releaseAddr(long IAID, SPtr<TIPv6Addr> addr )
     if (ptrIA)
         ptrIA->delAddr(addr);
     else
-	Log(Warning) << "Unable to release addr: IA (" 
-		     << IAID << ") not present in addrDB." << LogEnd;
+        Log(Warning) << "Unable to release addr: IA ("
+                     << IAID << ") not present in addrDB." << LogEnd;
 }
 
 void TClntOptIA_NA::setIface(int iface) {
-    this->Iface    = iface;
+    Iface_ = iface;
 }
 
+bool TClntOptIA_NA::isValid() const {
 
-bool TClntOptIA_NA::isValid()
-{
-    SPtr<TClntOptIAAddress> addr;
-    this->firstAddr();
-    while (addr = this->getAddr()) {
-	if (addr->getAddr()->linkLocal()) {
-	    Log(Warning) << "Address " << addr->getAddr()->getPlain() << " used in IA (IAID=" 
-			 << this->IAID << ") is link local. The whole IA option is considered invalid."
-			 << LogEnd;
-	    return false;
-	}
+    const TOptList& opts = SubOptions.getSTL();
+
+    for (TOptList::const_iterator it = opts.begin(); it != opts.end(); ++it) {
+        if ((*it)->getOptType() != OPTION_IAADDR)
+            continue;
+        const TOptIAAddress* addr = (const TOptIAAddress*)it->get();
+
+        if (addr->getAddr()->linkLocal()) {
+            Log(Warning) << "Address " << addr->getAddr()->getPlain() << " used in IA_NA (IAID="
+                         << IAID_ << ") is link local. The whole IA option is considered invalid."
+                         << LogEnd;
+            return false;
+        }
     }
 
-    if (Valid)
-        return this->T1<=this->T2;
-    else
+    // RFC3315, section 22.4:
+    // If a client receives an IA_NA with T1 greater than T2, and both T1
+    // and T2 are greater than 0, the client discards the IA_NA option and
+    // processes the remainder of the message as though the server had not
+    // included the invalid IA_NA option.
+    if (T1_ > T2_) {
+        Log(Warning) << "Received malformed IA_NA: T1(" << T1_ << ") is greater than T2("
+                     << T2_ << "). Ignoring IA_NA." << LogEnd;
         return false;
+    }
+
+    return Valid;
 }
