@@ -6,14 +6,15 @@
  *
  * released under GNU GPL v2 only licence
  *
- * $Id: ClntOptLifetime.cpp,v 1.6 2008-08-29 00:07:29 thomson Exp $
- *
  */
 #include "DHCPConst.h"
 #include "ClntOptLifetime.h"
+#include "OptDUID.h"
 #include "ClntMsg.h"
 #include "Logger.h"
 
+using namespace std;
+
 TClntOptLifetime::TClntOptLifetime(char * buf,  int n, TMsg* parent)
 	:TOptInteger(OPTION_INFORMATION_REFRESH_TIME, OPTION_INFORMATION_REFRESH_TIME_LEN, buf,n, parent){
 
@@ -26,13 +27,21 @@ TClntOptLifetime::TClntOptLifetime( char pref, TMsg* parent)
 bool TClntOptLifetime::doDuties()
 {
     string reason = "trying to set Lifetime timer.";
-    int ifindex = this->Parent->getIface();
 
-    if (!DUID) {
-        Log(Error) << "Unable to find proper DUID while " << reason << LogEnd;
+    if (!Parent) {
+        Log(Error) << "Unable to set lifetime: option parent not set." << LogEnd;
         return false;
     }
 
+    int ifindex = Parent->getIface();
+
+    SPtr<TOptDUID> duid = (Ptr*)Parent->getOption(OPTION_SERVERID);
+
+    if (!duid) {
+	Log(Error) << "Unable to find proper DUID while " << reason << LogEnd;
+	return false;
+    }
+
     SPtr<TClntIfaceIface> iface = (Ptr*)ClntIfaceMgr().getIfaceByID(ifindex);
     if (!iface) {
         Log(Error) << "Unable to find interface ifindex=" << ifindex
@@ -41,5 +50,5 @@ bool TClntOptLifetime::doDuties()
     }
 
     SPtr<TClntCfgIface> cfgIface = ClntCfgMgr().getIface(ifindex);
-    return iface->setLifetime(DUID, Parent->getAddr(), Value);
+    return iface->setLifetime(duid->getDUID(), Parent->getRemoteAddr(), Value);
 }
