@@ -18,44 +18,34 @@ class TMsg;
 
 #include "OptTA.h"
 #include "ClntOptIAAddress.h"
-#include "ClntIfaceMgr.h"
-#include "ClntTransMgr.h"
-#include "ClntCfgMgr.h"
-#include "ClntAddrMgr.h"
 #include "IPv6Addr.h"
-#include "ClntCommon.h"
+#include "Msg.h"
 
 class TClntOptTA : public TOptTA
 {
  public:
     TClntOptTA(unsigned int iaid, TMsg* parent);
     TClntOptTA(char * buf, int bufsize, TMsg* parent);
-    TClntOptTA(SmartPtr<TAddrIA> ta, TMsg* parent);
+    TClntOptTA(SPtr<TAddrIA> ta, TMsg* parent);
     ~TClntOptTA();    
 
     bool doDuties();
     int getStatusCode();
 
-    SmartPtr<TClntOptIAAddress> getAddr();
-    SmartPtr<TClntOptIAAddress> getAddr(SmartPtr<TIPv6Addr> addr);
+    SPtr<TClntOptIAAddress> getAddr();
+    SPtr<TClntOptIAAddress> getAddr(SPtr<TIPv6Addr> addr);
     void firstAddr();
     int countAddr();
-    bool isValid();
+    bool isValid() const;
     void setIface(int iface); // used to override interface (e.g. when msg is received via loopback)
 
-    void setContext(SmartPtr<TClntAddrMgr> addrMgr, SmartPtr<TClntIfaceMgr> ifaceMgr,
-		    SmartPtr<TClntCfgMgr> cfgMgr, int iface, SmartPtr<TIPv6Addr> clntAddr);
+    void setContext(int iface, SPtr<TIPv6Addr> clntAddr);
 
  private:
-    void releaseAddr(long IAID, SmartPtr<TIPv6Addr> addr );
+    void releaseAddr(long IAID, SPtr<TIPv6Addr> addr );
 
-    SmartPtr<TIPv6Addr> Addr;
+    SPtr<TIPv6Addr> Addr;
     int Iface;
-
-    TCtx * Ctx;
-    SmartPtr<TClntIfaceMgr> IfaceMgr;
-    SmartPtr<TClntCfgMgr>   CfgMgr;
-    SmartPtr<TClntAddrMgr>  AddrMgr;
 };
 
 
