@@ -8,10 +8,11 @@
  */
 
 #include "ClntOptTimeZone.h"
+#include "OptDUID.h"
 #include "ClntMsg.h"
 #include "Logger.h"
 
-TClntOptTimeZone::TClntOptTimeZone(string domain, TMsg* parent)
+TClntOptTimeZone::TClntOptTimeZone(const std::string& domain, TMsg* parent)
     :TOptString(OPTION_NEW_TZDB_TIMEZONE, domain, parent) {
 
 }
@@ -22,7 +23,7 @@ TClntOptTimeZone::TClntOptTimeZone(char *buf, int bufsize, TMsg* parent)
 
 }
 
-bool TClntOptTimeZone::isValid() {
+bool TClntOptTimeZone::isValid() const {
     /// @todo: check is somehow
     return true;
 }
@@ -31,9 +32,10 @@ bool TClntOptTimeZone::doDuties() {
     std::string reason = "trying to set time zone.";
     int ifindex = Parent->getIface();
 
-    if (!this->DUID) {
-        Log(Error) << "Unable to find proper DUID while " << reason << LogEnd;
-        return false;
+    SPtr<TOptDUID> duid = (Ptr*)Parent->getOption(OPTION_SERVERID);
+    if (!duid) {
+	Log(Error) << "Unable to find proper DUID while " << reason << LogEnd;
+	return false;
     }
 
     SPtr<TClntIfaceIface> iface = (Ptr*)ClntIfaceMgr().getIfaceByID(ifindex);
@@ -46,7 +48,7 @@ bool TClntOptTimeZone::doDuties() {
     SPtr<TClntCfgIface> cfgIface = ClntCfgMgr().getIface(ifindex);
     cfgIface->setTimezoneState(STATE_CONFIGURED);
 
-    return iface->setTimezone(DUID, Parent->getAddr(), Str);
+    return iface->setTimezone(duid->getDUID(), Parent->getRemoteAddr(), Str);
 }
 
 /// @todo remove this
