@@ -24,11 +24,11 @@
 class TClntOptTimeZone : public TOptString
 {
  public:
-    TClntOptTimeZone(string domain, TMsg* parent);
+    TClntOptTimeZone(const std::string& domain, TMsg* parent);
     TClntOptTimeZone(char *buf, int bufsize, TMsg* parent);
     bool doDuties();
     void setSrvDuid(SPtr<TDUID> duid);
-    bool isValid();
+    bool isValid() const;
  private:
     SPtr<TDUID> SrvDUID;
 };
