@@ -27,15 +27,18 @@
 #include "ClntMsgInfRequest.h"
 #include "ClntMsgDecline.h"
 #include "ClntMsgConfirm.h"
+#include "ClntMsgReconfigure.h"
 #include "Container.h"
 #include "DHCPConst.h"
 #include "Logger.h"
 
+using namespace std;
+
 TClntTransMgr * TClntTransMgr::Instance = 0;
 
-void TClntTransMgr::instanceCreate(const std::string config)
+void TClntTransMgr::instanceCreate(const std::string& config)
 {
-        if (Instance) {
+    if (Instance) {
         Log(Crit) << "ClntTransMgr instance already exists. Internal code error!" << LogEnd;
         return;
     }
@@ -44,13 +47,15 @@ void TClntTransMgr::instanceCreate(const std::string config)
 
 TClntTransMgr &TClntTransMgr::instance()
 {
-  if (!Instance)
-      Log(Crit) << "Error: ClntTransMgr not initialized. Crashing in 3... 2... 1..." << LogEnd;
-  return *Instance;
+    if (!Instance) {
+        Log(Crit) << "Error: ClntTransMgr not initialized. Trying to recover." << LogEnd;
+        instanceCreate(CLNTTRANSMGR_FILE);
+    }
+    return *Instance;
 }
 
-TClntTransMgr::TClntTransMgr(const std::string config)
-  :IsDone(true), Shutdown(true)
+TClntTransMgr::TClntTransMgr(const std::string& config)
+  :IsDone(true), Shutdown(true), CtrlIface_(-1)
 {
     // should we set REUSE option during binding sockets?
 #ifdef MOD_CLNT_BIND_REUSE
@@ -94,9 +99,9 @@ bool TClntTransMgr::populateAddrMgr(SPtr<TClntCfgIface> iface)
     while(ia = iface->getIA()) {
         if (ClntAddrMgr().getIA(ia->getIAID()))
             continue; // there is such IA already - read from disk cache (client-AddrMgr.xml)
-        SPtr<TAddrIA> addrIA = new TAddrIA(iface->getID(), TAddrIA::TYPE_IA,
-                                           0, 0, ia->getT1(), ia->getT2(),
-                                           ia->getIAID());
+        SPtr<TAddrIA> addrIA = new TAddrIA(iface->getName(), iface->getID(), IATYPE_IA,
+                                           SPtr<TIPv6Addr>(), SPtr<TDUID>(),
+                                           ia->getT1(), ia->getT2(), ia->getIAID());
         ClntAddrMgr().addIA(addrIA);
     }
 
@@ -105,8 +110,9 @@ bool TClntTransMgr::populateAddrMgr(SPtr<TClntCfgIface> iface)
     if ( (ta = iface->getTA()) &&  (!ClntAddrMgr().getTA(ta->getIAID())))
     {
         // if there is such TA already, then skip adding it
-        SPtr<TAddrIA> addrTA = new TAddrIA(iface->getID(), TAddrIA::TYPE_TA,
-                                           0, 0, DHCPV6_INFINITY, DHCPV6_INFINITY,
+        SPtr<TAddrIA> addrTA = new TAddrIA(iface->getName(), iface->getID(), IATYPE_TA,
+                                           SPtr<TIPv6Addr>(), SPtr<TDUID>(),
+                                           DHCPV6_INFINITY, DHCPV6_INFINITY,
                                            ta->getIAID());
         ClntAddrMgr().addTA(addrTA);
     }
@@ -116,8 +122,9 @@ bool TClntTransMgr::populateAddrMgr(SPtr<TClntCfgIface> iface)
     while (pd = iface->getPD()) {
         if (ClntAddrMgr().getPD(pd->getIAID()))
             continue; // there is such IA already - read from disk cache (client-AddrMgr.xml)
-        SPtr<TAddrIA> addrPD = new TAddrIA(iface->getID(), TAddrIA::TYPE_PD,
-                                           0, 0, pd->getT1(), pd->getT2(),
+        SPtr<TAddrIA> addrPD = new TAddrIA(iface->getName(), iface->getID(), IATYPE_PD,
+                                           SPtr<TIPv6Addr>(), SPtr<TDUID>(),
+                                           pd->getT1(), pd->getT2(),
                                            pd->getIAID());
         ClntAddrMgr().addPD(addrPD);
     }
@@ -158,8 +165,16 @@ bool TClntTransMgr::openSockets(SPtr<TClntCfgIface> iface) {
         return false;
     }
 
+    // By default, we use the first link-local address on the interface
     SPtr<TIPv6Addr> addr = new TIPv6Addr(llAddr);
 
+    // However, if the user specified bind-to address, we'll use that instead.
+    if (iface->getBindToAddr()) {
+        addr = iface->getBindToAddr();
+        Log(Debug) << "Using bind-to address " << addr->getPlain() << " on interface "
+                   << iface->getFullName() << LogEnd;
+    }
+
     // it's very important to open unicast socket first as it will be used for
     // unicast communication
     if (iface->getUnicast()) {
@@ -183,11 +198,10 @@ bool TClntTransMgr::openSockets(SPtr<TClntCfgIface> iface) {
     }
 
     if (llAddr) {
-            char buf[48];
-            inet_ntop6(llAddr,buf);
-            // Log(Info) << "Socket bound to " << buf << "/port=" << DHCPCLIENT_PORT << LogEnd;
-            this->ctrlIface = realIface->getID();
-            strncpy(this->ctrlAddr,buf,48);
+        char buf[48];
+        inet_ntop6(llAddr,buf);
+        CtrlIface_ = realIface->getID();
+        strncpy(CtrlAddr_, buf, 48);
     } 
 
     return true;
@@ -198,6 +212,9 @@ bool TClntTransMgr::openSockets(SPtr<TClntCfgIface> iface) {
  */
 void TClntTransMgr::removeExpired() {
 
+    /// @todo: call notify-script when address/prefix is expired
+    TNotifyScriptParams params;
+
     if (ClntAddrMgr().getValidTimeout())
         return;
 
@@ -205,6 +222,7 @@ void TClntTransMgr::removeExpired() {
     SPtr<TAddrAddr> ptrAddr;
     SPtr<TIfaceIface> ptrIface;
 
+    // are there any expired IA_NAs?
     ClntAddrMgr().firstIA();
     while (ptrIA = ClntAddrMgr().getIA()) {
         if (ptrIA->getValidTimeout())
@@ -214,19 +232,127 @@ void TClntTransMgr::removeExpired() {
         while (ptrAddr = ptrIA->getAddr()) {
             if (ptrAddr->getValidTimeout())
                 continue;
-            ptrIface = ClntIfaceMgr().getIfaceByID(ptrIA->getIface());
+            ptrIface = ClntIfaceMgr().getIfaceByID(ptrIA->getIfindex());
             Log(Warning) << "Address " << ptrAddr->get()->getPlain() << " assigned to the "
-                         << ptrIface->getName() << "/" << ptrIface->getID() 
-                         << " interface (in IA " << ptrIA->getIAID() <<") has expired." << LogEnd;
+                         << ptrIface->getFullName()
+                         << " interface (in IA_NA " << ptrIA->getIAID() <<") has expired." << LogEnd;
 
             // remove that address from the physical interace
             ptrIface->delAddr(ptrAddr->get(), ptrIface->getPrefixLength());
+
+	    // Note: Technically, removing address here is not needed, as it will
+	    // be removed in AddrMgr::doDuties() anyway
+	    ptrIA->delAddr(ptrAddr->get());
+	    Log(Info) << "Expired address " << ptrAddr->get()->getPlain()
+		      << " from IA " << ptrIA->getIAID()
+		      << " has been removed from addrDB." << LogEnd;
         }
+
+	// if there are no more addresses in this IA, declare it freed
+	if (!ptrIA->countAddr()) {
+	    Log(Debug) << "The IA_NA (with IAID=" << ptrIA->getIAID() << ") has expired. " << LogEnd;
+	    SPtr<TClntCfgIface> cfgIface = ClntCfgMgr().getIface(ptrIA->getIfindex());
+	    if (cfgIface) {
+		SPtr<TClntCfgIA> cfgIA = cfgIface->getIA(ptrIA->getIAID());
+		if (cfgIA) {
+		    cfgIA->setState(STATE_NOTCONFIGURED);
+		}
+	    } else {
+		// something is terribly wrong here
+	    }
+	}
     }
+
+    // are there any expired IA_TAs?
+    ClntAddrMgr().firstTA();
+    while (ptrIA = ClntAddrMgr().getTA()) {
+        if (ptrIA->getValidTimeout())
+            continue;
+
+        ptrIA->firstAddr();
+        while (ptrAddr = ptrIA->getAddr()) {
+            if (ptrAddr->getValidTimeout())
+                continue;
+            ptrIface = ClntIfaceMgr().getIfaceByID(ptrIA->getIfindex());
+            Log(Warning) << "Temporary address " << ptrAddr->get()->getPlain() << " assigned to the "
+                         << ptrIface->getFullName()
+                         << " interface (in IA_TA " << ptrIA->getIAID() <<") has expired." << LogEnd;
+
+            // remove that address from the physical interace
+            ptrIface->delAddr(ptrAddr->get(), ptrIface->getPrefixLength());
+
+	    // Note: Technically, removing address here is not needed, as it will
+	    // be removed in AddrMgr::doDuties() anyway
+	    ptrIA->delAddr(ptrAddr->get());
+	    Log(Info) << "Expired temporary address " << ptrAddr->get()->getPlain()
+		      << " from IA " << ptrIA->getIAID()
+		      << " has been removed from addrDB." << LogEnd;
+        }
+
+	// if there are no more addresses in this IA, declare it freed
+	if (!ptrIA->countAddr()) {
+	    Log(Debug) << "The IA_TA (with IAID=" << ptrIA->getIAID() << ") has expired. " << LogEnd;
+	    SPtr<TClntCfgIface> cfgIface = ClntCfgMgr().getIface(ptrIA->getIfindex());
+	    if (cfgIface) {
+                cfgIface->firstTA();
+		SPtr<TClntCfgTA> cfgTA = cfgIface->getTA();
+		if (cfgTA) {
+		    cfgTA->setState(STATE_NOTCONFIGURED);
+		}
+	    } else {
+		// something is terribly wrong here
+	    }
+	}
+    }
+
+    // are there any expired IA_PDs?
+    SPtr<TAddrIA> ptrPD;
+    SPtr<TAddrPrefix> ptrPrefix;
+    ClntAddrMgr().firstPD();
+    while (ptrPD = ClntAddrMgr().getPD()) {
+        if (ptrPD->getValidTimeout())
+            continue;
+
+        ptrPD->firstPrefix();
+        while (ptrPrefix = ptrPD->getPrefix()) {
+            if (ptrPrefix->getValidTimeout())
+                continue;
+            ptrIface = ClntIfaceMgr().getIfaceByID(ptrPD->getIfindex());
+            Log(Warning) << "Prefix " << ptrPrefix->get()->getPlain() << " obtained on the "
+                         << ptrIface->getFullName()
+                         << " interface (in IA_PD " << ptrPD->getIAID() <<") has expired." << LogEnd;
+
+            // remove that address from the physical interace
+	    ClntIfaceMgr().delPrefix(ptrPD->getIfindex(), ptrPrefix->get(), ptrPrefix->getLength(),
+                &params);
+
+	    // Note: Technically, removing address here is not needed, as it will
+	    // be removed in AddrMgr::doDuties() anyway
+	    ptrPD->delPrefix(ptrPrefix);
+	    Log(Info) << "Expired prefix " << ptrPrefix->get()->getPlain() << "/" << ptrPrefix->getLength()
+		      << " from IA_PD " << ptrPD->getIAID()
+		      << " has been removed from addrDB." << LogEnd;
+        }
+
+	// if there are no more addresses in this IA, declare it freed
+	if (!ptrPD->countPrefix()) {
+	    Log(Debug) << "The IA_PD (with IAID=" << ptrPD->getIAID() << ") has expired. " << LogEnd;
+	    SPtr<TClntCfgIface> cfgIface = ClntCfgMgr().getIface(ptrPD->getIfindex());
+	    if (cfgIface) {
+		SPtr<TClntCfgPD> cfgPD = cfgIface->getPD(ptrPD->getIAID());
+		if (cfgPD) {
+		    cfgPD->setState(STATE_NOTCONFIGURED);
+		}
+	    } else {
+		// something is terribly wrong here
+	    }
+	}
+    }
+
 }
 
 /** 
- * checks if loaded Address database is sanite (i.e. does not reffer to non-existing interface)
+ * checks if loaded Address database is sane (i.e. does not reffer to non-existing interface)
  * 
  */
 void TClntTransMgr::checkDB()
@@ -238,11 +364,11 @@ void TClntTransMgr::checkDB()
     ClntAddrMgr().doDuties();
     ClntAddrMgr().firstIA();
     while ( ptrIA = ClntAddrMgr().getIA()) {
-        iface = ClntCfgMgr().getIface( ptrIA->getIface() );
+        iface = ClntCfgMgr().getIface( ptrIA->getIfindex() );
         if (!iface) {
             Log(Warning) << "IA (iaid=" << ptrIA->getIAID() 
                          << ") loaded from old file, but currently there is no iface with ifindex="
-                                 << ptrIA->getIface();
+                                 << ptrIA->getIfindex();
             // IA with non-existent iface, purge iface
             ptrIA->firstAddr();
             while (ptrAddr = ptrIA->getAddr())
@@ -322,8 +448,12 @@ void TClntTransMgr::doDuties()
         SPtr<TClntCfgIface> x;
         x = ClntCfgMgr().checkInactiveIfaces();
         if (x) {
+            if (!populateAddrMgr(x)) {
+                Log(Error)<< "Call to populateAddrMgr() ended with error"
+                          << " Following operation may be unstable!" << LogEnd;
+            }
             if (!openSockets(x)) {
-                Log(Crit) << "Attempt to bind activates interfaces failed."
+                Log(Crit) << "Attempt to bind activated interfaces failed."
                           << " Following operation may be unstable!" << LogEnd;
             }
         }
@@ -407,7 +537,7 @@ void TClntTransMgr::shutdown()
         // find similar IAs 
         while (ptrNextIA = ClntAddrMgr().getIA()) {
             if ((*(ptrFirstIA->getDUID())==*(ptrNextIA->getDUID())) &&
-                (ptrFirstIA->getIface() == ptrNextIA->getIface() ) ) 
+                (ptrFirstIA->getIfindex() == ptrNextIA->getIfindex() ) ) 
             {
                 // IA serviced via this same server, add it do the list
                 releasedIAs.append(ptrNextIA);
@@ -415,9 +545,9 @@ void TClntTransMgr::shutdown()
                 // delete addressess from IfaceMgr
                 SPtr<TAddrAddr> ptrAddr;
                 SPtr<TIfaceIface> ptrIface;
-                ptrIface = ClntIfaceMgr().getIfaceByID(ptrNextIA->getIface());
+                ptrIface = ClntIfaceMgr().getIfaceByID(ptrNextIA->getIfindex());
                 if (!ptrIface) {
-                            Log(Error) << "Unable to find " << ptrNextIA->getIface()
+                            Log(Error) << "Unable to find " << ptrNextIA->getIfindex()
                                << " interface while releasing address." << LogEnd;
                     break;
                 }
@@ -428,28 +558,28 @@ void TClntTransMgr::shutdown()
             }
         }
 
-        ta = 0;
+        ta.reset();
         if (releasedIAs.count()) { 
-                // check if there are TA to release
-                releasedIAs.first();
-                iface = ClntCfgMgr().getIface(releasedIAs.get()->getIface());
-                if (iface && iface->countTA()) {
-                    iface->firstTA();
-                    SPtr<TClntCfgTA> cfgTA = iface->getTA();
-                        ta = ClntAddrMgr().getTA(cfgTA->getIAID());
-                        cfgTA->setState(STATE_DISABLED);
-                }
-
-        }
-            pd = 0;
-            ClntAddrMgr().firstPD();
-            while (pd = ClntAddrMgr().getPD()) {
-                releasedPDs.append(pd);
-                SPtr<TClntCfgPD> cfgPD = ClntCfgMgr().getPD(pd->getIAID());
-                if (cfgPD)
-                    cfgPD->setState(STATE_DISABLED);
+            // check if there are TA to release
+            releasedIAs.first();
+            iface = ClntCfgMgr().getIface(releasedIAs.get()->getIfindex());
+            if (iface && iface->countTA()) {
+                iface->firstTA();
+                SPtr<TClntCfgTA> cfgTA = iface->getTA();
+                ta = ClntAddrMgr().getTA(cfgTA->getIAID());
+                cfgTA->setState(STATE_DISABLED);
             }
-            sendRelease(releasedIAs,ta, releasedPDs);
+        }
+
+        pd.reset();
+        ClntAddrMgr().firstPD();
+        while (pd = ClntAddrMgr().getPD()) {
+            releasedPDs.append(pd);
+            SPtr<TClntCfgPD> cfgPD = ClntCfgMgr().getPD(pd->getIAID());
+            if (cfgPD)
+                cfgPD->setState(STATE_DISABLED);
+        }
+        sendRelease(releasedIAs, ta, releasedPDs);
     }
 
     // now check if there are any TA left
@@ -473,7 +603,7 @@ void TClntTransMgr::shutdown()
     }
 
     // are there any PDs left to release?
-    pd = 0;
+    pd.reset();
     ClntAddrMgr().firstPD();
     releasedIAs.clear();
     releasedPDs.clear();
@@ -484,7 +614,7 @@ void TClntTransMgr::shutdown()
             cfgPD->setState(STATE_DISABLED);
     }
     if (releasedPDs.count())
-        this->sendRelease(releasedIAs, 0, releasedPDs);
+        this->sendRelease(releasedIAs, SPtr<TAddrIA>(), releasedPDs);
     
     //CHANGED:the following two lines are uncommented.
     doDuties(); // just to send RELEASE msg
@@ -503,6 +633,11 @@ void TClntTransMgr::relayMsg(SPtr<TClntMsg> msgAnswer)
     if (!msgAnswer->check())
         return ;
 
+    if (msgAnswer->getType() == RECONFIGURE_MSG) {
+        handleReconfigure(msgAnswer);
+        return;
+    }
+
 #ifdef MOD_REMOTE_AUTOCONF
     if (neighborInfoGet(msgAnswer->getTransID())) {
         processRemoteReply(msgAnswer);
@@ -546,6 +681,29 @@ void TClntTransMgr::relayMsg(SPtr<TClntMsg> msgAnswer)
     ClntAddrMgr().dump();
 }
 
+/// processes received RECONFIGURE message
+///
+/// Verifies that received message is valid. Depending on received option, it will
+/// send RENEW, INF-REQUEST or REBIND message
+///
+/// @param reconfMsg pointer to received reconfigure message
+///
+void TClntTransMgr::handleReconfigure(SPtr<TClntMsg> reconfMsg) {
+    // see if there is reconfigure-msg option. If not, drop message.
+    // if yes, send specific message, e.g. call sendRenew(), sendRebind() or sendInfRequest()
+
+    if(!ClntCfgMgr().getReconfigure()) {
+        Log(Notice) << "Client is not configured to support reconfigure message." << LogEnd;
+        return;
+    }
+
+    /// @todo: check authentication here
+
+    /// @todo: server may tell client to send, RENEW, REBIND or INF-REQUEST
+    Log(Notice) << "Received RECONFIGURE, sending RENEW." << LogEnd;
+    sendRenew();
+}
+
 /** 
  * handle response
  * 
@@ -587,11 +745,19 @@ unsigned long TClntTransMgr::getTimeout()
     tmp     = ClntAddrMgr().getTentativeTimeout();
     if (timeout > tmp)
         timeout = tmp;
+    // Uncomment for timeout debugging
+    //Log(Debug) << "Timeout after AddrMgr=" << timeout << LogEnd;
+
+    if (timeout == 0) {
+	ClntAddrMgr().getTimeout();
+    }
 
     // IfaceMgr (Lifetime option) timeout
     tmp = ClntIfaceMgr().getTimeout();
     if (timeout > tmp)
         timeout = tmp;
+    // Uncomment for timeout debugging
+    // Log(Debug) << "Timeout after IfaceMgr=" << timeout << LogEnd;
 
     // Messages timeout
     SPtr<TClntMsg> ptrMsg;
@@ -627,13 +793,48 @@ void TClntTransMgr::sendRequest(TOptList requestOptions, int iface)
     for (TOptList::iterator opt= requestOptions.begin();
          opt!=requestOptions.end(); ++opt)
     {
-        if (!allowOptInMsg(REQUEST_MSG, (*opt)->getOptType()))
+        if (!allowOptInMsg(REQUEST_MSG, (*opt)->getOptType()) ||
+	    (*opt)->getOptType() == OPTION_AUTH)
             opt = requestOptions.erase(opt);
     }
     SPtr<TClntMsg> ptr = new TClntMsgRequest(requestOptions, iface);
     Transactions.append( (Ptr*)ptr );
 }
 
+void TClntTransMgr::sendRenew()
+{
+    // Find all IAs
+    List(TAddrIA) iaLst;
+    SPtr<TAddrIA> ia;
+    SPtr<TAddrIA> iaPattern;
+    ClntAddrMgr().firstIA();
+
+    // Need to be fixed:?? how to deal with mutiple network interfaces.
+    while (ia = ClntAddrMgr().getIA() ) {
+        iaLst.append(ia);
+        ia->setState(STATE_INPROCESS);
+    }
+
+    // Find all PDs
+    List(TAddrIA) pdLst;
+    ClntAddrMgr().firstPD();
+    while (ia = ClntAddrMgr().getPD()) 
+    {
+        pdLst.append(ia);
+        ia->setState(STATE_INPROCESS);
+    }
+
+    if (iaLst.count() + pdLst.count() == 0) {
+	// there are no IAs or PD to refresh. Just do nothing.
+	return;
+    }
+	 
+    Log(Info) << "Generating RENEW for " << iaLst.count() << " IA(s) and " << pdLst.count() << " PD(s). " << LogEnd;
+    SPtr <TClntMsg> ptrRenew = new TClntMsgRenew(iaLst, pdLst);
+    Transactions.append(ptrRenew);
+}
+
+
 // Send RELEASE message
 void TClntTransMgr::sendRelease( List(TAddrIA) IALst, SPtr<TAddrIA> ta, List(TAddrIA) pdLst)
 {
@@ -649,15 +850,15 @@ void TClntTransMgr::sendRelease( List(TAddrIA) IALst, SPtr<TAddrIA> ta, List(TAd
     if (IALst.count()) {
             IALst.first();
             ptrIA = IALst.get();
-            iface = ptrIA->getIface();
+            iface = ptrIA->getIfindex();
             addr  = ptrIA->getSrvAddr();
     } else if (pdLst.count()) {
             pdLst.first();
             ptrIA = pdLst.get();
-            iface = ptrIA->getIface();
+            iface = ptrIA->getIfindex();
             addr  = ptrIA->getSrvAddr();
     } else if (ta) {
-            iface = ta->getIface();
+            iface = ta->getIfindex();
             addr  = ta->getSrvAddr();
     } else {
             Log(Error) << "Unable to send RELEASE message. No IA, PD or TA defined." << LogEnd;
@@ -720,6 +921,9 @@ void TClntTransMgr::checkSolicit() {
         if (iface->noConfig())
             continue;
 
+        if (iface->stateless())
+            continue;
+
         // step 1: check if there are any IA to be configured
         List(TClntCfgIA) iaLst; // list of IA requiring configuration
         SPtr<TClntCfgIA> cfgIA;
@@ -739,7 +943,7 @@ void TClntTransMgr::checkSolicit() {
             iface->firstTA();
             ta = iface->getTA();
             if (ta->getState()!=STATE_NOTCONFIGURED)
-            ta = 0;
+                ta.reset();
         }
 
         // step 3: check if there are any PD to be configured
@@ -762,8 +966,13 @@ void TClntTransMgr::checkSolicit() {
                 Log(Cont) << " (with rapid-commit)";
             } 
             Log(Cont) << " on " << iface->getFullName() <<" interface." << LogEnd;
-            Transactions.append(new TClntMsgSolicit(iface->getID(), 0, iaLst, ta, pdLst, 
+            Transactions.append(new TClntMsgSolicit(iface->getID(),
+                                                    SPtr<TIPv6Addr>(), iaLst, ta, pdLst, 
                                                     iface->getRapidCommit()));
+	    
+	    // state of certain IAs has changed. Let's log it.
+	    ClntAddrMgr().dump();
+	    ClntCfgMgr().dump();
         }
 
     }//for every iface
@@ -787,7 +996,7 @@ void TClntTransMgr::checkConfirm()
             {
                 if (ptrIA->getState()!=STATE_CONFIRMME)	
                     continue;
-                if (ptrIA->getIface()==iface->getID())
+                if (ptrIA->getIfindex()==iface->getID())
                 {
                     IALst.append(ptrIA);
                     ptrIA->setState(STATE_INPROCESS);
@@ -800,6 +1009,10 @@ void TClntTransMgr::checkConfirm()
             Log(Info) << "Creating CONFIRM: " << IALst.count() << " IA(s) on " << iface->getFullName() << LogEnd;
             Transactions.append(
                 new TClntMsgConfirm(iface->getID(), IALst));
+
+	    // state of certain IAs has changed. Let's log it.
+	    ClntAddrMgr().dump();
+	    ClntCfgMgr().dump();
         }
     }
 }
@@ -812,6 +1025,7 @@ void TClntTransMgr::checkInfRequest()
     {
         if (iface->noConfig())
             continue;
+
         SPtr<TClntIfaceIface> ifaceIface = (Ptr*)ClntIfaceMgr().getIfaceByID(iface->getID());
         if (!ifaceIface) {
             Log(Error) << "Interface with ifindex=" << iface->getID() << " not found." << LogEnd;
@@ -842,8 +1056,6 @@ void TClntTransMgr::checkInfRequest()
                 iface->setNISPDomainState(STATE_NOTCONFIGURED);
             if (iface->getKeyGenerationState() == STATE_CONFIGURED) 
                 iface->setKeyGenerationState(STATE_NOTCONFIGURED);
-            if (iface->getAuthenticationState() == STATE_CONFIGURED) 
-                iface->setAuthenticationState(STATE_NOTCONFIGURED);
         }
 
         if ( (iface->getDNSServerState()     == STATE_NOTCONFIGURED) ||
@@ -885,7 +1097,7 @@ void TClntTransMgr::checkRenew()
     {
         if ( (ia->getT1Timeout()!=0) || 
              (ia->getState()!=STATE_CONFIGURED) ||
-             (ia->getTentative()==TENTATIVE_UNKNOWN) )
+             (ia->getTentative()==ADDRSTATUS_UNKNOWN) )
             continue;
         
         if (!iaPattern) {
@@ -895,7 +1107,7 @@ void TClntTransMgr::checkRenew()
             continue;
         }
 
-        if ( (ia->getIface() == iaPattern->getIface()) &&
+        if ( (ia->getIfindex() == iaPattern->getIfindex()) &&
              (ia->getDUID()  == iaPattern->getDUID()) )
         {
                 iaLst.append(ia);
@@ -918,7 +1130,7 @@ void TClntTransMgr::checkRenew()
             continue;
         }
 
-        if ( (ia->getIface() == iaPattern->getIface()) &&
+        if ( (ia->getIfindex() == iaPattern->getIfindex()) &&
              (ia->getDUID()  == iaPattern->getDUID()) )
         {
                 pdLst.append(ia);
@@ -934,6 +1146,10 @@ void TClntTransMgr::checkRenew()
     Log(Info) << "Generating RENEW for " << iaLst.count() << " IA(s) and " << pdLst.count() << " PD(s). " << LogEnd;
     SPtr <TClntMsg> ptrRenew = new TClntMsgRenew(iaLst, pdLst);
     Transactions.append(ptrRenew);
+
+    // state of certain IAs has changed. Let's log it.
+    ClntAddrMgr().dump();
+    ClntCfgMgr().dump();
 }
 
 void TClntTransMgr::checkDecline()
@@ -951,7 +1167,7 @@ void TClntTransMgr::checkDecline()
         ClntAddrMgr().firstIA();
         while((ptrIA=ClntAddrMgr().getIA())&&(!firstIA))
         {
-            if (ptrIA->getTentative()==TENTATIVE_YES)
+            if (ptrIA->getTentative()==ADDRSTATUS_YES)
                 firstIA=ptrIA;
         }
         if (firstIA)
@@ -959,7 +1175,7 @@ void TClntTransMgr::checkDecline()
             declineIALst.append(firstIA);
             while(ptrIA=ClntAddrMgr().getIA())
             {
-                if ((ptrIA->getTentative()==TENTATIVE_YES)&&
+                if ((ptrIA->getTentative()==ADDRSTATUS_YES)&&
                     (*ptrIA->getDUID()==*firstIA->getDUID()))
                 {
                     declineIALst.append(ptrIA);
@@ -968,11 +1184,11 @@ void TClntTransMgr::checkDecline()
             }
             //Here should be send decline for all tentative addresses in IAs
             SPtr<TClntMsgDecline> decline = 
-                new TClntMsgDecline(firstIA->getIface(), 0, declineIALst);
+                new TClntMsgDecline(firstIA->getIfindex(), SPtr<TIPv6Addr>(), declineIALst);
             Transactions.append( (Ptr*) decline);
 
             // decline sent, now remove those addrs from IfaceMgr
-            SPtr<TIfaceIface> ptrIface = ClntIfaceMgr().getIfaceByID(firstIA->getIface());
+            SPtr<TIfaceIface> ptrIface = ClntIfaceMgr().getIfaceByID(firstIA->getIfindex());
             SPtr<TDUID> duid;
 
             declineIALst.first();
@@ -982,7 +1198,7 @@ void TClntTransMgr::checkDecline()
                 Log(Info) << "Sending DECLINE for IA(IAID=" << ptrIA->getIAID() << ")" << LogEnd;
 
                 while ( ptrAddr= ptrIA->getAddr() ) {
-                    if (ptrAddr->getTentative() == TENTATIVE_YES) {
+                    if (ptrAddr->getTentative() == ADDRSTATUS_YES) {
                         Log(Cont) << ptrAddr->get()->getPlain() << " ";
 
                         /// @todo: check result
@@ -1001,9 +1217,12 @@ void TClntTransMgr::checkDecline()
 
             // create REQUEST message
             SPtr<TClntMsgRequest> request;
-            request = new TClntMsgRequest(declineIALst, duid, firstIA->getIface() );
+            request = new TClntMsgRequest(declineIALst, duid, firstIA->getIfindex() );
             Transactions.append( (Ptr*) request);
 
+	    // state of certain IAs has changed. Let's log it.
+	    ClntAddrMgr().dump();
+	    ClntCfgMgr().dump();
         }
     } while(firstIA);
 }
@@ -1015,7 +1234,7 @@ void TClntTransMgr::checkRequest()
 
     SPtr<TAddrIA> ia;
     SPtr<TClntCfgIA> cfgIA;
-    SPtr<TDUID> duid = 0;
+    SPtr<TDUID> duid;
     List(TAddrIA) requestIALst;
     int ifaceID = 0;
 
@@ -1040,7 +1259,7 @@ void TClntTransMgr::checkRequest()
         }
 
         // if we have found one IA, following ones must be on the same interface
-        if ( ifaceID && (ia->getIface()!=ifaceID) ) {
+        if ( ifaceID && (ia->getIfindex()!=ifaceID) ) {
             continue;
         }
 
@@ -1056,7 +1275,7 @@ void TClntTransMgr::checkRequest()
         requestIALst.append(ia);
         ia->setState(STATE_INPROCESS);
         duid = ia->getDUID();
-        ifaceID = ia->getIface();
+        ifaceID = ia->getIfindex();
     }
 
     if (requestIALst.count()) {
@@ -1073,11 +1292,11 @@ bool TClntTransMgr::isDone()
 }
 
 char* TClntTransMgr::getCtrlAddr() {
-        return this->ctrlAddr;
+    return CtrlAddr_;
 }
 
 int  TClntTransMgr::getCtrlIface() {
-        return this->ctrlIface;
+    return CtrlIface_;
 }
 
 TClntTransMgr::~TClntTransMgr() {
@@ -1102,7 +1321,7 @@ SPtr<TMsg> TClntTransMgr::getAdvertise()
 SPtr<TOpt> TClntTransMgr::getAdvertiseDUID()
 {
     if (!AdvertiseLst.count())
-        return 0;
+        return TOptPtr(); // NULL
     AdvertiseLst.first();
     SPtr<TMsg> msg = AdvertiseLst.get();
     return msg->getOption(OPTION_SERVERID);
@@ -1182,6 +1401,30 @@ void TClntTransMgr::printAdvertiseLst() {
     printLst(AdvertiseLst);
 }
 
+/// @brief checks/updates loaded database (regarding interface names/indexes)
+///
+///
+/// @return true if sanitization was successful, false if it failed
+bool TClntTransMgr::sanitizeAddrDB() {
+
+    // Those two maps will hold current interface names/ifindexes
+    TAddrMgr::NameToIndexMapping currentNameToIndex;
+    TAddrMgr::IndexToNameMapping currentIndexToName;
+
+    // Let's get name->index and index->name maps first
+    ClntIfaceMgr().firstIface();
+    while (SPtr<TIfaceIface> iface = ClntIfaceMgr().getIface()) {
+        currentNameToIndex.insert(make_pair(iface->getName(), iface->getID()));
+        currentIndexToName.insert(make_pair(iface->getID(), iface->getName()));
+    }
+
+    // Ok, let's iterate over all loaded entries in Ifa
+
+    return ClntAddrMgr().updateInterfacesInfo(currentNameToIndex,
+                                              currentIndexToName);
+}
+
+
 #ifdef MOD_REMOTE_AUTOCONF
 SPtr<TClntTransMgr::TNeighborInfo> TClntTransMgr::neighborInfoGet(SPtr<TIPv6Addr> addr) {
     for (TNeighborInfoLst::iterator it=Neighbors.begin(); it!=Neighbors.end(); ++it) {
