@@ -6,25 +6,33 @@
  *
  * released under GNU GPL v2 only licence
  *
- * $Id: Iface.cpp,v 1.29 2008-08-29 00:07:29 thomson Exp $
- *
  */
 
+#include <iostream>
+#include <sstream>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <iostream>
-#include <sstream>
+#ifdef WIN32
+#include <ws2tcpip.h>
+#else
+#include <sys/socket.h>
+#include <net/if.h>
+#endif
 #include "Iface.h"
 #include "Portable.h"
 #include "Logger.h"
 
+
+
 using namespace std;
 /*
  * stores informations about interface
  */
-TIfaceIface::TIfaceIface(const char * name, int id, unsigned int flags, char* mac, 
-			 int maclen, char* llAddr, int llAddrCnt, char * globalAddr, int globalCnt, int hwType)
+TIfaceIface::TIfaceIface(const char * name, int id, unsigned int flags, char* mac,
+                         int maclen, char* llAddr, int llAddrCnt, char * globalAddr,
+                         int globalCnt, int hwType)
+    :Mac(new char[maclen]),Maclen(maclen), M_bit_(false), O_bit_(false)
 {
     snprintf(this->Name,MAX_IFNAME_LENGTH,"%s",name);
 
@@ -32,10 +40,8 @@ TIfaceIface::TIfaceIface(const char * name, int id, unsigned int flags, char* ma
     this->Flags = flags;
 
     // store mac address
-    this->Maclen=maclen;
-    this->Mac=new char[maclen];
     memcpy(this->Mac,mac,maclen);
- 
+
     // store all link-layer addresses
     this->LLAddrCnt=llAddrCnt;
     if (llAddrCnt>0)
@@ -49,8 +55,8 @@ TIfaceIface::TIfaceIface(const char * name, int id, unsigned int flags, char* ma
 
     // store all global addresses
     for (int i=0; i<globalCnt; i++) {
-	SPtr<TIPv6Addr> addr = new TIPv6Addr(globalAddr+16*i);
-	this->GlobalAddrLst.append(addr);
+        SPtr<TIPv6Addr> addr = new TIPv6Addr(globalAddr+16*i);
+        this->GlobalAddrLst.append(addr);
     }
 
     // store hardware type
@@ -75,8 +81,8 @@ string TIfaceIface::getFullName() {
     ostringstream oss;
     oss << this->ID;
     return string(this->Name)
-	+"/"
-	+oss.str();
+        +"/"
+        +oss.str();
 }
 
 /*
@@ -89,17 +95,20 @@ unsigned int TIfaceIface::getFlags() {
 void TIfaceIface::updateState(struct iface *x)
 {
     this->Flags = x->flags;
-    free(this->Mac);
+    delete [] Mac;
+    Mac = new char[Maclen];
     Maclen = x->maclen;
-    this->Mac = new char[Maclen];
     memcpy(Mac, x->mac, Maclen);
 
-    if (LLAddrCnt && LLAddr)
-	free(LLAddr);
+    if (LLAddrCnt && LLAddr) {
+        delete [] LLAddr;
+        LLAddrCnt = 0;
+        LLAddr = 0;
+    }
     LLAddrCnt = x->linkaddrcount;
     if (LLAddrCnt) {
-	LLAddr = new char[16*LLAddrCnt];
-	memcpy(LLAddr, x->linkaddr, 16*LLAddrCnt);
+        LLAddr = new char[16*LLAddrCnt];
+        memcpy(LLAddr, x->linkaddr, 16*LLAddrCnt);
     }
     PresLLAddr = LLAddr;
 
@@ -107,50 +116,53 @@ void TIfaceIface::updateState(struct iface *x)
 
     // store all global addresses
     for (int i=0; i<x->globaladdrcount; i++) {
-	SPtr<TIPv6Addr> addr = new TIPv6Addr(x->globaladdr+16*i);
-	this->GlobalAddrLst.append(addr);
+        SPtr<TIPv6Addr> addr = new TIPv6Addr(x->globaladdr+16*i);
+        this->GlobalAddrLst.append(addr);
     }
 
     HWType = x->hardwareType;
+
+    setMBit(x->m_bit);
+    setOBit(x->o_bit);
 }
 
 
 /**
  * returns true if interface is UP
  */
-bool TIfaceIface::flagUp() 
+bool TIfaceIface::flagUp()
 {
-    return this->Flags&IF_UP;
+    return this->Flags & IFF_UP;
 }
 
 /**
  * returns true if interface is RUNNING
  */
-bool TIfaceIface::flagRunning() 
+bool TIfaceIface::flagRunning()
 {
-    return (bool)(this->Flags & IF_RUNNING);
+    return (bool)(this->Flags & IFF_RUNNING);
 }
 
 /**
  * returns true is interface is MULTICAST capable
  */
-bool TIfaceIface::flagMulticast() 
+bool TIfaceIface::flagMulticast()
 {
-    return (Flags&IF_MULTICAST)?true:false;
+    return (Flags&IFF_MULTICAST)?true:false;
 }
 
 /**
  * returns true is interface is LOOPBACK
  */
-bool TIfaceIface::flagLoopback() 
+bool TIfaceIface::flagLoopback()
 {
-    return (Flags&IF_LOOPBACK)?true:false;
+    return (Flags&IFF_LOOPBACK)?true:false;
 }
 
 /**
  * returns MAC length
  */
-int TIfaceIface::getMacLen() 
+int TIfaceIface::getMacLen()
 {
     return Maclen;
 }
@@ -158,12 +170,12 @@ int TIfaceIface::getMacLen()
 /**
  * returns MAC
  */
-char* TIfaceIface::getMac() 
+char* TIfaceIface::getMac()
 {
     return Mac;
 }
 
-void TIfaceIface::firstGlobalAddr() 
+void TIfaceIface::firstGlobalAddr()
 {
     GlobalAddrLst.first();
 }
@@ -176,27 +188,27 @@ unsigned int TIfaceIface::countGlobalAddr() {
     return this->GlobalAddrLst.count();
 }
 
-void TIfaceIface::addGlobalAddr(SPtr<TIPv6Addr> addr) 
+void TIfaceIface::addGlobalAddr(SPtr<TIPv6Addr> addr)
 {
     this->GlobalAddrLst.append(addr);
 }
 
-void TIfaceIface::delGlobalAddr(SPtr<TIPv6Addr> addr) 
+void TIfaceIface::delGlobalAddr(SPtr<TIPv6Addr> addr)
 {
     SPtr<TIPv6Addr> tempAddr;
-    for(int i = 0; i < this->GlobalAddrLst.count(); i++)
+    for(unsigned int i = 0; i < this->GlobalAddrLst.count(); i++)
     {
-	      tempAddr = this->GlobalAddrLst.get();
-	      if(*tempAddr == *addr){
-		        this->GlobalAddrLst.del();
-		        break;
-	      }
+        tempAddr = this->GlobalAddrLst.get();
+        if(*tempAddr == *addr){
+            this->GlobalAddrLst.del();
+            break;
+        }
     }
 }
 
 
 /**
- * returns HW type 
+ * returns HW type
  */
 int TIfaceIface::getHardwareType() {
     return this->HWType;
@@ -212,10 +224,10 @@ int TIfaceIface::getHardwareType() {
  */
 bool TIfaceIface::addAddr(SPtr<TIPv6Addr> addr,long pref, long valid, int prefixLen) {
     Log(Notice) << "Address " << addr->getPlain() << "/" << prefixLen << " added to "
-		<< getFullName() << " interface." << LogEnd;
+                << getFullName() << " interface." << LogEnd;
 
-    return (bool)ipaddr_add(this->Name, this->ID, addr->getPlain(), 
-			    pref, valid, prefixLen);
+    return (bool)ipaddr_add(this->Name, this->ID, addr->getPlain(),
+                            pref, valid, prefixLen);
 }
 
 /**
@@ -224,7 +236,7 @@ bool TIfaceIface::addAddr(SPtr<TIPv6Addr> addr,long pref, long valid, int prefix
  */
 bool TIfaceIface::delAddr(SPtr<TIPv6Addr> addr, int prefixLen) {
     Log(Notice) << "Address " << addr->getPlain() << "/" << prefixLen << " deleted from "
-		<< getFullName() << " interface." << LogEnd;
+                << getFullName() << " interface." << LogEnd;
     return (bool)ipaddr_del( this->Name, this->ID, addr->getPlain(), prefixLen);
 }
 
@@ -234,13 +246,13 @@ bool TIfaceIface::delAddr(SPtr<TIPv6Addr> addr, int prefixLen) {
 bool TIfaceIface::updateAddr(SPtr<TIPv6Addr> addr, long pref, long valid) {
     int result;
     Log(Notice) << "Address " << addr->getPlain() << " updated on "
-		<< getFullName() << " interface." << LogEnd;
+                << getFullName() << " interface." << LogEnd;
 
-    result = ipaddr_update((char *)this->Name, this->ID, (char *)addr->getPlain(), 
-			   pref, valid, this->PrefixLen);
+    result = ipaddr_update((char *)this->Name, this->ID, (char *)addr->getPlain(),
+                           pref, valid, this->PrefixLen);
 
     if (result!=LOWLEVEL_NO_ERROR)
-	return false;
+        return false;
 
     return true;
 }
@@ -282,7 +294,7 @@ bool TIfaceIface::addSocket(SPtr<TIPv6Addr> addr,int port, bool ifaceonly, bool
     // Log(Debug) << "Creating socket on " << *addr << " address." << LogEnd;
     SPtr<TIfaceSocket> ptr = new TIfaceSocket(this->Name, this->ID, port, addr, ifaceonly, reuse);
     if (ptr->getStatus()!=STATE_CONFIGURED) {
-      	return false;
+        return false;
     }
     SocketsLst.append(ptr);
     return true;
@@ -293,10 +305,10 @@ bool TIfaceIface::addSocket(SPtr<TIPv6Addr> addr,int port, bool ifaceonly, bool
  * binds socket on whole interface
  */
 bool TIfaceIface::addSocket(int port, bool ifaceonly, bool reuse) {
-    SPtr<TIfaceSocket> ptr = 
-	new TIfaceSocket(this->Name, this->ID, port, ifaceonly, reuse);
+    SPtr<TIfaceSocket> ptr =
+        new TIfaceSocket(this->Name, this->ID, port, ifaceonly, reuse);
     if (ptr->getStatus()!=STATE_CONFIGURED) {
-	      return false;
+              return false;
     }
     SocketsLst.append(ptr);
     return true;
@@ -309,12 +321,12 @@ bool TIfaceIface::addSocket(int port, bool ifaceonly, bool reuse) {
 bool TIfaceIface::delSocket(int fd) {
     SPtr<TIfaceSocket> sock;
     SocketsLst.first();
-    
+
     while ( sock = SocketsLst.get() ) {
-	      if (sock->getFD() == fd) {
-	          SocketsLst.del();
-	          return true;
-	      }
+              if (sock->getFD() == fd) {
+                  SocketsLst.del();
+                  return true;
+              }
     }
     return false;
 }
@@ -336,14 +348,14 @@ SPtr <TIfaceSocket> TIfaceIface::getSocket() {
 /*
  * returns socket by FileDescriptor (or NULL, if no such socket exists)
  */
-SPtr <TIfaceSocket> TIfaceIface::getSocketByFD(int fd) {
+SPtr<TIfaceSocket> TIfaceIface::getSocketByFD(int fd) {
     SPtr<TIfaceSocket> ptr;
     SocketsLst.first();
     while ( ptr = SocketsLst.get() ) {
-	      if ( ptr->getFD()==fd )
-	          return ptr;
+              if ( ptr->getFD()==fd )
+                  return ptr;
     }
-    return 0; // NULL
+    return SPtr<TIfaceSocket>(); // NULL
 }
 
 /*
@@ -358,7 +370,7 @@ int TIfaceIface::countSocket() {
  */
 TIfaceIface::~TIfaceIface() {
     if (this->LLAddrCnt>0) {
-      	delete [] this->LLAddr;
+        delete [] this->LLAddr;
     }
     delete [] Mac;
 }
@@ -367,16 +379,16 @@ SPtr<TIfaceSocket> TIfaceIface::getSocketByAddr(SPtr<TIPv6Addr> addr) {
     SPtr<TIfaceSocket> ptr;
     SocketsLst.first();
     while ( ptr = SocketsLst.get() ) {
-	      if ( *ptr->getAddr()==*addr )
-	          return ptr;
+              if ( *ptr->getAddr()==*addr )
+                  return ptr;
     }
-    return 0; // NULL
+    return SPtr<TIfaceSocket>(); // NULL
 }
 
 void TIfaceIface::setPrefixLength(int len) {
     if (len>128 || len<0) {
-	      Log(Error) << "Invalid length " << len << " set attempt was ignored on the " << this->getFullName() << " interface." << LogEnd;
-	      return;
+              Log(Error) << "Invalid length " << len << " set attempt was ignored on the " << this->getFullName() << " interface." << LogEnd;
+              return;
     }
     this->PrefixLen = len;
 }
@@ -385,6 +397,34 @@ int TIfaceIface::getPrefixLength() {
     return PrefixLen;
 }
 
+/// @brief set M (managed) bit as received from Router Advertisement
+///
+/// @param m bool flag (Sets of clears M bit)
+void TIfaceIface::setMBit(bool m) {
+    M_bit_ = m;
+}
+
+/// @brief set O (other conf) bit as received from Router Advertisement
+///
+/// @param o bool flag (Sets of clears O bit)
+void TIfaceIface::setOBit(bool o) {
+    O_bit_ = o;
+}
+
+/// @brief returns M (managed) bit as received from Router Advertisement
+///
+/// @return value of M bit
+bool TIfaceIface::getMBit() {
+    return M_bit_;
+}
+
+/// @brief returns O (other conf) bit as received from Router Advertisement
+///
+/// @return value of O bit
+bool TIfaceIface::getOBit() {
+    return O_bit_;
+}
+
 // --------------------------------------------------------------------
 // --- operators ------------------------------------------------------
 // --------------------------------------------------------------------
@@ -394,17 +434,22 @@ int TIfaceIface::getPrefixLength() {
 ostream & operator <<(ostream & strum, TIfaceIface &x) {
     char buf[48];
 
-    strum << "  <IfaceIface";
-    strum << " name=\"" << x.Name << "\"";
-    strum << " ifindex=\"" << x.ID << "\"";
-    strum << " hwType=\"" << x.getHardwareType() << "\"";
-    strum << " flags=\"" << x.Flags << "\">" << endl;
-    strum << "    <!-- PrefixLength configured to " << x.PrefixLen << " -->" << endl;
-    strum << "    <!-- " << x.LLAddrCnt << " link scoped addrs -->" << endl;
+    strum << "  <IfaceIface"
+          << " name=\"" << x.Name << "\""
+          << " ifindex=\"" << x.ID << "\""
+          << " hwType=\"" << x.getHardwareType() << "\""
+          << " flags=\"0x" << hex << x.Flags << dec << "\" "
+          << " mBit=\"" << (x.M_bit_?"1":"0") << "\" oBit=\"" << (x.O_bit_?"1":"0")
+          << "\">" << endl
+          << "    <!-- " << (x.flagLoopback()?"looback":"no-loopback")
+          << (x.flagRunning()?" running":" no-running")
+          << (x.flagMulticast()?" multicast -->":" no-multicast -->") << endl
+          << "    <!-- PrefixLength configured to " << x.PrefixLen << " -->" << endl
+          << "    <!-- " << x.LLAddrCnt << " link scoped addrs -->" << endl;
 
     for (int i=0; i<x.LLAddrCnt; i++) {
-	      inet_ntop6(x.LLAddr+i*16,buf);
-	      strum << "    <Addr>" << buf << "</Addr>" << endl;
+              inet_ntop6(x.LLAddr+i*16,buf);
+              strum << "    <Addr>" << buf << "</Addr>" << endl;
     }
 
     strum << "    <!-- " << x.countGlobalAddr() << " non-local addrs -->" << endl;
@@ -412,22 +457,22 @@ ostream & operator <<(ostream & strum, TIfaceIface &x) {
     x.firstGlobalAddr();
     SPtr<TIPv6Addr> addr;
     while (addr = x.getGlobalAddr()) {
-      	strum << "    " << *addr;
+        strum << "    " << *addr;
     }
 
     strum << "    <Mac>";
     for (int i=0; i<x.Maclen; i++) {
-	      strum.fill('0');
-	      strum.width(2);
-	      strum << (hex) << (int) x.Mac[i];
-	      if (i<x.Maclen-1) strum  << ":";
+              strum.fill('0');
+              strum.width(2);
+              strum << (hex) << (int) x.Mac[i];
+              if (i<x.Maclen-1) strum  << ":";
     }
     strum << "</Mac>" << endl;
 
     SPtr<TIfaceSocket> sock;
     x.firstSocket();
     while (sock = x.getSocket() ) {
-      	strum << "    " << *sock;
+        strum << "    " << *sock;
     }
     strum << "  </IfaceIface>" << endl;
     return strum;
