@@ -6,8 +6,6 @@
  *
  * released under GNU GPL v2 only licence
  *
- * $Id: Iface.h,v 1.18 2008-08-29 00:07:30 thomson Exp $
- *
  */
 
 #ifndef IFACEIFACE_H
@@ -25,13 +23,13 @@
  */
 class TIfaceIface{
  public:
-    friend ostream & operator <<(ostream & strum, TIfaceIface &x);
+    friend std::ostream & operator <<(std::ostream & strum, TIfaceIface &x);
 
     TIfaceIface(const char * name, int id, unsigned int flags, char* mac, 
                 int maclen, char* llAddr, int llAddrCnt, char * globalAddr, int globalCnt, int hwType);
     char * getName();
     int getID();
-    string getFullName();
+    std::string getFullName();
 
     // ---flags related---
     unsigned int getFlags();
@@ -41,6 +39,12 @@ class TIfaceIface{
     bool flagLoopback();
     void updateState(struct iface * x);
 
+    // --- RA bits ---
+    void setMBit(bool m);
+    void setOBit(bool o);
+    bool getMBit();
+    bool getOBit();
+
     // ---layer-2 related---
     int   getMacLen();
     char* getMac();
@@ -83,12 +87,11 @@ class TIfaceIface{
     int Maclen;
     char* LLAddr;
     int LLAddrCnt;
+    bool M_bit_; // M (managed) bit from Router Advertisement
+    bool O_bit_; // O (other conf) bit from Router Advertisement
 
     List(TIPv6Addr) GlobalAddrLst;
-    
-//    char * GlobalAddr;
-//    int GlobalAddrCnt;
-    
+
     int HWType;
 
     // sockets
