@@ -6,15 +6,15 @@
  *
  * released under GNU GPL v2 only licence
  *
- * $Id: IfaceMgr.cpp,v 1.28 2008-08-29 00:07:30 thomson Exp $
- *
  */
 
 #include <string.h>
+#include <unistd.h>
 #include <iostream>
 #include <fstream>
 #include <sstream>
 #include <string>
+#include <stdlib.h>
 #include <errno.h>
 #include "Portable.h"
 #include "IfaceMgr.h"
@@ -26,11 +26,13 @@
 #include "OptIAPrefix.h"
 #include "ScriptParams.h"
 
+using namespace std;
+
 /// constructor
 ///
 /// @param xmlFile xml file, where interface info will be stored
 /// @param getIfaces  specifies if interfaces should be detected
-TIfaceMgr::TIfaceMgr(string xmlFile, bool getIfaces)
+TIfaceMgr::TIfaceMgr(const std::string& xmlFile, bool getIfaces)
 {
     this->XmlFile = xmlFile;
     this->IsDone  = false;
@@ -66,10 +68,14 @@ TIfaceMgr::TIfaceMgr(string xmlFile, bool getIfaces)
                                                        ptr->globaladdr,
                                                        ptr->globaladdrcount,
                                                        ptr->hardwareType));
+        iface->setMBit(ptr->m_bit);
+        iface->setOBit(ptr->o_bit);
         this->IfaceLst.append(iface);
         ptr = ptr->next;
     }
     if_list_release(ifaceList); // allocated in pure C, and so release it there
+
+    dump();
 }
 
 /*
@@ -98,14 +104,14 @@ SPtr<TIfaceIface> TIfaceMgr::getIface() {
  * gets interface by it's name (or NULL if no such inteface exists)
  * @param name - interface name
  */
-SPtr<TIfaceIface> TIfaceMgr::getIfaceByName(string name) {
+SPtr<TIfaceIface> TIfaceMgr::getIfaceByName(const std::string& name) {
     SPtr<TIfaceIface> ptr;
     IfaceLst.first();
     while ( ptr = IfaceLst.get() ) {
         if ( !strcmp(name.c_str(),ptr->getName()) )
             return ptr;
     }
-    return 0; // NULL
+    return SPtr<TIfaceIface>(); // NULL
 }
 
 /*
@@ -119,7 +125,7 @@ SPtr<TIfaceIface> TIfaceMgr::getIfaceByID(int id) {
         if ( id == ptr->getID() )
             return ptr;
     }
-    return 0; //NULL
+    return SPtr<TIfaceIface>(); // NULL
 }
 
 /*
@@ -132,21 +138,21 @@ SPtr<TIfaceIface> TIfaceMgr::getIfaceBySocket(int fd) {
         if ( ptr->getSocketByFD(fd) )
             return ptr;
     }
-    return 0;
+    return SPtr<TIfaceIface>(); // NULL
 }
 
-/*
- * tries to read data from any socket on all interfaces
- * returns after time seconds.
- * @param time listens for time seconds
- * @param buf buffer
- * @param bufsize buffer size
- * @param peer informations about sender
- *
- * @return socket descriptor (or 0)
- */
+/// tries to read data from any socket on all interfaces
+/// returns after time seconds.
+/// @param time listens for time seconds
+/// @param buf buffer
+/// @param bufsize buffer size
+/// @param peer [out] sender address
+/// @param myaddr [out] local IPv6 address
+///
+/// @return socket descriptor (or negative values for errors)
 int TIfaceMgr::select(unsigned long time, char *buf,
-                      int &bufsize, SPtr<TIPv6Addr> peer) {
+                      int &bufsize, SPtr<TIPv6Addr> peer,
+                      SPtr<TIPv6Addr> myaddr) {
     struct timeval czas;
     int result;
     if (time > DHCPV6_INFINITY/2)
@@ -166,22 +172,33 @@ int TIfaceMgr::select(unsigned long time, char *buf,
     fds = *TIfaceSocket::getFDS();
 
     int maxFD;
-    //maxFD = FD_SETSIZE;
     maxFD = TIfaceSocket::getMaxFD() + 1;
 
-    result = ::select(maxFD,&fds,NULL, NULL, &czas);
+    // no sockets to listen  on... hopefully this is just inactive mode,
+    // not an error
+    if (!TIfaceSocket::getCount()) {
+        Log(Debug) << "No sockets open. Sleeping for " << time << " seconds." << LogEnd;
+#ifdef WIN32
+        Sleep(time*1000); // Windows sleep is specified in milliseconds
+#else
+        sleep(time); // Posix sleep is specified in seconds
+#endif
+        return 0;
+    }
+    result = ::select(maxFD, &fds, NULL, NULL, &czas);
 
     // something received
 
     if (result==0) { // timeout, nothing received
         bufsize = 0;
-        return 0;
+        return -1;
     }
     if (result<0) {
         char buf[512];
         strncpy(buf, strerror(errno),512);
-        Log(Debug) << "Failed to read sockets (select() returned " << result << "), error=" << buf << LogEnd;
-        return 0;
+        Log(Debug) << "Failed to read sockets (select() returned " << result
+                   << "), error=" << buf << LogEnd;
+        return -1;
     }
 
     SPtr<TIfaceIface> iface;
@@ -199,8 +216,8 @@ int TIfaceMgr::select(unsigned long time, char *buf,
     }
 
     if (!found) {
-        Log(Error) << "Seems like internal error. Unable to find any socket with incoming data." << LogEnd;
-        return 0;
+        Log(Error) << "Internal error. Can't find any socket with incoming data." << LogEnd;
+        return -1;
     }
 
     char myPlainAddr[48];   // my plain address
@@ -213,6 +230,7 @@ int TIfaceMgr::select(unsigned long time, char *buf,
     inet_pton6(peerPlainAddr,peerAddrPacked);
     inet_pton6(myPlainAddr,myAddrPacked);
     peer->setAddr(peerAddrPacked);
+    myaddr->setAddr(myAddrPacked);
 
     if (result==-1) {
         Log(Error) << "Socket recv() failure detected." << LogEnd;
@@ -220,7 +238,7 @@ int TIfaceMgr::select(unsigned long time, char *buf,
         return -1;
     }
 
-#ifndef WIN32
+#ifdef MOD_SRV_DST_ADDR_CHECK
     // check if we've received data addressed to us. There's problem with sockets binding.
     // If there are 2 open sockets (one bound to multicast and one to global address),
     // each packet sent on multicast address is also received on unicast socket.
@@ -231,7 +249,7 @@ int TIfaceMgr::select(unsigned long time, char *buf,
             Log(Debug) << "Received data on address " << myPlainAddr << ", expected "
                    << *sock->getAddr() << ", message ignored." << LogEnd;
             bufsize = 0;
-            return 0;
+            return -1;
     }
 #endif
 
@@ -243,7 +261,7 @@ int TIfaceMgr::select(unsigned long time, char *buf,
  * returns interface count
  */
 int TIfaceMgr::countIface() {
-        return IfaceLst.count();
+    return IfaceLst.count();
 }
 
 /*
@@ -262,6 +280,7 @@ void TIfaceMgr::dump()
  */
 TIfaceMgr::~TIfaceMgr()
 {
+    closeSockets();
 }
 
 string TIfaceMgr::printMac(char * mac, int macLen) {
@@ -324,7 +343,7 @@ void TIfaceMgr::optionToEnv(TNotifyScriptParams& params, SPtr<TOpt> opt, std::st
             // Will define something like this: OPTION_NEXT_HOP=2001:db8:1::1
             params.addParam("OPTION_NEXT_HOP", opt->getPlain());
         }
-	break;
+        break;
     }
     case OPTION_RTPREFIX: {
         params.addParam("OPTION_RTPREFIX", opt->getPlain());
@@ -344,7 +363,7 @@ void TIfaceMgr::optionToEnv(TNotifyScriptParams& params, SPtr<TOpt> opt, std::st
 
 
 
-void TIfaceMgr::notifyScripts(std::string scriptName, SPtr<TMsg> question, SPtr<TMsg> reply)
+void TIfaceMgr::notifyScripts(const std::string& scriptName, SPtr<TMsg> question, SPtr<TMsg> reply)
 {
     TNotifyScriptParams* params = (TNotifyScriptParams*)reply->getNotifyScriptParams();
     if (params) {
@@ -355,7 +374,8 @@ void TIfaceMgr::notifyScripts(std::string scriptName, SPtr<TMsg> question, SPtr<
     }
 }
 
-void TIfaceMgr::notifyScript(std::string scriptName, std::string action, TNotifyScriptParams& params) {
+void TIfaceMgr::notifyScript(const std::string& scriptName, std::string action,
+                             TNotifyScriptParams& params) {
     const char * argv[3];
 
     // get PATH
@@ -368,7 +388,7 @@ void TIfaceMgr::notifyScript(std::string scriptName, std::string action, TNotify
     argv[0] = scriptName.c_str();
     argv[1] = action.c_str();
     argv[2] = NULL;
-    
+
     Log(Debug) << "About to execute " << scriptName << " script, "
                << params.envCnt << " variables." << LogEnd;
     int returnCode = execute(scriptName.c_str(), argv, params.env);
@@ -381,14 +401,14 @@ void TIfaceMgr::notifyScript(std::string scriptName, std::string action, TNotify
     }
 }
 
-void TIfaceMgr::notifyScripts(std::string scriptName, SPtr<TMsg> question, SPtr<TMsg> reply,
-                              TNotifyScriptParams& params)
+void TIfaceMgr::notifyScripts(const std::string& scriptName, SPtr<TMsg> question,
+                              SPtr<TMsg> reply, TNotifyScriptParams& params)
 {
     if (!scriptName.length()) {
         Log(Debug) << "Not executing external script (Notify script disabled)." << LogEnd;
         return;
     }
-    
+
     stringstream tmp;
     string action;
 
@@ -403,18 +423,20 @@ void TIfaceMgr::notifyScripts(std::string scriptName, SPtr<TMsg> question, SPtr<
         break;
     case RENEW_MSG:
     case REBIND_MSG:
+    case INFORMATION_REQUEST_MSG:
         action = "update";
         break;
     default:
-        Log(Debug) << "Script execution skipped for " << reply->getName() << " response to " << question->getName() 
-                   << ". No action needed for this type of message." << LogEnd;
+        Log(Debug) << "Script execution skipped for " << reply->getName() << " response to "
+                   << question->getName() << ". No action needed for this message type." << LogEnd;
         return;
     }
 
     int ifindex = reply->getIface();
     SPtr<TIfaceIface> iface = (Ptr*)getIfaceByID(ifindex);
     if (!iface) {
-        Log(Error) << "Unable to find interface with ifindex=" << ifindex << ". Script NOT called." << LogEnd;
+        Log(Error) << "Unable to find interface with ifindex=" << ifindex
+                   << ". Script NOT called." << LogEnd;
         return;
     }
 
@@ -424,32 +446,46 @@ void TIfaceMgr::notifyScripts(std::string scriptName, SPtr<TMsg> question, SPtr<
     params.addParam("IFINDEX", tmp.str().c_str());
     tmp.str("");
 
-    params.addParam("REMOTE_ADDR", reply->getAddr()->getPlain());
+    string remote;
+    if (reply->getRemoteAddr()) {
+      remote = reply->getRemoteAddr()->getPlain();
+    } else {
+      remote = string(ALL_DHCP_RELAY_AGENTS_AND_SERVERS);
+    }
+
+    params.addParam("REMOTE_ADDR", remote);
 
     params.addParam("CLNT_MESSAGE", question->getName());
 
     params.addParam("SRV_MESSAGE", reply->getName());
 
-    SPtr<TIPv6Addr> ip;
-
     // add options from server REPLY
     reply->firstOption();
     while ( SPtr<TOpt> opt = reply->getOption() ) {
         optionToEnv(params, opt, "SRV");
     }
 
-#if 0
     // add options from client message
     question->firstOption();
     while( SPtr<TOpt> opt = question->getOption() ) {
         optionToEnv(params, opt, "CLNT");
     }
-#endif
 
     notifyScript(scriptName, action, params);
 }
 
+/// @brief closes all sockets
+void TIfaceMgr::closeSockets() {
+    Log(Debug) << "Closing all sockets." << LogEnd;
+    firstIface();
+    while (SPtr<TIfaceIface> iface = getIface()) {
+        iface->firstSocket();
 
+        while (SPtr<TIfaceSocket> socket = iface->getSocket()) {
+            iface->delSocket(socket->getFD());
+        }
+    }
+}
 
 // --------------------------------------------------------------------
 // --- operators ------------------------------------------------------
