@@ -13,11 +13,14 @@
 
 #include <string.h>
 #include <string>
+#include <ostream>
 #include "SocketIPv6.h"
 #include "Portable.h"
 #include "DHCPConst.h"
 #include "Logger.h"
 
+using namespace std;
+
 /**
  * static elements of TIfaceSocket class
  */ 
@@ -213,6 +216,9 @@ TIfaceSocket::~TIfaceSocket() {
     if (Status!=STATE_CONFIGURED) 
 	return;
 
+    Log(Debug) << "Closing socket " << this->FD << " on " << Addr->getPlain()
+               << ":" << Port << " on interface " << Iface << "/" << IfaceID << LogEnd;
+
     //execute low-level function
     sock_del(this->FD);
 
@@ -268,7 +274,7 @@ void TIfaceSocket::printError(int error, char * iface, int ifaceid, SPtr<TIPv6Ad
 /*
  * flush this data in XML format
  */
-ostream & operator <<(ostream & strum, TIfaceSocket &x)
+std::ostream & operator <<(std::ostream & strum, TIfaceSocket &x)
 {
     strum << dec 
 	  << "<IfaceSocket"
