@@ -8,7 +8,7 @@
  *
  */
 
-#include "limits.h"
+#include <limits.h>
 
 #ifndef DHCPDEFAULTS_H
 #define DHCPDEFAULTS_H
@@ -19,13 +19,20 @@
 // addresses reported as DECLINED are not used for 2 hours
 #define DECLINED_TIMEOUT ((unsigned long) 7200)
 
+// 1 (quiet) - 8 (debug)
+#define DEFAULT_LOGLEVEL 7
+
 // DHCPv6 server default values
 #define SERVER_DEFAULT_DOMAIN ""
 #define SERVER_DEFAULT_TIMEZONE ""
 #define SERVER_DEFAULT_CLNTMAXLEASE 10
-#define SERVER_DEFAULT_IFACEMAXLEASE ULONG_MAX
+#define SERVER_DEFAULT_CLASSMAXLEASE 1048576
+#define SERVER_DEFAULT_IFACEMAXLEASE UINT_MAX
 #define SERVER_DEFAULT_PREFERENCE 0
 #define SERVER_DEFAULT_RAPIDCOMMIT false
+#define SERVER_DEFAULT_LEASEQUERY false
+#define SERVER_DEFAULT_DNSUPDATE_MODE DNSUPDATE_MODE_NONE
+#define SERVER_DEFAULT_DNSUPDATE_REVDNS_ZONE_LEN 64
 
 #define SERVER_DEFAULT_MIN_T1    5
 #define SERVER_DEFAULT_MAX_T1    3600      /* 1 hour */
@@ -36,23 +43,28 @@
 #define SERVER_DEFAULT_MIN_VALID 10800     /* 3 hours */
 #define SERVER_DEFAULT_MAX_VALID 172800    /* 2 days */
 #define SERVER_DEFAULT_CLASS_SHARE 100
-#define SERVER_DEFAULT_CLASS_MAX_LEASE ULONG_MAX
+#define SERVER_DEFAULT_CLASS_MAX_LEASE UINT_MAX
 #define SERVER_DEFAULT_TA_PREF_LIFETIME 3600
 #define SERVER_DEFAULT_TA_VALID_LIFETIME 7200
 #define SERVER_DEFAULT_CACHE_SIZE 1048576   /* cache size, specified in bytes */
 
+#define SERVER_MAX_IA_RANDOM_TRIES 100
+#define SERVER_MAX_TA_RANDOM_TRIES 100
+#define SERVER_MAX_PD_RANDOM_TRIES 100
+
 // see DHCPConst.h for available enums
 #define SERVER_DEFAULT_UNKNOWN_FQDN UNKNOWN_FQDN_REJECT
 
-#define CLIENT_DEFAULT_T1 ULONG_MAX
-#define CLIENT_DEFAULT_T2 ULONG_MAX
+#define CLIENT_DEFAULT_T1 UINT_MAX
+#define CLIENT_DEFAULT_T2 UINT_MAX
 
 #define CLIENT_DEFAULT_UNICAST false
 #define CLIENT_DEFAULT_RAPID_COMMIT false
 
-#define CLIENT_DEFAULT_PREFIX_LENGTH 64
-#define CLIENT_DEFAULT_DIGEST DIGEST_NONE
+// It is now /128. See discussion in bug #222
+#define CLIENT_DEFAULT_PREFIX_LENGTH 128
 
 #define CLIENT_DEFAULT_FQDN_FLAG_S true
 
+
 #endif /* DHCPDEFAULTS_H */
