@@ -6,12 +6,6 @@
  *
  * released under GNU GPL v2 licence
  *
- * $Id: DHCPRelay.h,v 1.2 2005-02-01 00:57:36 thomson Exp $
- *
- * $Log: not supported by cvs2svn $
- * Revision 1.1  2005/01/11 22:53:35  thomson
- * Relay skeleton implemented.
- *
  */
 
 #ifndef DHCPSERVER_H
@@ -20,12 +14,10 @@
 #include <iostream>
 #include <string>
 
-using namespace std;
-
 class TDHCPRelay
 {
   public:
-    TDHCPRelay(string config);
+    TDHCPRelay(const std::string& config);
     void run();
     void stop();
     bool isDone();
