@@ -14,11 +14,10 @@
 #include <string>
 #include "SmartPtr.h"
 
-using namespace std;
 class TDHCPServer
 {
   public:
-    TDHCPServer(string config);
+    TDHCPServer(const std::string& config);
     void run();
     void stop();
     bool isDone();
@@ -27,7 +26,7 @@ class TDHCPServer
     ~TDHCPServer();
 
   private:
-    bool IsDone;
+    bool IsDone_;
 };
 
 #endif
