@@ -10,12 +10,10 @@
  *
  */
 
-
 #ifndef PORTABLE_H
 #define PORTABLE_H
 
-/* #define DIBBLER_VERSION "0.8.0-git (" __DATE__ " " __TIME__ ")" */
-#define DIBBLER_VERSION "0.8.2"
+#define DIBBLER_VERSION "1.0.1"
 
 #define DIBBLER_COPYRIGHT1 "| Dibbler - a portable DHCPv6, version " DIBBLER_VERSION
 #define DIBBLER_COPYRIGHT2 "| Authors : Tomasz Mrugalski<thomson(at)klub.com.pl>,Marek Senderski<msend(at)o2.pl>"
@@ -24,36 +22,15 @@
 
 #ifdef WIN32
 #include <winsock2.h>
+#define strcasecmp strcmpi
+#define snprintf _snprintf
 #endif
 #if defined(LINUX) || defined(BSD)
 #include <arpa/inet.h>
 #endif
 
-#ifdef WIN32
-#define strcasecmp strcmpi
-
-#ifndef uint8_t
-#define uint8_t  unsigned char
-#endif
-
-#ifndef uint16_t
-#define uint16_t unsigned short int
-#endif
-
-#ifndef uint32_t
-#define uint32_t unsigned int
-#endif
-
-#ifndef uint64_t
-#define uint64_t unsigned long long int
-#endif
-
-#define snprintf _snprintf
-#endif
-
-#ifdef BSD
 #include <stdint.h>
-#endif
+#include <unistd.h>
 
 /* this should look like this: 
 uint16_t readUint16(uint8_t* buf);
@@ -70,12 +47,15 @@ uint8_t* writeUint64(uint8_t* buf, uint64_t qword); */
 #ifdef __cplusplus 
 extern "C" {
 #endif
+    uint8_t readUint8(const BUFFER_TYPE* buf);
+    BUFFER_TYPE* writeUint8(BUFFER_TYPE* buf, uint8_t octet);
     uint16_t readUint16(const BUFFER_TYPE * buf);
     BUFFER_TYPE * writeUint16(BUFFER_TYPE * buf, uint16_t word);
     uint32_t readUint32(const BUFFER_TYPE * buf);
     BUFFER_TYPE * writeUint32(BUFFER_TYPE * buf, uint32_t dword);
     uint64_t readUint64(const BUFFER_TYPE * buf);
     BUFFER_TYPE * writeUint64(BUFFER_TYPE * buf, uint64_t qword);
+    BUFFER_TYPE* writeData(BUFFER_TYPE* buf,  BUFFER_TYPE* data, size_t len);
 #ifdef __cplusplus 
 }
 #endif
@@ -103,6 +83,8 @@ struct iface {
     int  globaladdrcount;          /* number of global IPV6 addresses */
     int  link_state;               /* used in link change detection routines */
     unsigned int flags;            /* look IF_xxx in portable.h */
+    unsigned char m_bit;           /* M bit in RA received? */
+    unsigned char o_bit;           /* O bit in RA received? */
     struct iface* next;            /* structure describing next iface in system */
 };
 
@@ -146,53 +128,52 @@ struct link_state_notify_t
 #define REQLOG_FILE        "dibbler-requestor.log"
 
 #ifdef WIN32
-#define WORKDIR            ".\\"
-#define DEFAULT_SCRIPT     ""
-#define CLNTCONF_FILE 	   "client.conf"
-#define SRVCONF_FILE       "server.conf"
-#define RELCONF_FILE       "relay.conf"
-#define CLNTLOG_FILE       "dibbler-client.log"
-#define SRVLOG_FILE        "dibbler-server.log"
-#define RELLOG_FILE        "dibbler-relay.log"
-#define NULLFILE           "nul"
+#define DEFAULT_WORKDIR       ".\\"
+#define DEFAULT_SCRIPT        ""
+#define DEFAULT_CLNTCONF_FILE "client.conf"
+#define SRVCONF_FILE          "server.conf"
+#define RELCONF_FILE          "relay.conf"
+#define DEFAULT_CLNTLOG_FILE  "dibbler-client.log"
+#define SRVLOG_FILE           "dibbler-server.log"
+#define RELLOG_FILE           "dibbler-relay.log"
+#define CLNT_AAASPI_FILE      "AAA-SPI"
+#define SRV_KEYMAP_FILE       "keys-mapping"
+#define NULLFILE              "nul"
+/* specifies if client should remove any configured DNS servers when configuring
+   DNS servers for the first time. This makes sense on WIN32 only. */
+#define FLUSH_OTHER_CONFIGURED_DNS_SERVERS true
 #endif
 
-#ifdef LINUX
-#define WORKDIR            "/var/lib/dibbler"
+#if defined(LINUX) || defined(BSD) || defined(SUNOS)
+
+#define DEFAULT_WORKDIR       "/var/lib/dibbler"
+#define DEFAULT_CLNTCONF_FILE "/etc/dibbler/client.conf"
+#define DEFAULT_CLNTPID_FILE  "/var/lib/dibbler/client.pid"
+#define DEFAULT_CLNTLOG_FILE  "/var/log/dibbler/dibbler-client.log"
+
 #define DEFAULT_SCRIPT     ""
-#define CLNTCONF_FILE      "/etc/dibbler/client.conf"
 #define SRVCONF_FILE       "/etc/dibbler/server.conf"
 #define RELCONF_FILE       "/etc/dibbler/relay.conf"
 #define RESOLVCONF_FILE    "/etc/resolv.conf"
 #define NTPCONF_FILE       "/etc/ntp.conf"
+#define RADVD_FILE         "/etc/dibbler/radvd.conf"
+#define SRVPID_FILE        "/var/lib/dibbler/server.pid"
+#define RELPID_FILE        "/var/lib/dibbler/relay.pid"
+#define SRVLOG_FILE        "/var/log/dibbler/dibbler-server.log"
+#define RELLOG_FILE        "/var/log/dibbler/dibbler-relay.log"
+#define CLNT_AAASPI_FILE   "/var/lib/dibbler/AAA/AAA-SPI"
+#define SRV_KEYMAP_FILE    "/var/lib/dibbler/AAA/keys-mapping"
+#define NULLFILE           "/dev/null"
+
+/* those defines were initially used on Linux only, but hopefully 
+   they will work on other Posix systems as well */
+#define RESOLVCONF         "/sbin/resolvconf"
 #define TIMEZONE_FILE      "/etc/localtime"
 #define TIMEZONES_DIR      "/usr/share/zoneinfo"
-#define RADVD_FILE         "/etc/dibbler/radvd.conf"
-#define CLNTPID_FILE       "/var/lib/dibbler/client.pid"
-#define SRVPID_FILE        "/var/lib/dibbler/server.pid"
-#define RELPID_FILE        "/var/lib/dibbler/relay.pid"
-#define CLNTLOG_FILE       "/var/log/dibbler/dibbler-client.log"
-#define SRVLOG_FILE        "/var/log/dibbler/dibbler-server.log"
-#define RELLOG_FILE        "/var/log/dibbler/dibbler-relay.log"
-#define NULLFILE           "/dev/null"
-#endif
 
-#ifdef BSD
-#define WORKDIR            "/var/lib/dibbler"
-#define DEFAULT_SCRIPT     ""
-#define CLNTCONF_FILE      "/etc/dibbler/client.conf"
-#define SRVCONF_FILE       "/etc/dibbler/server.conf"
-#define RELCONF_FILE       "/etc/dibbler/relay.conf"
-#define RESOLVCONF_FILE    "/etc/resolv.conf"
-#define NTPCONF_FILE       "/etc/ntp.conf"
-#define RADVD_FILE         "/etc/dibbler/radvd.conf"
-#define CLNTPID_FILE       "/var/lib/dibbler/client.pid"
-#define SRVPID_FILE        "/var/lib/dibbler/server.pid"
-#define RELPID_FILE        "/var/lib/dibbler/relay.pid"
-#define CLNTLOG_FILE       "/var/log/dibbler/dibbler-client.log"
-#define SRVLOG_FILE        "/var/log/dibbler/dibbler-server.log"
-#define RELLOG_FILE        "/var/log/dibbler/dibbler-relay.log"
-#define NULLFILE           "/dev/null"
+/* specifies if client should remove any configured DNS servers when configuring
+   DNS servers for the first time. This makes sense on WIN32 only. */
+#define FLUSH_OTHER_CONFIGURED_DNS_SERVERS false
 #endif
 
 /* --- options --- */
@@ -217,28 +198,15 @@ struct link_state_notify_t
 /* *** interface flags ************************************************** */
 /* ********************************************************************** */
 /* those flags are used to parse flags in the structure
-   returned by if_list_get(). They are highly system specific. */
-#define LOGLEVEL          0
+   returned by if_list_get(). They are highly system specific.
+   Posix systems use values imported from headers */
 
 #ifdef WIN32
-#define IF_RUNNING        0x1
-#define IF_UP             0x1
-#define IF_MULTICAST      0x4
-#define IF_LOOPBACK       0x8
-#endif
-
-#ifdef LINUX
-#define IF_UP		      0x1
-#define IF_LOOPBACK	      0x8
-#define IF_RUNNING	      0x40
-#define IF_MULTICAST	      0x1000
-#endif
-
-#ifdef BSD
-#define IF_UP        0x1
-#define IF_LOOPBACK  0x8
-#define IF_RUNNING   0x40
-#define IF_MULTICAST 0x8000
+#define IFF_RUNNING        IFF_UP
+/* those defines are in ws2ipdef.h
+#define IFF_UP             0x1
+#define IFF_MULTICAST      0x4
+#define IFF_LOOPBACK       0x8 */
 #endif
 
 /* ********************************************************************** */
@@ -272,8 +240,6 @@ struct link_state_notify_t
 #include <time.h>
 #endif
 
-#define now() (unsigned long) time(NULL)
-
 /* ********************************************************************** */
 /* *** interface/socket low level functions ***************************** */
 /* ********************************************************************** */
@@ -301,6 +267,23 @@ unsigned long pref, unsigned long valid, int prefixLength);
     extern int sock_del(int fd);
     extern int sock_send(int fd, char* addr, char* buf, int buflen, int port, int iface);
     extern int sock_recv(int fd, char* myPlainAddr, char* peerPlainAddr, char* buf, int buflen);
+
+    /** @brief gets MAC address from the specified IPv6 address
+     *
+     *  This is called immediately after we received message from that address,
+     *  so the neighbor cache is guaranteed to have entry for it. This is highly
+     *  os specific
+     *
+     * @param iface_name network interface name
+     * @param ifindex network interface index
+     * @param v6addr text represenation of the IPv6 address
+     * @param mac pointer to MAC buffer (length specified in mac_len)
+     * @param mac_len specifies MAC buffer length. Must be set so number of bytes set by the function
+     *
+     * @return 0 if successful, negative values if there are errors
+     */
+    extern int get_mac_from_ipv6(const char* iface_name, int ifindex, const char* v6addr,
+                                 char* mac, int* mac_len);
     
     /* pack/unpack address */
     extern void print_packed(char addr[]);
@@ -312,11 +295,10 @@ unsigned long pref, unsigned long valid, int prefixLength);
     extern void doRevDnsZoneRoot( char * src,  char * dst, int lenght);
     extern void truncatePrefixFromConfig( char * src,  char * dst, char lenght);
     extern int is_addr_tentative(char* ifacename, int iface, char* plainAddr);
-    extern void link_state_change_init(volatile struct link_state_notify_t * changed_links, volatile int * notify);
+    extern void link_state_change_init(volatile struct link_state_notify_t * changed_links,
+                                       volatile int * notify);
     extern void link_state_change_cleanup();
     extern void microsleep(int microsecs);
-    uint64_t htonll(uint64_t n);
-    uint64_t ntohll(uint64_t n);
     extern char * error_message();
 
     /* options */
@@ -353,6 +335,20 @@ unsigned long pref, unsigned long valid, int prefixLength);
     uint32_t getAAASPIfromFile();
 
     int execute(const char *filename, const char * argv[], const char *env[]);
+
+    /** @brief fills specified buffer with random data
+     * @param buffer random data will be written here
+     * @param len length of the buffer
+     */
+    void fill_random(uint8_t* buffer, int len);
+
+    /** @brief returns host name of this host
+     *
+     * @param hostname buffer (hostname will be stored here)
+     * @param hostname_len length of the buffer
+     * @return LOWLEVEL_NO_ERROR if successful, appropriate LOWLEVEL_ERROR_* otherwise
+     */
+    int get_hostname(char* hostname, int hostname_len);
     
 #ifdef __cplusplus
 }
