@@ -47,7 +47,7 @@ char * TOptAddrLst::storeSelf(char* buf) {
     return buf;
 }
 
-int TOptAddrLst::getSize()
+size_t TOptAddrLst::getSize()
 {
     return 4+16*AddrLst.count();
 }
@@ -61,13 +61,13 @@ SPtr<TIPv6Addr> TOptAddrLst::getAddr()
     return this->AddrLst.get();
 }
 
-bool TOptAddrLst::isValid()
+bool TOptAddrLst::isValid() const
 {
     return this->Valid;
 }
 
 std::string TOptAddrLst::getPlain() {
-    stringstream tmp;
+    std::stringstream tmp;
     AddrLst.first();
     SPtr<TIPv6Addr> addr;
     while (addr = AddrLst.get()) {
