@@ -20,12 +20,12 @@ public:
     TOptAddrLst(int type, List(TIPv6Addr) lst, TMsg* parent);
     TOptAddrLst(int type, const char *buf, unsigned short len, TMsg* parent);
     char * storeSelf( char* buf);
-    int getSize();
+    size_t getSize();
     void firstAddr();
     SPtr<TIPv6Addr> getAddr();
     const List(TIPv6Addr)& getAddrLst() { return AddrLst; }
     int countAddr();
-    bool isValid();
+    bool isValid() const;
     virtual bool doDuties() { return true; } // does nothing on its own
     std::string getPlain();
 protected:
