@@ -20,12 +20,12 @@ class TOptDUID : public TOpt
   public:
     TOptDUID(int type, SPtr<TDUID> duid, TMsg* parent);
     TOptDUID(int type, const char* buf, int len, TMsg* parent);
-    int getSize();
+    size_t getSize();
 
     char * storeSelf(char* buf);
     bool doDuties() { return true; }
-    SPtr<TDUID> getDUID();
-    bool isValid();
+    SPtr<TDUID> getDUID() const;
+    virtual bool isValid() const;
     std::string getPlain() { if (DUID) return DUID->getPlain(); else return std::string(""); }
   protected:
     SPtr<TDUID> DUID;
