@@ -6,8 +6,6 @@
  *
  * released under GNU GPL v2 licence
  *
- * $Id: OptGeneric.cpp,v 1.4 2008-11-11 22:41:48 thomson Exp $
- *
  */
 
 #include <stdlib.h>
@@ -16,6 +14,9 @@
 #include <sstream>
 #include "DHCPConst.h"
 #include "OptGeneric.h"
+#include "hex.h"
+
+using namespace std;
 
 TOptGeneric::TOptGeneric(int optType, const char* data, unsigned short dataLen, TMsg* parent)
     :TOpt(optType, parent) {
@@ -36,13 +37,12 @@ TOptGeneric::~TOptGeneric() {
     }
 }
 
-int TOptGeneric::getSize()
+size_t TOptGeneric::getSize()
 {
-    return this->DataLen+4;
+    return DataLen + 4;
 }
 
- char * TOptGeneric::storeSelf( char* buf)
-{
+char * TOptGeneric::storeSelf(char* buf) {
     buf = writeUint16(buf, OptType);
     buf = writeUint16(buf, this->DataLen);
     memmove(buf, this->Data, this->DataLen);
@@ -51,26 +51,15 @@ int TOptGeneric::getSize()
 
 std::string TOptGeneric::getPlain()
 {
-    stringstream plain;
-    stringstream hex;
-    bool printable = true;
-    for (int i=0;i<DataLen; ++i)
-    {
-	if (!isalpha(Data[i]) && Data[i]!=32/* spc */) {
-	    printable = false;
-	}
-	plain << Data[i];
-	hex << std::hex << setfill('0') << setw(2) << (unsigned int) Data[i];
-	if ((i<DataLen-1))
-	    hex << ":";
-    }
-    if (printable)
-	return plain.str();
-    else
-	return hex.str();
+    return hexToText((uint8_t*)(Data), DataLen, false);
 }
 
-bool TOptGeneric::isValid()
+bool TOptGeneric::isValid() const
 {
     return true;
 }
+
+bool TOptGeneric::operator == (const TOptGeneric &other) {
+    /// @todo: Implement comparison for real
+    return false;
+}
