@@ -20,20 +20,18 @@
 class TOptGeneric : public TOpt
 {
   public:
-    bool operator == (const TOptGeneric &other) const {
-	return false;
-    }
+    bool operator == (const TOptGeneric &other);
 
     TOptGeneric(int optType, const char * data, unsigned short dataLen, TMsg* parent);
     TOptGeneric(int optType, TMsg* parent);
     ~TOptGeneric();
-    int getSize();
+    size_t getSize();
     void setData(char * data, int dataLen);
     std::string getPlain();
     bool doDuties() { return true; }
 
     char * storeSelf(char* buf);
-    bool isValid();
+    virtual bool isValid() const;
   protected:
     char * Data;
     int DataLen;
