@@ -21,22 +21,22 @@ class TOptIAAddress : public TOpt
   public:
     TOptIAAddress(char* &addr, int& n, TMsg* parent);
     TOptIAAddress( SPtr<TIPv6Addr> addr, unsigned long pref, unsigned long valid, TMsg* parent);
-    int getSize();
-    
+    size_t getSize();
+
     char * storeSelf( char* buf);
-    SPtr<TIPv6Addr> getAddr();
-    unsigned long getPref();
-    unsigned long getValid();
+    SPtr<TIPv6Addr> getAddr() const;
+    unsigned long getPref() const;
+    unsigned long getValid() const;
     void setPref(unsigned long pref);
     void setValid(unsigned long valid);
-    bool isValid();
+    bool isValid() const;
 
     virtual bool doDuties() { return true; } // does nothing on its own
  private:
-    SPtr<TIPv6Addr> Addr;
-    unsigned long Valid;
-    unsigned long Pref;
-    bool ValidOpt;
+    SPtr<TIPv6Addr> Addr_;
+    unsigned long ValidLifetime_;
+    unsigned long PrefLifetime_;
+    bool Valid_;
 };
 
 #endif
