@@ -14,46 +14,51 @@
 #include "OptStatusCode.h"
 //#include "OptRapidCommit.h"
 
-TOptIA_NA::TOptIA_NA( long IAID, long t1,  long t2, TMsg* parent)
-	:TOpt(OPTION_IA_NA, parent) {
-    this->IAID = IAID;
-    this->T1   = t1;
-    this->T2   = t2;
+TOptIA_NA::TOptIA_NA(long IAID, long t1,  long t2, TMsg* parent)
+    :TOpt(OPTION_IA_NA, parent), IAID_(IAID), T1_(t1), T2_(t2), Valid_(true) {
 }
 
-unsigned long TOptIA_NA::getIAID() {
-    return IAID;
+void TOptIA_NA::setIAID(uint32_t iaid) {
+    IAID_ = iaid;
 }
 
-unsigned long TOptIA_NA::getT1() {
-    return T1;
+unsigned long TOptIA_NA::getIAID() const {
+    return IAID_;
+}
+
+unsigned long TOptIA_NA::getT1() const {
+    return T1_;
 }
 
 void TOptIA_NA::setT1(unsigned long t1) {
-    this->T1 = t1;
+    T1_ = t1;
 }
 
-unsigned long TOptIA_NA::getT2() {
-    return T2;
+unsigned long TOptIA_NA::getT2() const {
+    return T2_;
 }
 
 void TOptIA_NA::setT2(unsigned long t2) {
-    this->T2 = t2;
+    T2_ = t2;
 }
 
 TOptIA_NA::TOptIA_NA( char * &buf, int &bufsize, TMsg* parent)
-    :TOpt(OPTION_IA_NA, parent) {
-    if (bufsize<12) {
-        Valid=false;
-        bufsize=0;
+    :TOpt(OPTION_IA_NA, parent), Valid_(false) {
+    if (bufsize < 12) {
+        buf += bufsize;
+        bufsize = 0;
     } else {
-        Valid=true;
-        IAID = readUint32(buf);
-        buf += sizeof(uint32_t); bufsize -= sizeof(uint32_t);
-        T1 = readUint32(buf);
-        buf += sizeof(uint32_t); bufsize -= sizeof(uint32_t);
-        T2 = readUint32(buf);
+        IAID_ = readUint32(buf);
+        buf += sizeof(uint32_t);
+        bufsize -= sizeof(uint32_t);
+
+        T1_ = readUint32(buf);
+        buf += sizeof(uint32_t);
+        bufsize -= sizeof(uint32_t);
+
+        T2_ = readUint32(buf);
         buf += sizeof(uint32_t); bufsize -= sizeof(uint32_t);
+        Valid_ = true;
     }
 }
 
@@ -62,27 +67,27 @@ int TOptIA_NA::getStatusCode() {
     SPtr<TOpt> ptrOpt;
     SubOptions.first();
     while ( ptrOpt = SubOptions.get() ) {
-	if ( ptrOpt->getOptType() == OPTION_STATUS_CODE) {
-	    SPtr <TOptStatusCode> ptrStatus;
-	    ptrStatus = (Ptr*) ptrOpt;
-	    return ptrStatus->getCode();
-	}
+        if ( ptrOpt->getOptType() == OPTION_STATUS_CODE) {
+            SPtr <TOptStatusCode> ptrStatus;
+            ptrStatus = (Ptr*) ptrOpt;
+            return ptrStatus->getCode();
+        }
     }
     return -1;
 }
 
-int TOptIA_NA::getSize() {
+size_t TOptIA_NA::getSize() {
     int mySize = 16;
-    return mySize+getSubOptSize();
+    return mySize + getSubOptSize();
 }
 
 char * TOptIA_NA::storeSelf( char* buf) {
     buf = writeUint16(buf, OptType);
     buf = writeUint16(buf, getSize() - 4 );
 
-    buf = writeUint32(buf, IAID);
-    buf = writeUint32(buf, T1);
-    buf = writeUint32(buf, T2);
+    buf = writeUint32(buf, IAID_);
+    buf = writeUint32(buf, T1_);
+    buf = writeUint32(buf, T2_);
 
     buf = storeSubOpt(buf);
     return buf;
@@ -99,11 +104,11 @@ unsigned long TOptIA_NA::getMaxValid() {
             if (maxValid<ptrIAAddr->getValid())
                 maxValid=ptrIAAddr->getValid();
         }
-    }   
+    }
     return maxValid;
 }
 
-bool TOptIA_NA::isValid() {
+bool TOptIA_NA::isValid() const {
     return this->Valid;
 }
 
@@ -115,8 +120,8 @@ int TOptIA_NA::countAddrs() {
     SPtr<TOpt> opt;
     this->firstOption();
     while (opt = this->getOption() ) {
-	if (opt->getOptType() == OPTION_IAADDR)
-	    cnt++;
+        if (opt->getOptType() == OPTION_IAADDR)
+            cnt++;
     }
     return cnt;
 }
