@@ -6,12 +6,6 @@
  *
  * released under GNU GPL v2 only licence
  *
- * $Id: OptStatusCode.h,v 1.3 2008-08-29 00:07:31 thomson Exp $
- *
- * $Log: not supported by cvs2svn $
- * Revision 1.2  2004-12-08 00:17:04  thomson
- * DOS end of lines removed, header added.
- *
  */
 
 #ifndef OPTSTATUSCODE_H
@@ -24,20 +18,20 @@
 class TOptStatusCode : public TOpt
 {
   public:
-    TOptStatusCode( char * &buf,  int &len, TMsg* parent);
-    TOptStatusCode(int status,string Message, TMsg* parent);
+    TOptStatusCode(const char * buf, size_t len, TMsg* parent);
+    TOptStatusCode(int status, const std::string& Message, TMsg* parent);
     
-    int getSize();
+    size_t getSize();
     char * storeSelf( char* buf);
     
     int getCode();  
-    string getText();
+    std::string getText();
     
     bool doDuties();    
  private:
-    string Message;
-    int StatusCode;
-    bool Valid;
+    std::string Message_;
+    int StatusCode_;
+    bool Valid_;
 };
 
 #endif 
