@@ -21,8 +21,8 @@ class TOptString : public TOpt
     TOptString(int type, std::string str, TMsg* parent);
     TOptString(int type, const char *buf, unsigned short len, TMsg* parent);
     char * storeSelf( char* buf);
-    int getSize();
-    string getString();
+    size_t getSize();
+    std::string getString();
     virtual bool doDuties() { return true; } // do nothing, actual code in ClntOpt* classes
     std::string getPlain() { return Str; }
  protected:
