@@ -666,8 +666,6 @@ int iproute_modify(int cmd, unsigned flags, int argc, char **argv)
 	int nhs_ok = 0;
 	int scope_ok = 0;
 	int table_ok = 0;
-	int proto_ok = 0;
-	int type_ok = 0;
 
 	memset(&req, 0, sizeof(req));
 
@@ -734,6 +732,20 @@ int iproute_modify(int cmd, unsigned flags, int argc, char **argv)
 				invarg("invalid \"scope\" value\n", *argv);
 			req.r.rtm_scope = scope;
 			scope_ok = 1;
+
+                } else if (strcmp(*argv, "lifetime") == 0) {
+                    /* this one is added by thomson */
+                        static struct ifa_cacheinfo ci;
+                        memset(&ci, 0, sizeof(ci));
+
+                        NEXT_ARG();
+                        __u32 validlft;
+                        if (get_u32(&validlft, *argv, 0))
+                            invarg("\lifetime value is invalid\n", *argv);
+
+                        ci.ifa_valid = validlft;
+                        ci.ifa_prefered = validlft - 1;
+                        addattr_l(&req.n, sizeof(req), IFA_CACHEINFO, &ci, sizeof(ci));
 		} else if (strcmp(*argv, "mtu") == 0) {
 			unsigned mtu;
 			NEXT_ARG();
@@ -838,7 +850,6 @@ int iproute_modify(int cmd, unsigned flags, int argc, char **argv)
 			if (rtnl_rtprot_a2n(&prot, *argv))
 				invarg("\"protocol\" value is invalid\n", *argv);
 			req.r.rtm_protocol = prot;
-			proto_ok =1;
 		} else if (matches(*argv, "table") == 0) {
 			__u32 tid;
 			NEXT_ARG();
@@ -877,7 +888,6 @@ int iproute_modify(int cmd, unsigned flags, int argc, char **argv)
 			    rtnl_rtntype_a2n(&type, *argv) == 0) {
 				NEXT_ARG();
 				req.r.rtm_type = type;
-				type_ok = 1;
 			}
 
 			if (matches(*argv, "help") == 0)
@@ -896,17 +906,17 @@ int iproute_modify(int cmd, unsigned flags, int argc, char **argv)
 	}
 
 	if (d || nhs_ok)  {
-		int idx;
 
-		ll_init_map(&rth);
+            ll_init_map(&rth);
 
-		if (d) {
-			if ((idx = ll_name_to_index(d)) == 0) {
-				fprintf(stderr, "Cannot find device \"%s\"\n", d);
-				return -1;
-			}
-			addattr32(&req.n, sizeof(req), RTA_OIF, idx);
-		}
+            if (d) {
+                int idx;
+                if ((idx = ll_name_to_index(d)) == 0) {
+                    fprintf(stderr, "Cannot find device \"%s\"\n", d);
+                    return -1;
+                }
+                addattr32(&req.n, sizeof(req), RTA_OIF, idx);
+            }
 	}
 
 	if (mxrta->rta_len > RTA_LENGTH(0)) {
@@ -988,6 +998,7 @@ static int iproute_flush_cache(void)
 		
 	if ((write (flush_fd, (void *)buffer, len)) < len) {
 		fprintf (stderr, "Cannot flush routing cache\n");
+                close(flush_fd);
 		return -1;
 	}
 	close(flush_fd);
