@@ -6,13 +6,18 @@
  *
  * released under GNU GPL v2 licence
  *
- * $Id: lowlevel-win32.c,v 1.22 2008-10-13 22:41:18 thomson Exp $
- *
  */
 
 #define WIN32_LEAN_AND_MEAN
+
+/* tells stdlib.h to include rand_s - more secure version */
+#define _CRT_RAND_S
 #include <winsock2.h>
 
+/* causes compilation errors on Visual Studio 2008 because of IPAddr type being undefined */
+/* commented out for now */
+/* #include <icmpapi.h> */
+
 #include <Ws2tcpip.h.>
 #include <Ws2spi.h>
 
@@ -80,7 +85,7 @@ uint32_t getAAASPIfromFile() {
     if (!file)
         return 0;
 
-    fscanf(file, "%x", &ret);
+    fscanf(file, "%10x", &ret);
     fclose(file);
 
     return ret;
@@ -116,7 +121,6 @@ char * getAAAKey(uint32_t SPI, uint32_t *len) {
     if (0 > fd)
         return NULL;
 
-    /** @todo should be freed somewhere */
     retval = malloc(st.st_size);
     if (!retval)
         return NULL;
@@ -125,14 +129,17 @@ char * getAAAKey(uint32_t SPI, uint32_t *len) {
         ret = read(fd, retval + offset, st.st_size - offset);
         if (!ret) break;
         if (ret < 0) {
+            free(retval);
             return NULL;
         }
         offset += ret;
     }
     close(fd);
 
-    if (offset != st.st_size)
+    if (offset != st.st_size) {
+        free(retval);
         return NULL;
+    }
 
     *len = st.st_size;
     return retval;
@@ -254,10 +261,10 @@ extern	struct iface* if_list_get()
 
     	//set interface flags
     	iface->flags=0;
-    	if (adaptaddr->OperStatus==IfOperStatusUp)
-	        iface->flags|=IF_UP|IF_RUNNING|IF_MULTICAST;
+    	if (adaptaddr->OperStatus == IfOperStatusUp)
+	        iface->flags |= IFF_UP|IFF_RUNNING|IFF_MULTICAST;
 	    if (adaptaddr->IfType==IF_TYPE_SOFTWARE_LOOPBACK)
-    	    iface->flags|=IF_LOOPBACK;
+    	    iface->flags |= IFF_LOOPBACK;
 	
     	//go to next returned adapter
     	if (adaptaddr->Next)
@@ -280,7 +287,6 @@ extern int is_addr_tentative(char* ifacename, int iface, char* plainAddr)
     PIP_ADAPTER_UNICAST_ADDRESS linkaddr;
     char netAddr[16];
     unsigned long buflen=0;
-    int retVal=-1;
     
     PIP_ADAPTER_UNICAST_ADDRESS found=NULL;
     
@@ -308,11 +314,11 @@ extern int is_addr_tentative(char* ifacename, int iface, char* plainAddr)
     
     free(buffer);
     if (!found)
-        return TENTATIVE_UNKNOWN; /* not found */
+        return ADDRSTATUS_UNKNOWN; /* not found */
     if (found->DadState==IpDadStateDuplicate)
-        return TENTATIVE_YES;     /* tentative */
+        return ADDRSTATUS_YES;     /* tentative */
     else
-        return TENTATIVE_NO;      /* not tentative */
+        return ADDRSTATUS_NO;      /* not tentative */
 }
 extern int ipaddr_add(const char * ifacename, int ifaceid, const char * addr, 
                       unsigned long pref, unsigned long valid, int prefixLen)
@@ -329,10 +335,11 @@ extern int ipaddr_add(const char * ifacename, int ifaceid, const char * addr,
     intptr_t i;
     sprintf(arg5,"interface=\"%s\"", ifacename);
     sprintf(arg6,"address=%s", addr);
-    sprintf(arg7,"validlifetime=%d", valid);
-    sprintf(arg8,"preferredlifetime=%d", pref);
+    sprintf(arg7,"validlifetime=%u", valid);
+    sprintf(arg8,"preferredlifetime=%u", pref);
     // use _P_DETACH to speed things up, (but the tentative detection will surely fail)
     i=_spawnl(_P_WAIT, netshPath, netshPath, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, NULL);
+    if (i) { i = _spawnl(_P_WAIT, netshPath, netshPath, arg1, arg2, "set", arg4, arg5, arg6, arg7, arg8, NULL); }
     return i;
 }
 
@@ -368,7 +375,6 @@ extern int sock_add(char * ifacename,int ifaceid, char * addr, int port, int thi
     struct ipv6_mreq ipmreq; 
     int	hops=1;
     char packedAddr[16];
-    char multiAddr[16] = { 0xff,2, 0,0, 0,0, 0,0, 0,0, 0,0, 0,1, 0,2};
     
     inet_pton6(addr,packedAddr);
     if ((s=socket(AF_INET6,SOCK_DGRAM, 0)) == INVALID_SOCKET)
@@ -424,6 +430,11 @@ int sock_send(int fd, char * addr, char * buf, int buflen, int port,int iface)
     int i;
     char packaddr[16];
     char ifaceStr[10];
+
+    memset(addrStr,  0, sizeof(addrStr));
+    memset(portStr,  0, sizeof(portStr));
+    memset(packaddr, 0, sizeof(packaddr));
+    memset(ifaceStr, 0, sizeof(ifaceStr));
     
     strcpy(addrStr,addr);
     itoa(port,portStr,10);
@@ -482,9 +493,14 @@ extern int dns_add(const char* ifname, int ifaceid, const char* addrPlain) {
     
     sprintf(arg5,"\"%s\"", ifname);
     sprintf(arg6,"address=%s", addrPlain);
-    i=_spawnl(_P_DETACH,netshPath,netshPath,arg1,arg2,arg3,arg4,arg5,arg6,NULL);
-
-    return LOWLEVEL_NO_ERROR;
+    i=_spawnl(_P_WAIT,netshPath,netshPath,arg1,arg2,arg3,arg4,arg5,arg6,NULL);
+    if (i == 0) {
+        return LOWLEVEL_NO_ERROR;
+    } else {
+        sprintf(Message, "%s %s %s %s %s %s %s returned non-zero returncode %d",
+                netshPath, arg1, arg2, arg3, arg4, arg5, arg6, i);
+        return LOWLEVEL_ERROR_UNSPEC;
+    }
 }
 extern int dns_del(const char* ifname, int ifaceid, const char* addrPlain) {
     // netsh interface ipv6 add dns interface="eth0" address=2000::123
@@ -496,11 +512,19 @@ extern int dns_del(const char* ifname, int ifaceid, const char* addrPlain) {
     char arg6[256]; // address=...
     intptr_t i;
     sprintf(arg5,"\"%s\"", ifname);
-    sprintf(arg6,"address=%s", addrPlain);
-    i=_spawnl(_P_DETACH,netshPath,netshPath,arg1,arg2,arg3,arg4,arg5,arg6,NULL);
+    if (addrPlain)
+		sprintf(arg6,"address=%s", addrPlain);
+	else
+		sprintf(arg6,"all");
+    i=_spawnl(_P_WAIT,netshPath,netshPath,arg1,arg2,arg3,arg4,arg5,arg6,NULL);
 
-    /// @todo: check status
-    return LOWLEVEL_NO_ERROR;
+    if (i == 0) {
+        return LOWLEVEL_NO_ERROR;
+    } else {
+        sprintf(Message, "%s %s %s %s %s %s %s returned non-zero returncode %d",
+                netshPath, arg1, arg2, arg3, arg4, arg5, arg6, i);
+        return LOWLEVEL_ERROR_UNSPEC;
+    }
 }
 
 extern int domain_add(const char* ifname, int ifaceid, const char* domain) {
@@ -594,18 +618,19 @@ int prefix_add(const char* ifname, int ifindex, const char* prefixPlain, int pre
     
     sprintf(arg5, "%s/%d", prefixPlain, prefixLength);
     sprintf(arg6,"interface=\"%s\"", ifname);
-    sprintf(arg7,"preferredlifetime=%d", prefered);
-    sprintf(arg8,"validlifetime=%d", valid);
+    sprintf(arg7,"preferredlifetime=%u", prefered);
+    sprintf(arg8,"validlifetime=%u", valid);
 
     sprintf(buf, "%s %s %s %s %s %s %s %s %s %s", arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10);
-    i=_spawnl(_P_DETACH,netshPath,netshPath,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10, NULL);
+    i=_spawnl(_P_WAIT,netshPath,netshPath,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10, NULL);
 
-    if (i==-1) {
-        /// @todo: some better error support
-        return -1;
+    // if error code is non-zero, then the addition failed. One of the reasons why this
+    // could happen is because the address or prefix already exists, so we'll try
+    // to update its parameters (if this is RENEW/REBIND)
+    if (i) {
+        i = _spawnl(_P_WAIT, netshPath, netshPath, arg1, arg2, "set", arg4, arg5, arg6, arg7, arg8, arg9, arg10, NULL);
     }
-
-    return LOWLEVEL_NO_ERROR;
+    return i;
 }
 
 int prefix_update(const char* ifname, int ifindex, const char* prefixPlain, int prefixLength,
@@ -627,7 +652,7 @@ int prefix_del(const char* ifname, int ifindex, const char* prefixPlain, int pre
     
     sprintf(arg5, "%s/%d", prefixPlain, prefixLength);
     sprintf(arg6,"interface=\"%s\"", ifname);
-    i=_spawnl(_P_DETACH,netshPath,netshPath,arg1,arg2,arg3,arg4,arg5,arg6, NULL);
+    i=_spawnl(_P_WAIT,netshPath,netshPath,arg1,arg2,arg3,arg4,arg5,arg6, NULL);
 
     if (i==-1) {
         /// @todo: some better error support
@@ -647,10 +672,39 @@ void link_state_change_cleanup()
    /// @todo: implement this
 }
 
-int execute(const char *filename, char * argv[], char *env[])
+int execute(const char *filename, const char * argv[], const char *env[])
 {
     intptr_t i;
-	i=_spawnvpe(_P_WAIT, filename, argv, env);
+    i=_spawnvpe(_P_WAIT, filename, argv, env);
+    return i;
+}
 
-	return i;
-}
\ No newline at end of file
+int get_mac_from_ipv6(const char* iface_name, int ifindex, const char* v6addr,
+                      char* mac, int* mac_len) {
+    /// @todo: Implement MAC reading for Windows
+    return LOWLEVEL_ERROR_NOT_IMPLEMENTED;
+}
+
+/** @brief returns host name of this host
+ *
+ * @param hostname buffer (hostname will be stored here)
+ * @param hostname_len length of the buffer
+ * @return LOWLEVEL_NO_ERROR if successful, appropriate LOWLEVEL_ERROR_* otherwise
+ */
+int get_hostname(char* hostname, int hostname_len) {
+    memset(hostname,0, hostname_len);
+    if (GetComputerNameExA(ComputerNameDnsFullyQualified, hostname, &hostname_len)) {
+        return LOWLEVEL_NO_ERROR;
+    }
+    return LOWLEVEL_ERROR_UNSPEC;
+}
+
+void fill_random(uint8_t* buffer, int len) {
+    unsigned int number;
+    int i = 0;
+
+    for (i=0; i < len; ++i) {
+        rand_s(&number);
+        buffer[i] = (number%256);
+    }
+}
