@@ -5,28 +5,28 @@
  *
  * released under GNU GPL v2 only licence
  *
- * $Id: RelCfgMgr.cpp,v 1.13 2008-08-29 00:07:31 thomson Exp $
- *
  */
 
+#include <cstdlib>
+#include <unistd.h>
 #include <iostream>
 #include <fstream>
 #include <string>
 #include "IfaceMgr.h"
 #include "RelIfaceMgr.h"
 #include "RelCfgIface.h"
+
+using namespace std;
 #include "FlexLexer.h"
 #include "RelParser.h"
 #include "RelCfgMgr.h"
 
-using namespace std;
-
 int TRelCfgMgr::NextRelayID = RELAY_MIN_IFINDEX;
 
 TRelCfgMgr * TRelCfgMgr::Instance = 0;
 
-TRelCfgMgr::TRelCfgMgr(const std::string cfgFile, const std::string xmlFile)
-    :TCfgMgr(), XmlFile(xmlFile)
+TRelCfgMgr::TRelCfgMgr(const std::string& cfgFile, const std::string& xmlFile)
+    :TCfgMgr(), XmlFile(xmlFile), ClientLinkLayerAddress_(false)
 {
     // load config file
     if (!this->parseConfigFile(cfgFile)) {
@@ -38,7 +38,7 @@ TRelCfgMgr::TRelCfgMgr(const std::string cfgFile, const std::string xmlFile)
     this->IsDone = false;
 }
 
-bool TRelCfgMgr::parseConfigFile(string cfgFile) {
+bool TRelCfgMgr::parseConfigFile(const std::string& cfgFile) {
     int result;
     ifstream f;
 
@@ -53,6 +53,7 @@ bool TRelCfgMgr::parseConfigFile(string cfgFile) {
     }
     yyFlexLexer lexer(&f,&clog);
     RelParser parser(&lexer);
+    parser.CfgMgr = this; // just a workaround to access CfgMgr while still being in constructor
     result = parser.yyparse();
     Log(Debug) << "Parsing config done." << LogEnd;
     f.close();
@@ -225,7 +226,7 @@ SPtr<TRelCfgIface> TRelCfgMgr::getIfaceByID(int iface)
 	    return ptrIface;
     }
     Log(Error) << "There is no interface with ifindex=" << iface << " in the CfgMgr." << LogEnd;
-    return 0;
+    return SPtr<TRelCfgIface>(); // NULL
 }
 
 
@@ -239,7 +240,7 @@ SPtr<TRelCfgIface> TRelCfgMgr::getIfaceByInterfaceID(int iface)
     }
     Log(Error) << "There is no interface with interfaceID=" << iface 
 	       << " in the CfgMgr." << LogEnd;
-    return 0;
+    return SPtr<TRelCfgIface>(); // NULL
 }
 
 ERelIfaceIdOrder TRelCfgMgr::getInterfaceIDOrder()
@@ -257,7 +258,7 @@ SPtr<TRelOptEcho> TRelCfgMgr::getEcho()
     return Echo;
 }
 
-void TRelCfgMgr::instanceCreate( const std::string cfgFile, const std::string xmlFile )
+void TRelCfgMgr::instanceCreate(const std::string& cfgFile, const std::string& xmlFile )
 {
     if (Instance)
         Log(Crit) << "RelCfgMgr instance already created. Application error!" << LogEnd;
@@ -266,8 +267,10 @@ void TRelCfgMgr::instanceCreate( const std::string cfgFile, const std::string xm
 
 TRelCfgMgr& TRelCfgMgr::instance()
 {
-    if (!Instance)
-        Log(Crit) << "RelCfgMgr instance not created yet. Application error. Crashing in 3... 2... 1..." << LogEnd;
+    if (!Instance) {
+        Log(Crit) << "RelCfgMgr instance not created yet. Application error. Emergency shutdown." << LogEnd;
+        exit(EXIT_FAILURE);
+    }
     return *Instance;
 }
 
@@ -324,3 +327,22 @@ ostream & operator<<(ostream &out, TRelCfgMgr &x) {
     out << "</RelCfgMgr>" << std::endl;
     return out;
 }
+
+
+void TRelCfgMgr::setRelayID(SPtr<TOpt> relayID)
+{
+    RelayID_ = relayID;
+}
+
+SPtr<TOpt> TRelCfgMgr::getRelayID()
+{
+    return RelayID_;
+}
+
+void TRelCfgMgr::setClientLinkLayerAddress(bool enabled) {
+    ClientLinkLayerAddress_ = enabled;
+}
+
+bool TRelCfgMgr::getClientLinkLayerAddress() {
+    return ClientLinkLayerAddress_;
+}
