@@ -23,13 +23,13 @@
 class TRelCfgMgr : public TCfgMgr
 {
 public:  
-    friend ostream & operator<<(std::ostream &strum, TRelCfgMgr &x);
+    friend std::ostream & operator<<(std::ostream &strum, TRelCfgMgr &x);
     virtual ~TRelCfgMgr();
 
-    static void instanceCreate(const std::string cfgFile, const std::string xmlFile);
+    static void instanceCreate(const std::string& cfgFile, const std::string& xmlFile);
     static TRelCfgMgr& instance();
 
-    bool parseConfigFile(const std::string cfgFile);
+    bool parseConfigFile(const std::string& cfgFile);
 
     //Interfaces access methods
     void firstIface();
@@ -46,16 +46,24 @@ public:
     bool setupGlobalOpts(SPtr<TRelParsGlobalOpt> opt);
 
     // configuration parameters
-    string getWorkdir();
+    std::string getWorkdir();
     bool guessMode();
     ERelIfaceIdOrder getInterfaceIDOrder();
 
     SPtr<TOptVendorData> getRemoteID();
     SPtr<TRelOptEcho>     getEcho();
 
-private:
+    void setRelayID(SPtr<TOpt> relayID);
+    SPtr<TOpt> getRelayID();
+
+    void setClientLinkLayerAddress(bool enabled);
+    bool getClientLinkLayerAddress();
+
+protected:
     static TRelCfgMgr * Instance;
-    TRelCfgMgr(const std::string cfgFile, const std::string xmlFile);
+    TRelCfgMgr(const std::string& cfgFile, const std::string& xmlFile);
+
+private:
 
     std::string XmlFile;
 
@@ -75,6 +83,9 @@ private:
 
     SPtr<TOptVendorData> RemoteID;
     SPtr<TRelOptEcho> Echo;
+    SPtr<TOpt> RelayID_;
+
+    bool ClientLinkLayerAddress_;
 };
 
 #endif /* RELCONFMGR_H */
