@@ -11,56 +11,57 @@
 #include "RelParsGlobalOpt.h"
 #include "Portable.h"
 
-TRelParsGlobalOpt::TRelParsGlobalOpt(void) {
-    WorkDir   = WORKDIR;
-    GuessMode = false;
-    InterfaceIDOrder = REL_IFACE_ID_ORDER_BEFORE;
+using namespace std;
+
+TRelParsGlobalOpt::TRelParsGlobalOpt(void) 
+    : WorkDir_(DEFAULT_WORKDIR), GuessMode_(false),
+      InterfaceIDOrder_(REL_IFACE_ID_ORDER_BEFORE) {
 }
 
 TRelParsGlobalOpt::~TRelParsGlobalOpt(void) {
 }
 
 string TRelParsGlobalOpt::getWorkDir() {
-    return this->WorkDir;
+    return WorkDir_;
 }
 
-void TRelParsGlobalOpt::setWorkDir(string dir) {
-    this->WorkDir=dir;
+void TRelParsGlobalOpt::setWorkDir(std::string dir) {
+    WorkDir_ = dir;
 }
 
 
 void TRelParsGlobalOpt::setGuessMode(bool guess) {
-    GuessMode = guess;
+    GuessMode_ = guess;
 }
 
 bool TRelParsGlobalOpt::getGuessMode() {
-    return GuessMode;
+    return GuessMode_;
 }
 
 void TRelParsGlobalOpt::setInterfaceIDOrder(ERelIfaceIdOrder order) {
-    InterfaceIDOrder = order;
+    InterfaceIDOrder_ = order;
 }
 
 ERelIfaceIdOrder TRelParsGlobalOpt::getInterfaceIDOrder() {
-    return InterfaceIDOrder;
+    return InterfaceIDOrder_;
 }
 
 void TRelParsGlobalOpt::setRemoteID(SPtr<TOptVendorData> remoteID)
 {
-    RemoteID = remoteID;
+    RemoteID_ = remoteID;
 }
 
 SPtr<TOptVendorData> TRelParsGlobalOpt::getRemoteID()
 {
-    return RemoteID;
+    return RemoteID_;
 }
 
 void TRelParsGlobalOpt::setEcho(SPtr<TRelOptEcho> echo)
 {
-    Echo = echo;
+    Echo_ = echo;
 }
 
 SPtr<TRelOptEcho> TRelParsGlobalOpt::getEcho()
 {
-    return Echo;
+    return Echo_;
 }
