@@ -7,6 +7,7 @@
  *
  */
 
+#include <cstdlib>
 #include <fstream>
 #include "Logger.h"
 #include "IPv6Addr.h"
@@ -22,10 +23,12 @@
 
 TRelIfaceMgr * TRelIfaceMgr::Instance = 0;
 
+using namespace std;
+
 /*
  * constructor. Do nothing particular, just invoke IfaceMgr constructor
  */
-TRelIfaceMgr::TRelIfaceMgr(string xmlFile) 
+TRelIfaceMgr::TRelIfaceMgr(const std::string& xmlFile) 
     : TIfaceMgr(xmlFile, true) {
 }
 
@@ -88,57 +91,59 @@ SPtr<TRelMsg> TRelIfaceMgr::select(unsigned long timeout) {
     int dataLen=2048;
 
     SPtr<TIPv6Addr> peer (new TIPv6Addr());
+    SPtr<TIPv6Addr> myaddr(new TIPv6Addr());
     int sockid;
-    int msgtype;
 
     // read data
-    sockid = TIfaceMgr::select(timeout, data, dataLen, peer);
-    if (sockid>0) {
-	if (dataLen<4) {
-	    Log(Warning) << "Received message is truncated (" << dataLen << " bytes)." << LogEnd;
-	    return 0; //NULL
-	}
-	
-	// check message type
-	msgtype = data[0];
+    sockid = TIfaceMgr::select(timeout, data, dataLen, peer, myaddr);
+    if (sockid < 0) {
+        Log(Warning) << "Socket read error: " << sockid << LogEnd;
+        return SPtr<TRelMsg>(); // NULL
+    }
 
-        if (msgtype > LEASEQUERY_REPLY_MSG) {
-            Log(Warning) << "Invalid message type " << msgtype << " received." << LogEnd;
-            return 0;
-        }
-	SPtr<TMsg> ptr;
-	SPtr<TIfaceIface> iface;
-	SPtr<TIfaceSocket> sock;
+    if (dataLen<4) {
+        Log(Warning) << "Received message is truncated (" << dataLen << " bytes)." << LogEnd;
+        return SPtr<TRelMsg>(); // NULL
+    }
 
-	// get interface
-	iface = this->getIfaceBySocket(sockid);
+    // check message type
+    int msgtype = data[0];
 
-	sock = iface->getSocketByFD(sockid);
+    if (msgtype > LEASEQUERY_REPLY_MSG) {
+        Log(Warning) << "Invalid message type " << msgtype << " received." << LogEnd;
+        return SPtr<TRelMsg>(); // NULL
+    }
+    SPtr<TMsg> ptr;
+    SPtr<TIfaceIface> iface;
+    SPtr<TIfaceSocket> sock;
 
-	Log(Debug) << "Received " << dataLen << " bytes on the " << iface->getName() << "/" 
-		   << iface->getID() << " interface (socket=" << sockid << ", addr=" << peer->getPlain() 
-		   << ", port=" << sock->getPort() << ")." << LogEnd;
-	
-	if (sock->getPort()!=DHCPSERVER_PORT) {
-	    Log(Error) << "Message was received on invalid (" << sock->getPort() << ") port." << LogEnd;
-	    return 0;
-	}
+    // get interface
+    iface = this->getIfaceBySocket(sockid);
 
-	return this->decodeMsg(iface, peer, data, dataLen);
-    } 
-    return 0;
+    sock = iface->getSocketByFD(sockid);
+
+    Log(Debug) << "Received " << dataLen << " bytes on the " << iface->getName() << "/"
+               << iface->getID() << " interface (socket=" << sockid << ", addr="
+               << peer->getPlain() << ", port=" << sock->getPort() << ")." << LogEnd;
+
+    if (sock->getPort()!=DHCPSERVER_PORT) {
+        Log(Error) << "Message was received on invalid (" << sock->getPort() << ") port." << LogEnd;
+        return SPtr<TRelMsg>(); // NULL
+    }
+
+    return decodeMsg(iface, peer, data, dataLen);
 }
 
-SPtr<TRelMsg> TRelIfaceMgr::decodeRelayForw(SPtr<TIfaceIface> iface, 
-						SPtr<TIPv6Addr> peer, 
+SPtr<TRelMsg> TRelIfaceMgr::decodeRelayForw(SPtr<TIfaceIface> iface,
+						SPtr<TIPv6Addr> peer,
 						char * data, int dataLen) {
     int ifindex = iface->getID();
     SPtr<TRelMsg> msg = new TRelMsgRelayForw(ifindex, peer, data, dataLen);
     return msg;
 }
 
-SPtr<TRelMsg> TRelIfaceMgr::decodeGeneric(SPtr<TIfaceIface> iface, 
-					      SPtr<TIPv6Addr> peer, 
+SPtr<TRelMsg> TRelIfaceMgr::decodeGeneric(SPtr<TIfaceIface> iface,
+					      SPtr<TIPv6Addr> peer,
 					      char * buf, int bufsize) {
     int ifindex = iface->getID();
     return new TRelMsgGeneric(ifindex, peer, buf, bufsize);
@@ -159,7 +164,7 @@ SPtr<TRelMsg> TRelIfaceMgr::decodeRelayRepl(SPtr<TIfaceIface> iface,
     /* decode RELAY_FORW message */
     if (bufsize < 34) {
 	Log(Warning) << "Truncated RELAY_REPL message received." << LogEnd;
-	return 0;
+        return SPtr<TRelMsg>(); // NULL
     }
 
     // char type = buf[0]; // ignore it
@@ -177,7 +182,7 @@ SPtr<TRelMsg> TRelIfaceMgr::decodeRelayRepl(SPtr<TIfaceIface> iface,
 	if (len>bufsize) {
 	    Log(Error) << "Message RELAY-REPL truncated. There are " << (bufsize-len) 
 		       << " bytes left to parse. Message dropped." << LogEnd;
-	    return 0;
+            return SPtr<TRelMsg>(); // NULL
 	}
 	buf     += 4;
 	bufsize -= 4;
@@ -185,12 +190,12 @@ SPtr<TRelMsg> TRelIfaceMgr::decodeRelayRepl(SPtr<TIfaceIface> iface,
 	case OPTION_INTERFACE_ID:
 	    if (bufsize<4) {
 		Log(Warning) << "Truncated INTERFACE_ID option in RELAY_REPL message. Message dropped." << LogEnd;
-		return 0;
+                return SPtr<TRelMsg>(); // NULL
 	    }
 	    if (len!=4) {
 		Log(Warning) << "Invalid INTERFACE_ID option, expected length " << 4
 			     << ", actual length " << len << "." << LogEnd;
-		return 0;
+                return SPtr<TRelMsg>(); // NULL
 	    }
 
 	    ptrIfaceID = new TRelOptInterfaceID(buf, bufsize, 0);
@@ -225,7 +230,7 @@ SPtr<TRelMsg> TRelIfaceMgr::decodeRelayRepl(SPtr<TIfaceIface> iface,
 	if (!RelCfgMgr().guessMode()) {
 	    /* guessMode disabled */
 	    Log(Warning) << "InterfaceID option is missing, guessMode disabled, unable to forward. Packet dropped." << LogEnd;
-	    return 0;
+            return SPtr<TRelMsg>(); // NULL
 	}
 
 	/* guess mode enabled, let's find any interface */
@@ -238,7 +243,7 @@ SPtr<TRelMsg> TRelIfaceMgr::decodeRelayRepl(SPtr<TIfaceIface> iface,
 	}
 	if (!tmp) {
 	    Log(Error) << "Guess-mode failed. Unable to find any interface the message can be relayed on. Please send interface-id option in server replies." << LogEnd;
-	    return 0;
+            return SPtr<TRelMsg>(); // NULL
 	}
 	Log(Notice) << "Guess-mode successful. Using " << tmp->getFullName() << " as destination interface." << LogEnd;
 	ptrIfaceID = new TRelOptInterfaceID(tmp->getInterfaceID(), 0);
@@ -258,7 +263,7 @@ SPtr<TRelMsg> TRelIfaceMgr::decodeRelayRepl(SPtr<TIfaceIface> iface,
 	break;
     case RELAY_FORW_MSG:
 	Log(Error) << "RELAY_REPL contains RELAY_FORW message." << LogEnd;
-	return 0;
+        return SPtr<TRelMsg>(); // NULL
     default:
 	msg = this->decodeGeneric(iface, peer, relayBuf, relayLen);
     };
@@ -271,10 +276,14 @@ SPtr<TRelMsg> TRelIfaceMgr::decodeMsg(SPtr<TIfaceIface> iface,
 					  SPtr<TIPv6Addr> peer, 
 					  char * data, int dataLen) {
     if (dataLen <= 0) 
-	return 0;
+        return SPtr<TRelMsg>(); // NULL
     
     // create specific message object
     switch (data[0]) {
+    default: {
+	Log(Warning) << "Unknown message type " << (int)(data[0]) << ", relaying anyway." << LogEnd;
+        // no break here
+    }
     case SOLICIT_MSG:
     case REQUEST_MSG:
     case CONFIRM_MSG:
@@ -285,19 +294,17 @@ SPtr<TRelMsg> TRelIfaceMgr::decodeMsg(SPtr<TIfaceIface> iface,
     case INFORMATION_REQUEST_MSG:
     case ADVERTISE_MSG:
     case REPLY_MSG:
-	return this->decodeGeneric(iface, peer, data, dataLen);
+	return decodeGeneric(iface, peer, data, dataLen);
     case RELAY_FORW_MSG: 
-	return this->decodeRelayForw(iface, peer, data, dataLen);
+	return decodeRelayForw(iface, peer, data, dataLen);
     case RELAY_REPL_MSG:
-	return this->decodeRelayRepl(iface, peer, data, dataLen);
+	return decodeRelayRepl(iface, peer, data, dataLen);
     case RECONFIGURE_MSG:
-    default:
-	Log(Warning) << "Message type " << (int)(data[0]) << " is not supported." << LogEnd;
-	return 0;
+        return SPtr<TRelMsg>(); // NULL
     }
 }
 
-void TRelIfaceMgr::instanceCreate( const std::string xmlFile )
+void TRelIfaceMgr::instanceCreate(const std::string& xmlFile)
 {
     if (Instance)
       Log(Crit) << "RelIfaceMgr instance already created. Application error!" << LogEnd;
@@ -306,8 +313,10 @@ void TRelIfaceMgr::instanceCreate( const std::string xmlFile )
 
 TRelIfaceMgr& TRelIfaceMgr::instance()
 {
-    if (!Instance)
-      Log(Crit) << "RelIfaceMgr istance not created yet. Application error. Crashing in 3... 2... 1..." << LogEnd;
+    if (!Instance) {
+      Log(Crit) << "RelIfaceMgr istance not created yet. Application error. Emergency shutdown." << LogEnd;
+      exit(EXIT_FAILURE);
+    }
     return *Instance;
 }
 
