@@ -6,16 +6,6 @@
  *
  * released under GNU GPL v2 only licence
  *
- * $Id: RelTransMgr.h,v 1.3 2008-08-29 00:07:33 thomson Exp $
- *
- * $Log: not supported by cvs2svn $
- * Revision 1.2  2005-01-13 22:45:55  thomson
- * Relays implemented.
- *
- * Revision 1.1  2005/01/11 22:53:36  thomson
- * Relay skeleton implemented.
- *
- *
  */
 
 class TRelTransMgr;
@@ -31,9 +21,9 @@ class TRelTransMgr;
 
 class TRelTransMgr
 {
-    friend ostream & operator<<(ostream &strum, TRelTransMgr &x);
+    friend std::ostream & operator<<(std::ostream &strum, TRelTransMgr &x);
   public:
-    static void instanceCreate(const std::string xmlFile);
+    static void instanceCreate(const std::string& xmlFile);
     static TRelTransMgr& instance();
 
     ~TRelTransMgr();
@@ -48,16 +38,21 @@ class TRelTransMgr
 
     bool isDone();
     void shutdown();
-    
+
     char * getCtrlAddr();
     int    getCtrlIface();
-    
-  private:
-    TRelTransMgr(const std::string xmlFile);
-    static TRelTransMgr * Instance;
-    string XmlFile;
-    bool IsDone;
 
+protected:
+    TRelTransMgr(const std::string& xmlFile);
+    static TRelTransMgr * Instance;
+
+    SPtr<TOpt> getClientLinkLayerAddr(SPtr<TRelMsg> msg);
+    SPtr<TOpt> getLinkAddrFromSrcAddr(SPtr<TRelMsg> msg);
+    SPtr<TOpt> getLinkAddrFromDuid(SPtr<TOpt> duid_opt);
+
+  private:
+    std::string XmlFile;
+    bool IsDone;
     int ctrlIface;
     char ctrlAddr[48];
 };
@@ -65,5 +60,3 @@ class TRelTransMgr
 
 
 #endif
-
-
