@@ -5,10 +5,10 @@
  *
  * Released under GNU GPL v2 licence
  *
- * $Id: ReqTransMgr.cpp,v 1.10 2008-01-01 20:21:14 thomson Exp $
  */
 
 #include <stdio.h>
+#include <string.h>
 #include <sstream>
 #include "SocketIPv6.h"
 #include "ReqTransMgr.h"
@@ -19,15 +19,19 @@
 #include "Logger.h"
 #include "ReqOpt.h"
 #include "Portable.h"
+#include "hex.h"
+
+using namespace std;
 
 ReqTransMgr::ReqTransMgr(TIfaceMgr * ifaceMgr)
+    :CfgMgr(NULL)
 {
-    this->IfaceMgr = ifaceMgr;
+    IfaceMgr = ifaceMgr;
 }
 
 void ReqTransMgr::SetParams(ReqCfgMgr * cfgMgr)
 {
-    this->CfgMgr = cfgMgr;
+    CfgMgr = cfgMgr;
 }
 
 bool ReqTransMgr::BindSockets()
@@ -41,7 +45,6 @@ bool ReqTransMgr::BindSockets()
     if (!Iface) {
         Log(Crit) << "Unable to bind sockets: Interface " << CfgMgr->iface << " not found." << LogEnd;
         return false;
-        this->Iface = Iface;
     }
 
 #ifndef WIN32
@@ -139,7 +142,7 @@ bool ReqTransMgr::SendMsg()
         TReqOptAddr * optAddr = new TReqOptAddr(OPTION_IAADDR, a, msg);
         optAddr->storeSelf(buf+bufLen);
         bufLen += optAddr->getSize();
-        free(optAddr);
+        delete optAddr;
         
     } else {
         Log(Debug) << "Creating DUID-based query. Asking for " << CfgMgr->duid << " DUID." << LogEnd;
@@ -154,7 +157,7 @@ bool ReqTransMgr::SendMsg()
         optDuid->storeSelf(buf+bufLen);
         bufLen += optDuid->getSize();
 
-        free(optDuid);
+        delete optDuid;
     }
 
     SPtr<TDUID> clientDuid = new TDUID("00:01:00:01:0e:ec:13:db:00:02:02:02:02:02");
@@ -186,10 +189,11 @@ bool ReqTransMgr::WaitForRsp()
     int bufLen = 1024;
     memset(buf, 0, bufLen);
     SPtr<TIPv6Addr> sender = new TIPv6Addr();
+    SPtr<TIPv6Addr> myaddr(new TIPv6Addr());
 
     int sockFD;
     Log(Debug) << "Waiting " << CfgMgr->timeout << " seconds for reply reception." << LogEnd;
-    sockFD = this->IfaceMgr->select(CfgMgr->timeout, buf, bufLen, sender);
+    sockFD = this->IfaceMgr->select(CfgMgr->timeout, buf, bufLen, sender, myaddr);
     
     Log(Debug) << "Returned socketID=" << sockFD << LogEnd;
     if (sockFD>0) {
@@ -269,7 +273,7 @@ bool ReqTransMgr::ParseOpts(int msgType, int recurseLevel, char * buf, int bufLe
 	    
 	    char *Message = new char[length+10];
 	    memcpy(Message,buf+pos+2,length-2);
-	    sprintf(Message+length-2, "(%d)", st);
+	    sprintf(Message+length-2, "(%u)", st);
 	    o = string(Message);
 	    delete [] Message;
 	    break;
@@ -332,15 +336,6 @@ bool ReqTransMgr::ParseOpts(int msgType, int recurseLevel, char * buf, int bufLe
 
 string ReqTransMgr::BinToString(char * buf, int bufLen)
 {
-    std::ostringstream o;
-    o << setfill('0');
-    for (int i=0; i<bufLen; i++) {
-	o << setw(2) << hex << (unsigned int)buf[i];
-	if (i+1!=bufLen) {
-	    o << ":";
-	}
-    }
-
-    return o.str();
+    return (hexToText((uint8_t*)buf, bufLen, true));
 }
 
