@@ -14,14 +14,12 @@
 #include "OptEmpty.h" // rapid-commit option
 #include "SrvMsgReply.h"
 #include "SrvMsg.h"
-#include "SrvOptOptionRequest.h"
-#include "SrvOptStatusCode.h"
+#include "OptOptionRequest.h"
+#include "OptStatusCode.h"
 #include "SrvOptIAAddress.h"
 #include "SrvOptIA_NA.h"
 #include "SrvOptIA_PD.h"
 #include "SrvOptTA.h"
-#include "SrvOptServerIdentifier.h"
-#include "SrvOptTimeZone.h"
 #include "SrvOptFQDN.h"
 #include "AddrClient.h"
 #include "AddrIA.h"
@@ -29,17 +27,15 @@
 #include "IfaceMgr.h"
 #include "Logger.h"
 
+using namespace std;
+
 /**
  * this constructor is used to create REPLY message as a response for CONFIRM message
  *
- * @param ifaceMgr
- * @param transMgr
- * @param CfgMgr
- * @param AddrMgr
  * @param confirm
  */
 TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgConfirm> confirm)
-    :TSrvMsg(confirm->getIface(),confirm->getAddr(), REPLY_MSG,
+    :TSrvMsg(confirm->getIface(),confirm->getRemoteAddr(), REPLY_MSG,
              confirm->getTransID())
 {
     getORO( (Ptr*)confirm );
@@ -48,34 +44,34 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgConfirm> confirm)
     copyAAASPI((Ptr*)confirm);
     copyRemoteID((Ptr*)confirm);
 
-    handleConfirmOptions( confirm->getOptLst() );
+    if (!handleConfirmOptions( confirm->getOptLst() )) {
+        IsDone = true;
+        return;
+    }
 
     appendMandatoryOptions(ORO);
-    // appendRequestedOptions(ClientDUID, confirm->getAddr(), confirm->getIface(), ORO);
-    // appendStatusCode();
     appendAuthenticationOption(ClientDUID);
 
-    pkt = new char[this->getSize()];
-    this->MRT = 31;
+    this->MRT_ = 31;
     IsDone = false;
-    this->send();
 }
 
+
 bool TSrvMsgReply::handleConfirmOptions(TOptList & options) {
-    SPtr<TSrvCfgIface> ptrIface = SrvCfgMgr().getIfaceByID( Iface );
-    if (!ptrIface) {
+
+    SPtr<TSrvCfgIface> cfgIface = SrvCfgMgr().getIfaceByID(Iface);
+    if (!cfgIface) {
         Log(Crit) << "Msg received through not configured interface. "
             "Somebody call an exorcist!" << LogEnd;
-        this->IsDone = true;
+        IsDone = true;
         return false;
     }
 
-    bool OnLink = true;
+    EAddrStatus onLink = ADDRSTATUS_YES;
     int checkCnt = 0;
-    List(TSrvOptIA_NA) validIAs;
 
     TOptList::iterator opt = options.begin();
-    while ( (opt!=options.end()) && OnLink ) {
+    while ( (opt!=options.end()) && (onLink==ADDRSTATUS_YES) ) {
 
         switch ( (*opt)->getOptType()) {
         case OPTION_IA_NA: {
@@ -83,103 +79,102 @@ bool TSrvMsgReply::handleConfirmOptions(TOptList & options) {
             SPtr<TSrvOptIA_NA> ia = (Ptr*) (*opt);
 
             // now we check whether this IA exists in Server Address database or not.
-
-            SPtr<TOpt> subOpt;
-            unsigned long addrCnt = 0;
+            SPtr<TOpt> opt;
             ia->firstOption();
-            while ( (subOpt = ia->getOption()) && (OnLink) ) {
-                if (subOpt->getOptType() != OPTION_IAADDR){
+            while ((opt = ia->getOption()) && (onLink == ADDRSTATUS_YES) ) {
+                if (opt->getOptType() != OPTION_IAADDR){
                     continue;
                 }
 
-                SPtr<TSrvOptIAAddress> optAddr = (Ptr*) subOpt;
-                Log(Debug) << "CONFIRM message: checking if " << optAddr->getAddr()->getPlain() << " is supported:";
-                if (!SrvCfgMgr().isIAAddrSupported(this->Iface, optAddr->getAddr())) {
-                    Log(Cont) << "no." << LogEnd;
-                    OnLink = false;
-                } else {
-                    Log(Cont) << "yes." << LogEnd;
-                    addrCnt++;
-                }
+                SPtr<TSrvOptIAAddress> optAddr = (Ptr*) opt;
+                onLink = cfgIface->confirmAddress(IATYPE_IA, optAddr->getAddr());
                 checkCnt++;
             }
-            if (addrCnt) {
-                SPtr<TSrvOptIA_NA> tempIA = new TSrvOptIA_NA(ia, PeerAddr,
-                                                             ClientDUID, Iface, addrCnt,CONFIRM_MSG, this);
-                validIAs.append(tempIA);
-            }
             break;
         }
         case OPTION_IA_TA: {
             SPtr<TSrvOptTA> ta = (Ptr*) (*opt);
-            // now we check whether this IA exists in Server Address database or not.
 
-            SPtr<TOpt> subOpt;
+            SPtr<TOpt> opt;
             ta->firstOption();
-            while (subOpt = ta->getOption() && (OnLink)) {
-                if (subOpt->getOptType() != OPTION_IAADDR)
+            while ((opt = ta->getOption()) && (onLink == ADDRSTATUS_YES)) {
+                if (opt->getOptType() != OPTION_IAADDR)
                     continue;
-                SPtr<TSrvOptIAAddress> optAddr = (Ptr*) subOpt;
-                if (!SrvCfgMgr().isTAAddrSupported(this->Iface, optAddr->getAddr())) {
-                    OnLink = false;
-                }
+
+                SPtr<TSrvOptIAAddress> optAddr = (Ptr*) opt;
+                onLink = cfgIface->confirmAddress(IATYPE_TA, optAddr->getAddr());
+                checkCnt++;
+            }
+            break;
+        }
+        case OPTION_IA_PD: {
+            SPtr<TSrvOptIA_PD> ta = (Ptr*) (*opt);
+
+            SPtr<TOpt> opt;
+            ta->firstOption();
+            while ((opt = ta->getOption()) && (onLink == ADDRSTATUS_YES)) {
+                if (opt->getOptType() != OPTION_IAPREFIX)
+                    continue;
+
+                SPtr<TSrvOptIAPrefix> optPrefix = (Ptr*) opt;
+                onLink = cfgIface->confirmAddress(IATYPE_PD, optPrefix->getPrefix());
                 checkCnt++;
             }
             break;
         }
         default:
-            handleDefaultOption( *opt);
+            handleDefaultOption(*opt);
             break;
         }
         ++opt;
     }
-    if (!checkCnt) {
-        // no check
-        SPtr <TSrvOptStatusCode> ptrCode =
-            new TSrvOptStatusCode(STATUSCODE_NOTONLINK,
-                                  "No addresses checked. Did you send any?",
-                                  this);
-        Options.push_back( (Ptr*) ptrCode );
-    } else
-    if (!OnLink) {
-        // not-on-link
-        SPtr <TSrvOptStatusCode> ptrCode =
-            new TSrvOptStatusCode(STATUSCODE_NOTONLINK,
-                                  "Sorry, those addresses are not valid for this link.",
-                                  this);
-        Options.push_back( (Ptr*) ptrCode );
-    } else {
-        // success
-        SPtr <TSrvOptStatusCode> ptrCode =
-            new TSrvOptStatusCode(STATUSCODE_SUCCESS,
-                                  "Your addresses are correct for this link! Yay!",
-                                  this);
-        Options.push_back( (Ptr*) ptrCode);
 
-        if(validIAs.count()){
-            SPtr<TSrvOptIA_NA> ia;
-            validIAs.first();
-            while(ia=validIAs.get() ){
-                Options.push_back((Ptr*)ia );
-            }
-        }
+    if (!checkCnt) {
+        Log(Info) << "No addresses or prefixes in CONFIRM. Not sending reply." << LogEnd;
+        return false;
     }
 
-    return true;
+    switch (onLink) {
+    case ADDRSTATUS_YES: {
+        SPtr <TOptStatusCode> ptrCode =
+            new TOptStatusCode(STATUSCODE_SUCCESS,
+                               "Your addresses are correct for this link! Yay!",
+                               this);
+        Options.push_back( (Ptr*) ptrCode);
+        return true;
+    }
+
+    case ADDRSTATUS_NO: {
+        SPtr <TOptStatusCode> ptrCode =
+            new TOptStatusCode(STATUSCODE_NOTONLINK,
+                               "Sorry, those addresses are not valid for this link.",
+                               this);
+        Options.push_back( (Ptr*) ptrCode );
+        return true;
+    }
+
+    default:
+    case ADDRSTATUS_UNKNOWN: {
+        Log(Info) << "Address/prefix being confirmed is outside of defined class,"
+                  << " but there is no subnet defined, so can't answer authoritatively."
+                  << " Will not send answer." << LogEnd;
+        return false;
+    }
+    }
+
+    // should never get here
+    return false;
 }
 
 
 /*
  * this constructor is used to create REPLY message as a response for DECLINE message
  *
- * @param ifaceMgr
- * @param transMgr
- * @param CfgMgr
- * @param AddrMgr
  * @param decline
  */
 TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgDecline> decline)
-    :TSrvMsg(decline->getIface(),decline->getAddr(), REPLY_MSG, decline->getTransID())
+    :TSrvMsg(decline->getIface(), decline->getRemoteAddr(), REPLY_MSG,
+             decline->getTransID())
 {
     getORO( (Ptr*)decline );
     copyClientID( (Ptr*)decline );
@@ -187,12 +182,12 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgDecline> decline)
     copyAAASPI( (Ptr*)decline );
     copyRemoteID( (Ptr*)decline );
 
-
     SPtr<TOpt> ptrOpt;
 
     SPtr<TAddrClient> ptrClient = SrvAddrMgr().getClient(ClientDUID);
     if (!ptrClient) {
-        Log(Warning) << "Received DECLINE from unknown client, DUID=" << *ClientDUID << ". Ignored." << LogEnd;
+        Log(Warning) << "Received DECLINE from unknown client, DUID="
+                     << *ClientDUID << ". Ignored." << LogEnd;
         IsDone = true;
         return;
     }
@@ -261,13 +256,14 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgDecline> decline)
             char buf[10];
             sprintf(buf,"%d",AddrsDeclinedCnt);
             string tmp = buf;
-            SPtr<TSrvOptStatusCode> optStatusCode =
-                new TSrvOptStatusCode(STATUSCODE_SUCCESS,tmp+" addrs declined.",this);
+            SPtr<TOptStatusCode> optStatusCode =
+                new TOptStatusCode(STATUSCODE_SUCCESS, tmp + " addrs declined.", this);
             replyIA_NA->addOption( (Ptr*) optStatusCode );
             break;
         };
         case OPTION_IA_TA:
-            Log(Info) << "TA address declined. Oh well. Since it's temporary, let's ignore it entirely." << LogEnd;
+            Log(Info) << "TA address declined. It's temporary, so let's ignore it entirely."
+                      << LogEnd;
             break;
         default:
             handleDefaultOption(ptrOpt);
@@ -279,23 +275,17 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgDecline> decline)
 
     appendAuthenticationOption(ClientDUID);
 
-    pkt = new char[this->getSize()];
     IsDone = false;
-    this->MRT = 31;
-    this->send();
+    MRT_ = 31;
 }
 
 /**
  * this constructor is used to create REPLY message as a response for REBIND message
  *
- * @param ifaceMgr
- * @param transMgr
- * @param CfgMgr
- * @param AddrMgr
  * @param rebind
  */
 TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRebind> rebind)
-    :TSrvMsg(rebind->getIface(),rebind->getAddr(), REPLY_MSG, rebind->getTransID())
+    :TSrvMsg(rebind->getIface(),rebind->getRemoteAddr(), REPLY_MSG, rebind->getTransID())
 {
     getORO( (Ptr*)rebind );
     copyClientID( (Ptr*)rebind );
@@ -315,7 +305,7 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRebind> rebind)
           {
             SPtr<TSrvOptIA_NA> optIA_NA;
             optIA_NA = new TSrvOptIA_NA((Ptr*)ptrOpt,
-                                        rebind->getAddr(), ClientDUID,
+                                        rebind->getRemoteAddr(), ClientDUID,
                                         rebind->getIface(), addrCount, REBIND_MSG,
                                         this);
             if (optIA_NA->getStatusCode() != STATUSCODE_NOBINDING )
@@ -330,8 +320,7 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRebind> rebind)
           }
         case OPTION_IA_PD: {
             SPtr<TSrvOptIA_PD> pd;
-            pd = new TSrvOptIA_PD( (Ptr*) ptrOpt, rebind->getAddr(),
-                                   ClientDUID, rebind->getIface(), REQUEST_MSG, this);
+            pd = new TSrvOptIA_PD( (Ptr*)rebind, (Ptr*) ptrOpt, this);
             Options.push_back((Ptr*)pd);
             break;
         }
@@ -350,27 +339,20 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRebind> rebind)
     }
 
     appendMandatoryOptions(ORO);
-    appendRequestedOptions(ClientDUID, rebind->getAddr(),rebind->getIface(), ORO);
+    appendRequestedOptions(ClientDUID, rebind->getRemoteAddr(),rebind->getIface(), ORO);
     appendAuthenticationOption(ClientDUID);
 
-    pkt = new char[this->getSize()];
     IsDone = false;
-    this->MRT = 0;
-    this->send();
-
+    MRT_ = 0;
 }
 
 /**
  * this constructor is used to create REPLY message as a response for RELEASE message
  *
- * @param ifaceMgr
- * @param transMgr
- * @param CfgMgr
- * @param AddrMgr
  * @param release
  */
 TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRelease> release)
-    :TSrvMsg(release->getIface(),release->getAddr(), REPLY_MSG,
+    :TSrvMsg(release->getIface(),release->getRemoteAddr(), REPLY_MSG,
              release->getTransID())
 {
     getORO( (Ptr*) release );
@@ -393,7 +375,7 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRelease> release)
 
     SPtr<TAddrClient> client = SrvAddrMgr().getClient(ClientDUID);
     if (!client) {
-        Log(Warning) << "Received RELEASE from unknown client." << LogEnd;
+        Log(Warning) << "Received RELEASE from unknown client DUID=" << ClientDUID->getPlain() << LogEnd;
         IsDone = true;
         return;
     }
@@ -428,7 +410,7 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRelease> release)
             // if there was DNS Update performed, execute deleting Update
             SPtr<TFQDN> fqdn = ptrIA->getFQDN();
             if (fqdn) {
-                this->fqdnRelease(ptrIface, ptrIA, fqdn);
+                delFQDN(ptrIface, ptrIA, fqdn);
             }
 
             // let's verify each address
@@ -453,7 +435,7 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRelease> release)
             {
                 SPtr<TSrvOptIA_NA> ansIA(new TSrvOptIA_NA(clntIA->getIAID(), clntIA->getT1(),clntIA->getT2(),this));
                 Options.push_back((Ptr*)ansIA);
-                ansIA->addOption(new TSrvOptStatusCode(STATUSCODE_NOBINDING, "Not every address had binding.",this));
+                ansIA->addOption(new TOptStatusCode(STATUSCODE_NOBINDING, "Not every address had binding.",this));
             };
             break;
         }
@@ -525,9 +507,8 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRelease> release)
             // send result to the client
             if (!anyDeleted)
             {
-                SPtr<TSrvOptIA_PD> ansPD(new TSrvOptIA_PD(pd->getIAID(), 0, 0, this));
-                Options.push_back((Ptr*)ansPD);
-                ansPD->addOption(new TSrvOptStatusCode(STATUSCODE_NOBINDING, "Not every address had binding.",this));
+                Options.push_back(new TSrvOptIA_PD(pd->getIAID(), 0u, 0u, 
+                                                   STATUSCODE_NOBINDING, "Not every address had binding.", this));
             };
             break;
         }
@@ -537,20 +518,18 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRelease> release)
         }; // switch(...)
     } // while
 
-    Options.push_back(new TSrvOptStatusCode(STATUSCODE_SUCCESS,
+    Options.push_back(new TOptStatusCode(STATUSCODE_SUCCESS,
                                          "All IAs in RELEASE message were processed.",this));
 
     NotifyScripts = notifyParams;
 
-    pkt = new char[this->getSize()];
     IsDone = false;
-    this->MRT=46;
-    this->send();
+    MRT_ = 46;
 }
 
 // used as RENEW reply
 TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRenew> renew)
-    :TSrvMsg(renew->getIface(),renew->getAddr(), REPLY_MSG, renew->getTransID())
+    :TSrvMsg(renew->getIface(),renew->getRemoteAddr(), REPLY_MSG, renew->getTransID())
 {
     getORO( (Ptr*)renew );
     copyClientID( (Ptr*)renew );
@@ -558,11 +537,6 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRenew> renew)
     copyAAASPI((Ptr*)renew);
     copyRemoteID((Ptr*)renew);
 
-    // uncomment this to test REBIND
-    //IsDone = true;
-    //return;
-    // uncomment this to test REBIND
-
     unsigned long addrCount=0;
     SPtr<TOpt> ptrOpt;
 
@@ -574,16 +548,14 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRenew> renew)
         case OPTION_IA_NA: {
             SPtr<TSrvOptIA_NA> optIA_NA;
             optIA_NA = new TSrvOptIA_NA((Ptr*)ptrOpt,
-                                        renew->getAddr(), ClientDUID,
+                                        renew->getRemoteAddr(), ClientDUID,
                                         renew->getIface(), addrCount, RENEW_MSG, this);
             Options.push_back((Ptr*)optIA_NA);
             break;
         }
         case OPTION_IA_PD: {
             SPtr<TSrvOptIA_PD> optPD;
-            optPD = new TSrvOptIA_PD( (Ptr*)ptrOpt,
-                                     renew->getAddr(), ClientDUID,
-                                     renew->getIface(), RENEW_MSG, this);
+            optPD = new TSrvOptIA_PD((Ptr*) renew, (Ptr*)ptrOpt, this);
             Options.push_back( (Ptr*) optPD);
             break;
         }
@@ -609,13 +581,11 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRenew> renew)
     }
 
     appendMandatoryOptions(ORO);
-    appendRequestedOptions(ClientDUID,renew->getAddr(),renew->getIface(), ORO);
+    appendRequestedOptions(ClientDUID,renew->getRemoteAddr(),renew->getIface(), ORO);
     appendAuthenticationOption(ClientDUID);
 
-    pkt = new char[this->getSize()];
     IsDone = false;
-    this->MRT = 0;
-    this->send();
+    MRT_ = 0;
 }
 
 /**
@@ -624,7 +594,7 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRenew> renew)
  * @param request
  */
 TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRequest> request)
-    :TSrvMsg(request->getIface(), request->getAddr(), REPLY_MSG, request->getTransID())
+    :TSrvMsg(request->getIface(), request->getRemoteAddr(), REPLY_MSG, request->getTransID())
 {
     getORO( (Ptr*)request );
     copyClientID( (Ptr*)request );
@@ -632,97 +602,28 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgRequest> request)
     copyAAASPI( (Ptr*)request );
     copyRemoteID( (Ptr*)request );
 
-    /// @todo move this to a common message handling place
-    // is this client supported?
-    if (!SrvCfgMgr().isClntSupported(ClientDUID, request->getAddr(), Iface)) {
-        //No reply for this client
-        Log(Notice) << "Message from client (DUID=" << ClientDUID->getPlain() << ",addr="
-                    << request->getAddr()->getPlain()
-                    << ") was rejected due to accept-only or reject-client." << LogEnd;
-        IsDone=true;
-        return;
-    }
-
-    SPtr<TIPv6Addr> clntAddr = request->getAddr();
-    unsigned int clntIface = request->getIface();
-
-    // try to provide answer for all options
-    SPtr<TOpt> opt;
-    request->firstOption();
-    while ( opt = request->getOption() ) {
-
-        switch (opt->getOptType()) {
-        case OPTION_IA_NA: {
-            SPtr<TSrvOptIA_NA> optIA;
-            optIA = new TSrvOptIA_NA( (Ptr*) opt, ClientDUID, clntAddr, clntIface, REQUEST_MSG, this);
-            Options.push_back((Ptr*)optIA);
-            break;
-        }
-        case OPTION_IA_TA: {
-            SPtr<TSrvOptTA> ta;
-            ta = new TSrvOptTA( (Ptr*) opt, ClientDUID, clntAddr, clntIface, REQUEST_MSG, this);
-            Options.push_back((Ptr*)ta);
-            break;
-        }
-
-        case OPTION_IA_PD: {
-            SPtr<TSrvOptIA_PD> pd;
-            pd = new TSrvOptIA_PD( (Ptr*) opt, clntAddr, ClientDUID, clntIface, REQUEST_MSG, this);
-            Options.push_back((Ptr*)pd);
-            break;
-        }
-        case OPTION_STATUS_CODE :
-        case OPTION_RELAY_MSG   :
-        case OPTION_PREFERENCE  :
-        case OPTION_RECONF_MSG  :
-        case OPTION_INTERFACE_ID:
-        case OPTION_IAADDR      :
-        case OPTION_UNICAST     :
-        case OPTION_RAPID_COMMIT:
-            Log(Warning) << "Invalid option " << opt->getOptType()<<" received." << LogEnd;
-            break;
-        case OPTION_FQDN : {
-            SPtr<TSrvOptFQDN> requestFQDN = (Ptr*) opt;
-            SPtr<TOptFQDN> anotherFQDN = (Ptr*) opt;
-            SPtr<TSrvOptFQDN> optFQDN;
-
-            string hint = anotherFQDN->getFQDN();
-
-            SPtr<TIPv6Addr> clntAssignedAddr = SrvAddrMgr().getFirstAddr(ClientDUID);
-            if (clntAssignedAddr)
-                optFQDN = this->prepareFQDN(requestFQDN, ClientDUID, clntAssignedAddr, hint, true);
-            else
-                optFQDN = this->prepareFQDN(requestFQDN, ClientDUID, clntAddr, hint, true);
-            if (optFQDN) {
-                Options.push_back((Ptr*) optFQDN);
-            }
-            break;
-        }
-        case OPTION_VENDOR_OPTS:
-        {
-            SPtr<TOptVendorSpecInfo> v = (Ptr*) opt;
-            appendVendorSpec(ClientDUID, Iface, v->getVendor(), ORO);
-            break;
-        }
-        default:
-            handleDefaultOption(opt);
-            break;
-        }
-    }
+    processOptions((Ptr*)request, false); // be verbose
 
     appendMandatoryOptions(ORO);
-    appendRequestedOptions(ClientDUID, clntAddr, clntIface, ORO);
+    appendRequestedOptions(ClientDUID, PeerAddr_, Iface, ORO);
     appendAuthenticationOption(ClientDUID);
 
-    pkt = new char[this->getSize()];
+#ifndef MOD_DISABLE_AUTH
+    SPtr<TOpt> reconfAccept = request->getOption(OPTION_RECONF_ACCEPT);
+    if (reconfAccept) {
+        appendReconfigureKey();
+    }
+#endif
+
     IsDone = false;
-    this->MRT = 330;
-    this->send();
+    MRT_ = 330;
 }
 
-//SOLICIT
+/// @brief ctor used for generating REPLY message as SOLICIT response (in rapid-commit mode)
+///
+/// @param solicit client's message
 TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgSolicit> solicit)
-    :TSrvMsg(solicit->getIface(), solicit->getAddr(), REPLY_MSG, solicit->getTransID())
+    :TSrvMsg(solicit->getIface(), solicit->getRemoteAddr(), REPLY_MSG, solicit->getTransID())
 {
     getORO( (Ptr*)solicit );
     copyClientID( (Ptr*)solicit );
@@ -730,81 +631,23 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgSolicit> solicit)
     copyAAASPI( (Ptr*)solicit );
     copyRemoteID( (Ptr*)solicit );
 
-    /// @todo remove this and use Iface and PeerAddr
-    SPtr<TIPv6Addr> clntAddr = solicit->getAddr();
-    unsigned int clntIface = solicit->getIface();
+    processOptions((Ptr*) solicit, false);
 
     // append RAPID-COMMIT option
     Options.push_back(new TOptEmpty(OPTION_RAPID_COMMIT, this));
 
-    /// @todo move this to a common message handling place
-    // is this client supported?
-    if (!SrvCfgMgr().isClntSupported(ClientDUID, clntAddr, clntIface)) {
-        //No reply for this client
-        Log(Notice) << "Client (DUID=" << ClientDUID->getPlain() << ",addr=" << *clntAddr
-                    << ") was rejected due to accept-only or reject-client." << LogEnd;
-        IsDone=true;
-        return;
-    }
-
-    //So if we can do something for this client at least set configuration
-    //parameters - let's do  option by option - try to answer to it
-    SPtr<TOpt> opt;
-    solicit->firstOption();
-    while ( opt = solicit->getOption() )
-    {
-        switch (opt->getOptType())
-        {
-        case OPTION_SERVERID    :
-            Log(Warning) << "Solicit message contains ServerID option. Rejecting message. " << LogEnd;
-            IsDone = true;
-            return;
-        case OPTION_IA_NA       : {
-            SPtr<TSrvOptIA_NA> optIA_NA;
-            optIA_NA = new TSrvOptIA_NA((Ptr*) opt, ClientDUID, clntAddr, clntIface, REQUEST_MSG,this);
-            Options.push_back((Ptr*)optIA_NA);
-            break;
-        }
-        case OPTION_IA_TA: {
-            SPtr<TSrvOptTA> ta;
-            ta = new TSrvOptTA( (Ptr*) opt, ClientDUID, clntAddr,
-                               clntIface, REQUEST_MSG, this);
-            Options.push_back((Ptr*)ta);
-            break;
-        }
-        case OPTION_STATUS_CODE :
-        case OPTION_RELAY_MSG   :
-        case OPTION_PREFERENCE  :
-        case OPTION_RECONF_MSG  :
-        case OPTION_INTERFACE_ID:
-        case OPTION_IAADDR      :
-        case OPTION_UNICAST     :
-            Log(Warning) << "Invalid option type("
-                         <<opt->getOptType()<<") received." << LogEnd;
-            break;
-        case OPTION_RAPID_COMMIT:
-            Log(Info) << "SOLICIT with RAPID-COMMIT received." << LogEnd;
-            break;
-        default:
-            handleDefaultOption(opt);
-            break;
-        }
-    }
     appendMandatoryOptions(ORO);
-    appendRequestedOptions(ClientDUID, clntAddr, clntIface, ORO);
+    appendRequestedOptions(ClientDUID, PeerAddr_, Iface, ORO);
     appendAuthenticationOption(ClientDUID);
 
-    pkt = new char[this->getSize()];
     IsDone = false;
-    SPtr<TIPv6Addr> ptrAddr;
-    this->MRT = 330;
-    this->send();
+    MRT_ = 330;
 }
 
 // INFORMATION-REQUEST answer
 TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgInfRequest> infRequest)
     :TSrvMsg(infRequest->getIface(),
-             infRequest->getAddr(),REPLY_MSG,infRequest->getTransID())
+             infRequest->getRemoteAddr(),REPLY_MSG,infRequest->getTransID())
 {
     getORO( (Ptr*)infRequest );
     copyClientID( (Ptr*)infRequest );
@@ -831,8 +674,6 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgInfRequest> infRequest)
             case OPTION_RECONF_MSG  :
             case OPTION_IA_NA       :
             case OPTION_IA_TA       :
-            case OPTION_AAAAUTH     :
-            case OPTION_KEYGEN      :
                 Log(Warning) << "Invalid option " << ptrOpt->getOptType() <<" received." << LogEnd;
                 break;
             default:
@@ -842,7 +683,8 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgInfRequest> infRequest)
     }
 
     appendMandatoryOptions(ORO);
-    if ( !appendRequestedOptions(ClientDUID, infRequest->getAddr(),infRequest->getIface(), ORO) ) {
+    if ( !appendRequestedOptions(ClientDUID, infRequest->getRemoteAddr(),
+                                 infRequest->getIface(), ORO) ) {
         Log(Warning) << "No options to answer in INF-REQUEST, so REPLY will not be send." << LogEnd;
         IsDone=true;
         return;
@@ -850,10 +692,30 @@ TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsgInfRequest> infRequest)
 
     appendAuthenticationOption(ClientDUID);
 
-    pkt = new char[this->getSize()];
     IsDone = false;
-    MRT = 330; /// @todo:
-    send();
+    MRT_ = 330;
+}
+
+TSrvMsgReply::TSrvMsgReply(SPtr<TSrvMsg> msg, TOptList& options)
+    :TSrvMsg(msg->getIface(), msg->getRemoteAddr(), REPLY_MSG,
+             msg->getTransID()) {
+
+    // Let's just use specified options as they are
+    Options = options;
+
+    // Append client-id
+    SPtr<TOpt> client_id = msg->getOption(OPTION_CLIENTID);
+    if (client_id) {
+        Options.push_back(client_id);
+    }
+
+    // Append server-id
+    SPtr<TOptDUID> ptrSrvID;
+    ptrSrvID = new TOptDUID(OPTION_SERVERID, SrvCfgMgr().getDUID(), this);
+    Options.push_back((Ptr*)ptrSrvID);
+
+    MRT_ = 330;
+    IsDone = false;
 }
 
 void TSrvMsgReply::doDuties() {
@@ -861,10 +723,10 @@ void TSrvMsgReply::doDuties() {
 }
 
 unsigned long TSrvMsgReply::getTimeout() {
-    unsigned long diff = now() - this->FirstTimeStamp;
-    if (diff>SERVER_REPLY_CACHE_TIMEOUT)
+    unsigned long diff = (uint32_t)time(NULL) - FirstTimeStamp_;
+    if (diff > SERVER_REPLY_CACHE_TIMEOUT)
         return 0;
-    return SERVER_REPLY_CACHE_TIMEOUT-diff;
+    return SERVER_REPLY_CACHE_TIMEOUT - diff;
 }
 
 bool TSrvMsgReply::check() {
@@ -878,6 +740,6 @@ TSrvMsgReply::~TSrvMsgReply()
 
 }
 
-string TSrvMsgReply::getName() {
+string TSrvMsgReply::getName() const {
     return "REPLY";
 }
