@@ -6,8 +6,6 @@
  * changes: Krzysztof Wnuk <keczi@poczta.onet.pl>
  * released under GNU GPL v2 only licence
  *
- * $Id: SrvMsgReply.h,v 1.12 2009-03-24 01:16:53 thomson Exp $
- *
  */
 
 class TSrvMsgReply;
@@ -23,8 +21,9 @@ class TSrvMsgReply;
 #include "SrvMsgRelease.h"
 #include "SrvMsgSolicit.h"
 #include "SrvMsgInfRequest.h"
+#include "SrvOptIAAddress.h"
 
-#include "SrvOptClientIdentifier.h"
+#include "OptDUID.h"
 
 class TSrvMsgReply : public TSrvMsg
 {
@@ -37,11 +36,12 @@ class TSrvMsgReply : public TSrvMsg
     TSrvMsgReply(SPtr<TSrvMsgRequest> question);
     TSrvMsgReply(SPtr<TSrvMsgSolicit> question);
     TSrvMsgReply(SPtr<TSrvMsgInfRequest> question);
+    TSrvMsgReply(SPtr<TSrvMsg> msg, TOptList& options);
 
     void doDuties();
     unsigned long getTimeout();
     bool check();
-    string getName();
+    std::string getName() const;
     
     ~TSrvMsgReply();
 private:
@@ -54,6 +54,7 @@ private:
     bool handleDeclineOptions(TOptList& options);
     bool handleConfirmOptions(TOptList& options);
     bool handleInfRequestOptions(TOptList& options);
+    EAddrStatus confirmAddress(TIAType type, SPtr<TIPv6Addr> addr);
 };
 
 
