@@ -15,7 +15,6 @@
 #include "DHCPConst.h"
 #include "SrvIfaceMgr.h"
 #include "SrvMsgAdvertise.h"
-#include "SrvOptServerIdentifier.h"
 #include "SrvOptIA_NA.h"
 #include "AddrClient.h"
 
@@ -39,6 +38,6 @@ TSrvMsgRequest::~TSrvMsgRequest() {
 
 }
 
-string TSrvMsgRequest::getName() {
+std::string TSrvMsgRequest::getName() const {
     return "REQUEST";
 }
