@@ -19,14 +19,14 @@
 
 TSrvMsgSolicit::TSrvMsgSolicit(int iface, SPtr<TIPv6Addr> addr,
 			       char* buf, int bufSize)
-    :TSrvMsg(iface,addr,buf,bufSize) {
+    :TSrvMsg(iface, addr, buf, bufSize) {
 }
 
 void TSrvMsgSolicit::doDuties() {
     // this function should not be called on the server side
 }
 
-string TSrvMsgSolicit::getName() {
+std::string TSrvMsgSolicit::getName() const {
     return "SOLICIT";
 }
 
